<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:output 
    method="xml"
    indent="yes"
    doctype-public="-//treshna.com//DTD Bond V2.0//EN" 
    doctype-system="../bond-2.0.dtd" />
  <xsl:strip-space elements="*" />

  <xsl:template match="glade-interface">
    <bond>
      <project defaultvisible="false" />
      <xsl:apply-templates />
    </bond>
  </xsl:template>
  
  <xsl:template match="requires" />

  <xsl:template match="packing">
    <xsl:apply-templates />
  </xsl:template>

  <!-- A hackish way of parsing the class names of a glade XML file into
       Bond2 markup language; should be using variables. -->
  <xsl:template match="widget">
    <xsl:choose>
      <xsl:when test="@class='GtkWindow'">
        <window>
          <xsl:apply-templates />
        </window>
      </xsl:when>
      <xsl:when test="@class='GtkTable'">
        <box>
          <xsl:apply-templates />
        </box>
      </xsl:when>
      <xsl:when test="@class='GtkToolbar'">
        <toolbar>
          <xsl:apply-templates />
        </toolbar>
      </xsl:when>
      <xsl:when test="@class='GtkButton'">
        <button>
          <xsl:apply-templates />
          <trigger event="click" />
        </button>
      </xsl:when>
      <xsl:when test="@class='GtkSpinButton'">
        <spin>
          <xsl:apply-templates />
        </spin>
      </xsl:when>
      <xsl:when test="@class='GtkRadioButton'">
        <radio>
          <xsl:apply-templates />
        </radio>
      </xsl:when>
      <xsl:when test="@class='GtkCheckButton'">
        <check>
          <xsl:apply-templates />
        </check>
      </xsl:when>
      <xsl:when test="@class='GtkFrame'">
        <frame label="{child/widget[@class='GtkLabel']/property[@name='label']}">
          <xsl:apply-templates />
        </frame>
      </xsl:when>
      <xsl:when test="@class='GtkNotebook'">
        <notebook>
          <xsl:apply-templates />
        </notebook>
      </xsl:when>
      <xsl:when test="@class='GtkVBox'">
        <vbox>
          <xsl:apply-templates />
        </vbox>
      </xsl:when>
      <xsl:when test="@class='GtkHBox'">
        <hbox>
          <xsl:apply-templates />
        </hbox>
      </xsl:when>
      <xsl:when test="@class='GtkHButtonBox'">
        <hbox>
          <xsl:apply-templates />
        </hbox>
      </xsl:when>
      <xsl:when test="@class='GtkVButtonBox'">
        <vbox>
          <xsl:apply-templates />
        </vbox>
      </xsl:when>
      
      <!-- GtkEntry doesn't seem to be supported in bond??? -->
      <xsl:when test="@class='GtkEntry' and not(../@internal-child='entry')">
        <entry>
          <xsl:apply-templates />
        </entry>
      </xsl:when>
      <xsl:when test="@class='GtkMenuBar'">
        <menubar>
          <xsl:apply-templates />
        </menubar>
      </xsl:when>
      <xsl:when test="@class='GtkMenuItem' and child/widget/@class='GtkMenu'">
        <menu>
          <xsl:apply-templates />
        </menu>
      </xsl:when>
      <xsl:when test="@class='GtkMenuItem'">
        <menuitem>
          <xsl:apply-templates />
        </menuitem>
      </xsl:when>
      <xsl:when test="@class='GtkImageMenuItem'">
        <menuitem>
          <xsl:apply-templates />
        </menuitem>
      </xsl:when>
      <xsl:when test="@class='GtkHandleBox'">
        <handlebox>
          <xsl:apply-templates />
        </handlebox>
      </xsl:when>
      <xsl:when test="@class='GtkLabel'">
        <label>
          <xsl:value-of select="property[@name='label']" />
        </label>
      </xsl:when>
      <xsl:when test="@class='GtkImage'">
        <image>
          <xsl:apply-templates />
        </image>
      </xsl:when>
      <xsl:when test="@class='GtkCombo'">
        <dropdown>
          <xsl:if test="child[@internal-child='list']">
            <populatelist>
              <xsl:for-each select="child[@internal-child='list']/widget[@class='GtkList']/child/widget[@class='GtkListItem']">
                <listitem>
                  <xsl:value-of select="property[@name='label']" />
                </listitem>
              </xsl:for-each>
            </populatelist>
          </xsl:if>
        </dropdown>
      </xsl:when>

      <xsl:when test="@class='GtkVSeparator'">
        <separator alignment="vertical">
          <xsl:apply-templates />
        </separator>
      </xsl:when>
      <xsl:when test="@class='GtkHSeparator'">
        <separator alignment="horizontal">
          <xsl:apply-templates />
        </separator>
      </xsl:when>
      <xsl:when test="@class='GtkStatusbar'">
        <statusbar>
          <xsl:apply-templates />
        </statusbar>
      </xsl:when>

      <!-- It seems only a parent of a 'GtkTreeView' is a 
           'GtkScrolledWindow' -->
      <xsl:when test="@class='GtkScrolledWindow' 
                      and ./child/widget/@class='GtkTreeView'">
        <list>
          <xsl:comment>
            <xsl:text>Please Edit Me!</xsl:text>
          </xsl:comment>
          <column title="Column 1" />
          <column title="Column 2" />
        </list>
      </xsl:when>
      <xsl:when test="@class='GtkScrolledWindow' 
                      and ./child/widget/@class='GtkTextView'">
        <textview wrap="word">
          <xsl:attribute name="name">
            <xsl:value-of select="child/widget/@id" />
          </xsl:attribute>
          <xsl:comment>
            <xsl:text>May need manual editing</xsl:text>
          </xsl:comment>
          <content>
            <xsl:value-of select="./child/widget/property[@name='text']" />
          </content>
        </textview>
      </xsl:when>

      <xsl:when test="@class='GtkTreeView' or @class='GtkMenu'">
        <xsl:apply-templates />
      </xsl:when>

      <!-- these three widgets are children of a toolbar -->
      <xsl:when test="@class='toggle'">
        <toggle>
          <xsl:apply-templates />
        </toggle>
      </xsl:when>
      <xsl:when test="@class='radio'">
        <radio>
          <xsl:apply-templates />
        </radio>
      </xsl:when>
      <xsl:when test="@class='button'">
        <button>
          <xsl:apply-templates />
          <trigger event="click" />
        </button>
      </xsl:when>

      <!-- This is for debugging. To make sure we haven't left
           anything out, print a comment with the name of the unknown
           element, the element and continue parsing. -->
      <xsl:otherwise>
        <xsl:comment>This widget was not parsed:<xsl:value-of select="@class" /></xsl:comment>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Convert attributes of the containing widget -->
  <xsl:template match="property">
    <!-- Add the name property (applies to all widgets) -->
    <xsl:if test="../@id">
      <xsl:attribute name="name">
        <xsl:value-of select="../@id" />
      </xsl:attribute>
    </xsl:if>

    <xsl:choose>
      <!-- remove 'n_' prefix for 'rows' and 'columns' properties -->
      <xsl:when test="@name='n_rows'">
        <xsl:attribute name="rows">
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>
      <xsl:when test="@name='n_columns'">
        <xsl:attribute name="columns">
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>
      <xsl:when test="@name='visibility' and text()='False'">
        <xsl:attribute name="visible">
          <xsl:call-template name="property-content" />          
        </xsl:attribute>
      </xsl:when>
      <xsl:when test="@name='title'">
        <xsl:call-template name="get-property" />
      </xsl:when>

      <xsl:when test="@name='stock_pixmap'">
        <xsl:attribute name="stock">
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>


      <!-- These properties are children of the packing element -->
      <xsl:when test="@name='left_attach'">
        <xsl:attribute name="xpos">
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>
      <xsl:when test="@name='right_attach' and (text()-(../property[@name='left_attach']/text()))!='1'">
        <xsl:attribute name="xspan">
          <xsl:value-of select="text()-(../property[@name='left_attach']/text())" />
        </xsl:attribute>
      </xsl:when>
      <xsl:when test="@name='top_attach'">
        <xsl:attribute name="ypos">
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>
      <xsl:when test="@name='bottom_attach' and (text()-(../property[@name='top_attach']/text()))!='1'">
        <xsl:attribute name="yspan">
          <xsl:value-of select="text()-(../property[@name='top_attach']/text())" />
        </xsl:attribute>
      </xsl:when>
      
      <xsl:when test="@name='padding' and text()!='0'">
        <xsl:call-template name="get-property" />
      </xsl:when>

      <xsl:when test="@name='x_padding' and text()!='0'">
        <xsl:attribute name="xpad">
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>

      <!--
      <xsl:when test="@name='wrap' and test()!='False'">
        <xsl:call-template name="get-property" />
      </xsl:when>
      -->

      <xsl:when test="@name='y_padding' and text()!='0'">
        <xsl:attribute name="ypad">
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>

      <xsl:when test="@name='expand'">
        <xsl:call-template name="get-property" />
      </xsl:when>

      <xsl:when test="@name='fill' and text()!='True'">
        <xsl:call-template name="get-property" />
      </xsl:when>

      <xsl:when test="@name='x_options' and text()='fill'">
        <xsl:attribute name="xfill">
          <xsl:text>true</xsl:text>
        </xsl:attribute>
      </xsl:when>
      
      <xsl:when test="@name='y_options' and text()='fill'">
        <xsl:attribute name="yfill">
          <xsl:text>true</xsl:text>
        </xsl:attribute>
      </xsl:when>
      <xsl:when test="@name='y_options' and not(text())">
        <xsl:attribute name="yexpand">
          <xsl:text>false</xsl:text>
        </xsl:attribute>
      </xsl:when>

      <!--
      <xsl:when test="@name='wrap_mode' and text()='GTK_WRAP_NONE'">
        <xsl:attribute name='wrap'>
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>

      <xsl:when test="@name='cursor_visible' and text()!='True'">
        <xsl:attribute name='cursorvisible'>
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>
      -->

      <xsl:when test="@name='shadow_type' and ../@class='GtkFrame'">
        <xsl:attribute name='shadow'>
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>

      <xsl:when test="@name='toolbar_style'">
        <xsl:attribute name='style'>
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>
      
      <xsl:when test="@name='label' and not(../@class='GtkLabel')">
        <xsl:choose>
          <xsl:when test="contains(text(), 'gtk-')">
            <xsl:attribute name="stock">
              <xsl:call-template name="property-content" />
            </xsl:attribute>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="get-property" />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <!-- homogeneous is only implemented in box, vbox and hbox -->
      <xsl:when test="@name='homogeneous' and text()='True' 
                      and (../@class='GtkTable' 
                          or ../@class='GtkVBox'
                          or ../@class='GtkHBox'
                          or ../@class='GtkHButtonBox'
                          or ../@class='GtkVButtonBox')">
        <xsl:call-template name="get-property" />
      </xsl:when>

      <xsl:when test="text()!='4' and @name='border_width'">
        <xsl:attribute name="border">
          <xsl:call-template name="property-content" />
        </xsl:attribute>
      </xsl:when>

      <xsl:when test="text()!='4' and @name='spacing'">
        <xsl:call-template name="get-property" />
      </xsl:when>

      <xsl:when test="text()!='4' and (@name='row_spacing' or @name='column_spacing')">
        <xsl:call-template name="get-property" />
      </xsl:when>

      <!-- bond supports justify for textview widgets only -->
      <xsl:when test="@name='justification' and ../@class='GtkTextView'">
        <xsl:call-template name="get-property" />        
      </xsl:when>

      <xsl:when test="@name='use_underline' and text()='True'
                      and (../@class='GtkCheckButton'
                        or ../@class='GtkRadioButton')">
        <xsl:call-template name="get-property" />
      </xsl:when>

      <!-- visible is implemented in most elements, but has different
           default values for each -->
      <xsl:when test="@name='visible' 
                      and (
                          (text()='False'
                          and not(../@class='GtkStatusbar' 
                               or ../@class='GtkToolbar'
                               or ../@class='GtkVSeparator'
                               or ../@class='GtkMenuItem'
                               or ../@class='GtkHandleBox'))
                          or ../@class='GtkWindow')">
        <xsl:call-template name="get-property" />
      </xsl:when>


      <xsl:when test="@name='group'">
        <xsl:call-template name="get-property" />
      </xsl:when>
      <!-- otherwise this has currently been unparsed -->
      <xsl:otherwise />
    </xsl:choose>
  </xsl:template>

  <!-- only these signals are currently supported in Bond2 ??? -->
  <xsl:template match="signal">
    <xsl:variable name="event">
      <xsl:choose>
        <xsl:when test="@name='changed'">change</xsl:when>
        <xsl:when test="@name='focus_in_event'">entry</xsl:when>
        <xsl:when test="@name='focus_out_event'">exit</xsl:when>
        <xsl:when test="@name='activate'">click</xsl:when>
        <xsl:when test="@name='clicked'">click</xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@name" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="api-call">
      <xsl:call-template name="get-api-call" />
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$event!=''">
        <trigger event="{$event}">
          <xsl:if test="$api-call!=''">
            <api call="{$api-call}">
              <widget><xsl:value-of select="@object" /></widget>
            </api>
          </xsl:if>
        </trigger>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="get-api-call">
    <xsl:param name="output-all-handlers" select="'false'" />
    <xsl:variable name="api-call" select="substring-after(@handler, 'bond_')"/>
    <xsl:choose>
      <xsl:when test="$api-call='' and $output-all-handlers='true'">
        <xsl:value-of select="@handler" />
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$api-call" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="child
                       [../@class='GtkTable' 
                       or ../@class='GtkVBox' 
                       or ../@class='GtkHBox'
                       or ../@class='GtkHButtonBox'
                       or ../@class='GtkVButtonBox']">
    <cell>
      <xsl:if test="widget/@class='GtkScrolledWindow'">
        <xsl:attribute name="yexpand">
          <xsl:text>true</xsl:text>
        </xsl:attribute>
      </xsl:if>
      <xsl:apply-templates select="packing" />
      <xsl:apply-templates select="widget" />
    </cell>
  </xsl:template>

  <xsl:template match="child
                       [../@class='GtkNotebook'
                       and not(./widget/@class='GtkLabel')]">
    <page title="{following-sibling::child[1]/widget[@class='GtkLabel']/property[@name='label']}">
      <xsl:apply-templates select="packing" />
      <xsl:apply-templates select="widget" />
    </page>
  </xsl:template>

  <xsl:template match="child
                       [../@class='GtkNotebook'
                       and ./widget/@class='GtkLabel']" />


  <xsl:template match="child[../@class='GtkFrame' 
                       and ./widget/@class='GtkLabel']" />

  <xsl:template match="child">
    <xsl:apply-templates select="packing" />
    <xsl:apply-templates select="widget" />
  </xsl:template>

  <xsl:template name="property-content">
    <xsl:choose>
      <xsl:when test="text()='True' or text()='yes'">true</xsl:when>
      <xsl:when test="text()='False' or text()='no'">false</xsl:when>
      <xsl:when test="text()='GTK_ORIENTATION_HORIZONTAL'">horizontal</xsl:when>
      <xsl:when test="text()='GTK_ORIENTATION_VERTICAL'">vertical</xsl:when>
      <xsl:when test="text()='GTK_JUSTIFY_LEFT'">left</xsl:when>
      <xsl:when test="text()='GTK_JUSTIFY_RIGHT'">right</xsl:when>
      <xsl:when test="text()='GTK_JUSTIFY_CENTER'">center</xsl:when>
      <xsl:when test="text()='GTK_JUSTIFY_FILL'">fill</xsl:when>
      <xsl:when test="text()='GTK_SHADOW_NONE'">none</xsl:when>
      <xsl:when test="text()='GTK_SHADOW_IN'">in</xsl:when>
      <xsl:when test="text()='GTK_SHADOW_OUT'">out</xsl:when>
      <xsl:when test="text()='GTK_SHADOW_ETCHED_IN'">etched-in</xsl:when>
      <xsl:when test="text()='GTK_SHADOW_ETCHED_OUT'">etched-out</xsl:when>
      <xsl:when test="text()='GTK_TOOLBAR_BOTH'">both</xsl:when>
      <xsl:when test="text()='GTK_TOOLBAR_ICONS'">icons</xsl:when>
      <xsl:when test="text()='GTK_TOOLBAR_TEXT'">text</xsl:when>
      <xsl:when test="text()='GTK_TOOLBAR_BOTH_HORIZ'">both-beside</xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="text()" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="get-property">
    <xsl:attribute name="{@name}">
      <xsl:call-template name="property-content" />
    </xsl:attribute>
  </xsl:template>

  <!-- Note: incomplete!!! -->
  <xsl:template name="predict-api-call">
    <xsl:value-of select="@handler" />
  </xsl:template>

  <xsl:template match="*" />


</xsl:stylesheet>
