<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">
  <xsl:output 
    method="xml" 
    indent="yes" 
    doctype-public="-//treshna.com//DTD Bond V2.0//EN" 
    doctype-system="../bond-2.0.dtd" />
  <xsl:strip-space elements="*" />

  <xsl:template match="/">
    <xsl:apply-templates select="*|@*|node()" />
  </xsl:template>

  <xsl:template match="*|@*|node()">
    <xsl:copy>
      <xsl:apply-templates select="*|@*|node()"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="trigger[not(child::api)]">
    <xsl:variable name="api-call">
      <xsl:call-template name="get-api-call" />
    </xsl:variable>
    <xsl:variable name="api-widget">
      <xsl:call-template name="get-api-widget" />
    </xsl:variable>
    <trigger event="{@event}">
      <xsl:if test="$api-call!=''">
        <api call="{$api-call}">
          <xsl:if test="$api-widget!=''">
            <widget><xsl:value-of select="$api-widget" /></widget>
          </xsl:if>
        </api>
      </xsl:if>
    </trigger>
  </xsl:template>

  <xsl:template name="get-api-call">
    <xsl:variable name="widget-stock">
      <xsl:choose>
        <xsl:when test="contains(../@stock, 'gtk-')!=''">
          <xsl:value-of select="substring-after(../@stock, 'gtk-')" />
        </xsl:when>
        <xsl:when test="contains(../@label, 'gtk-')!=''">
          <xsl:value-of select="substring-after(../@label, 'gtk-')" />
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="widget-label" select="../@label" />
    <xsl:choose>
      <xsl:when test="$widget-stock!=''">
        <xsl:choose>
          <xsl:when test="$widget-stock='open' or
                          $widget-stock='close' or
                          $widget-stock='add' or
                          $widget-stock='new' or
                          $widget-stock='delete' or
                          $widget-stock='refresh' or
                          $widget-stock='save' or
                          $widget-stock='clear' or
                          $widget-stock='quit'">
            <xsl:value-of select="$widget-stock" />
          </xsl:when>
          
          <!-- appy = refresh? remove = delete ? -->
          <xsl:when test="$widget-stock='remove'">delete</xsl:when>      
          <xsl:when test="$widget-stock='apply'">refresh</xsl:when>
          
          <xsl:when test="$widget-stock='go-forward'">next</xsl:when>
          <xsl:when test="$widget-stock='go-back'">prev</xsl:when>
          <xsl:when test="$widget-stock='goto-first'">first</xsl:when>
          <xsl:when test="$widget-stock='goto-last'">last</xsl:when>
          <xsl:when test="$widget-stock='jump-to'">moveto</xsl:when>
          <xsl:otherwise />
        </xsl:choose>
      </xsl:when>
      <xsl:when test="//window[@title=$widget-label] or 
                      //notebook/page[@title=$widget-label]">open</xsl:when>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="get-api-widget">
    <xsl:variable name="widget-stock">
      <xsl:choose>
        <xsl:when test="contains(../@stock, 'gtk-')!=''">
          <xsl:value-of select="substring-after(../@stock, 'gtk-')" />
        </xsl:when>
        <xsl:when test="contains(../@label, 'gtk-')!=''">
          <xsl:value-of select="substring-after(../@label, 'gtk-')" />
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="widget-label" select="../@label" />
    <xsl:choose>
      <xsl:when test="$widget-stock!=''">
        <xsl:value-of select="ancestor::window/@name" />
      </xsl:when>
      <xsl:when test="//window[@title=$widget-label]">
        <xsl:value-of select="//window[@title=$widget-label]/@name" />
      </xsl:when>
      <xsl:when test="//notebook/page[@title=$widget-label]">
        <xsl:value-of select="//window/@name[notebook/page/@title=$widget-label]" />
      </xsl:when>
    </xsl:choose>
  </xsl:template>
</xsl:stylesheet>
