#include <stdio.h>
#include <stdlib.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "xml_display.h"

static int xml_display_node_pad (xmlNode * node, int depth, FILE * outfile);

static int
xml_display_attribute_pad (xmlAttr * attribute, int depth, FILE * outfile)
{
  xmlNode *child;

  if (!attribute)
    return -1;

  printf ("%*sAttribute: %s\n", depth, "", attribute->name);
  fprintf (outfile, "%*sAttribute: %s\n", depth, "", attribute->name);

  for (child = attribute->children; child != NULL; child = child->next)
    {
      xml_display_node_pad (child, depth + 1, outfile);
    }

  return 0;
}

int
xml_display_attribute (xmlAttr * a, FILE * outfile)
{
  return xml_display_attribute_pad (a, 0, outfile);
}

static int
xml_display_node_pad (xmlNode * node, int depth, FILE * outfile)
{
  xmlNode *child;
  xmlAttr *attr;

  if (!node)
    return -1;

  printf ("%*sNode: %s\n", depth, "", node->name);
  fprintf (outfile, "%*sNode: %s\n", depth, "", node->name);

  if (node->content)
    {
      fprintf (outfile, "%*sContent: %s\n", depth, "", node->content);
      printf ("%*sContent: %s\n", depth, "", node->content);
    }

  for (attr = node->properties; attr != NULL; attr = attr->next)
    xml_display_attribute_pad (attr, depth + 1, outfile);

  for (child = node->children; child != NULL; child = child->next)
    xml_display_node_pad (child, depth + 1, outfile);

  return 0;
}


int
xml_display_node (xmlNode * node, FILE * outfile)
{
  return xml_display_node_pad (node, 0, outfile);
}

static int
xml_display_doc_pad (xmlDoc * doc, int depth, FILE * outfile)
{
  xmlNode *node;

  if (!doc)
    return -1;

  fprintf (outfile, "%*sDoc: %s\n", depth, "", doc->name);
  printf ("%*sDoc: %s\n", depth, "", doc->name);

  for (node = doc->children; node != NULL; node = node->next)
    {
      xml_display_node_pad (node, depth + 1, outfile);
    }

  return 0;
}

int
xml_display_doc (xmlDoc * doc, FILE * outfile)
{
  return xml_display_doc_pad (doc, 0, outfile);
}
