#include <stdio.h>
#include <stdlib.h>
#include <slang.h>
#include <string.h>

#include <glib.h>
#include <libgnomeprint/gnome-font.h>
#include <gtk/gtk.h>

#include <bond/db.h>
#include <bond/dbclient.h>
#include <bond/dbobject.h>

#include "print.h"
#include "db.h"
#include "general.h"
#include "layout.h"

int slang_init (void)
   {      
   if (SLang_init_slang () == -1 ||
       SLang_init_slmath () == -1 ||
       SLang_init_slunix () == -1 ||
       SLang_init_slfile () == -1)
      return -1;
   
   return 0;
   }

void list_fonts (void)
   {
   GList *list, *start;
   char *name;
   SLang_Array_Type *at;
   int i, len;
   
   start = gnome_font_list ();
   len = g_list_length (start);
   at = SLang_create_array (SLANG_STRING_TYPE, 0, NULL, &len, 1);
      
   for (list = start, i = 0; list != NULL; list = list->next, i++)
      {      
      name = strdup (list->data);
      if (SLang_set_array_element (at, &i, &name) == -1)
	 {
	 SLang_free_array (at);
	 return;
	 }      
      }
   
   gnome_font_list_free (start);
   
   SLang_push_array (at, 0);
   SLang_free_array (at);   
   }

#define MY_ADD_FUNC_0(ns, name, func, ret) \
   if (SLns_add_intrinsic_function (ns, name, (FVOID_STAR)func, ret, 0) == -1) \
      return -1

#define MY_ADD_FUNC_1(ns, name, func, ret, arg1) \
   if (SLns_add_intrinsic_function (ns, name, (FVOID_STAR)func, ret, 1, arg1) == -1) \
      return -1

#define MY_ADD_FUNC_2(ns, name, func, ret, arg1, arg2) \
   if (SLns_add_intrinsic_function (ns, name, (FVOID_STAR)func, ret, 2, arg1, arg2) == -1) \
      return -1

#define MY_ADD_FUNC_3(ns, name, func, ret, arg1, arg2, arg3) \
   if (SLns_add_intrinsic_function (ns, name, (FVOID_STAR)func, ret, 3, arg1, arg2, arg3) == -1) \
      return -1

#define MY_ADD_FUNC_4(ns, name, func, ret, arg1, arg2, arg3, arg4) \
   if (SLns_add_intrinsic_function (ns, name, (FVOID_STAR)func, ret, 4, arg1, arg2, arg3, arg4) == -1) \
      return -1

#define MY_ADD_FUNC_5(ns, name, func, ret, arg1, arg2, arg3, arg4, arg5) \
   if (SLns_add_intrinsic_function (ns, name, (FVOID_STAR)func, ret, 5, arg1, arg2, arg3, arg4, arg5) == -1) \
      return -1

#define MY_ADD_FUNC_6(ns, name, func, ret, arg1, arg2, arg3, arg4, arg5, arg6) \
   if (SLns_add_intrinsic_function (ns, name, (FVOID_STAR)func, ret, 6, arg1, arg2, arg3, arg4, arg5, arg6) == -1) \
      return -1

#define MY_ADD_FUNC_7(ns, name, func, ret, arg1, arg2, arg3, arg4, arg5, arg6, arg7) \
   if (SLns_add_intrinsic_function (ns, name, (FVOID_STAR)func, ret, 7, arg1, arg2, arg3, arg4, arg5, arg6, arg7) == -1) \
      return -1

int add_functions (void)
   {
   SLang_NameSpace_Type *ns = SLns_create_namespace ("print");   
   SLang_NameSpace_Type *db = SLns_create_namespace ("bond");
   SLang_NameSpace_Type *gen = SLns_create_namespace ("gen");
   SLang_NameSpace_Type *lo = SLns_create_namespace ("layout");
   
   if (ns == NULL)
      return -1;
   
   if (db == NULL)
      return -1;

   if (gen == NULL)
      return -1;
   
   if (lo == NULL)
      return -1;
   
   MY_ADD_FUNC_0 (ns, "fonts", list_fonts, SLANG_VOID_TYPE);
   
   MY_ADD_FUNC_2 (ns, "start_file", print_start_file, SLANG_INT_TYPE,
		  SLANG_STRING_TYPE, SLANG_STRING_TYPE);
   MY_ADD_FUNC_1 (ns, "start_printer", print_start_printer, SLANG_INT_TYPE,
		  SLANG_STRING_TYPE);
   MY_ADD_FUNC_1 (ns, "start_preview", print_start_preview, SLANG_INT_TYPE,
		  SLANG_STRING_TYPE);
   MY_ADD_FUNC_1 (ns, "stop", print_stop, SLANG_VOID_TYPE, 
		  SLANG_INT_TYPE);

   MY_ADD_FUNC_0 (ns, "paper_width", print_paper_width, SLANG_INT_TYPE);
   MY_ADD_FUNC_0 (ns, "paper_height", print_paper_height, SLANG_INT_TYPE);
   
   MY_ADD_FUNC_1 (ns, "newpage", print_newpage, SLANG_VOID_TYPE, 
		  SLANG_INT_TYPE);   
   MY_ADD_FUNC_1 (ns, "showpage", print_showpage, SLANG_VOID_TYPE, 
		  SLANG_INT_TYPE);
   if (SLns_add_intrinsic_variable (ns, "current_page", &print_current_page, 
				    SLANG_INT_TYPE, 1) == -1)
      return -1;

   MY_ADD_FUNC_1 (ns, "newpath", print_newpath, SLANG_VOID_TYPE, 
		  SLANG_INT_TYPE);
   MY_ADD_FUNC_3 (ns, "moveto", print_moveto, SLANG_VOID_TYPE, 
		 SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE);
   MY_ADD_FUNC_3 (ns, "lineto", print_lineto, SLANG_VOID_TYPE, 
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE);
   MY_ADD_FUNC_6 (ns, "box", print_box, SLANG_VOID_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE);
   MY_ADD_FUNC_1 (ns, "stroke", print_stroke, SLANG_VOID_TYPE, 
		  SLANG_INT_TYPE);
   MY_ADD_FUNC_2 (ns, "setlinewidth", print_setlinewidth, SLANG_VOID_TYPE,
		  SLANG_INT_TYPE, SLANG_DOUBLE_TYPE);
	 
   MY_ADD_FUNC_4 (ns, "setcolour", print_setcolour, SLANG_VOID_TYPE,
		  SLANG_INT_TYPE, SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE,
		  SLANG_DOUBLE_TYPE);
   MY_ADD_FUNC_2 (ns, "getfont", print_getfont, SLANG_INT_TYPE,
		  SLANG_STRING_TYPE, SLANG_DOUBLE_TYPE);
   MY_ADD_FUNC_2 (ns, "setfont", print_setfont, SLANG_VOID_TYPE, 
		  SLANG_INT_TYPE, SLANG_INT_TYPE);
   MY_ADD_FUNC_1 (ns, "font_height", print_lineheight, SLANG_INT_TYPE,
		  SLANG_INT_TYPE);
   MY_ADD_FUNC_2 (ns, "line_width", print_linewidth, SLANG_INT_TYPE,
		  SLANG_INT_TYPE, SLANG_STRING_TYPE);
   
   MY_ADD_FUNC_3 (ns, "show", print_show, SLANG_INT_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_STRING_TYPE);
   MY_ADD_FUNC_5 (ns, "showat", print_showat, SLANG_INT_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE,
		  SLANG_INT_TYPE, SLANG_STRING_TYPE);
   MY_ADD_FUNC_6 (ns, "showcentered", print_show_centered, SLANG_INT_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_STRING_TYPE);   
   MY_ADD_FUNC_6 (ns, "show_paragraph", print_show_paragraph, SLANG_INT_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_STRING_TYPE);

   MY_ADD_FUNC_4 (ns, "image", print_image, SLANG_VOID_TYPE, 
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE,
		  SLANG_STRING_TYPE);
   
   MY_ADD_FUNC_5 (ns, "image_scaled", print_image_scaled, SLANG_VOID_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE, 
		  SLANG_DOUBLE_TYPE, SLANG_STRING_TYPE);

   /* Database functions */

   MY_ADD_FUNC_1 (db, "createobject", db_createobject, SLANG_INT_TYPE,
		  SLANG_STRING_TYPE);
   MY_ADD_FUNC_3 (db, "getobjectbyfield", db_getobjectbyfield_wrap, SLANG_VOID_TYPE,
		  SLANG_INT_TYPE, SLANG_STRING_TYPE, SLANG_STRING_TYPE);
   MY_ADD_FUNC_2 (db, "getfieldstr", db_getfieldstr_wrap, SLANG_STRING_TYPE,
		  SLANG_INT_TYPE, SLANG_STRING_TYPE);
   MY_ADD_FUNC_2 (db, "getfieldint", db_getfieldint_wrap, SLANG_INT_TYPE,
		  SLANG_INT_TYPE, SLANG_STRING_TYPE);
   MY_ADD_FUNC_1 (db, "freeobject", db_freeobject_wrap, SLANG_VOID_TYPE,
		  SLANG_INT_TYPE);
   MY_ADD_FUNC_1 (db, "loadallobjects", db_loadallobjects_wrap, SLANG_VOID_TYPE,
		  SLANG_INT_TYPE);
   MY_ADD_FUNC_1 (db, "num", db_objnum_wrap, SLANG_INT_TYPE,
		  SLANG_INT_TYPE);
   MY_ADD_FUNC_1 (db, "movenext", db_movenext_wrap, SLANG_VOID_TYPE,
		  SLANG_INT_TYPE);
   MY_ADD_FUNC_2 (db, "moverecord", db_moverecord_wrap, SLANG_VOID_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE);
   MY_ADD_FUNC_2 (db, "sql", db_sql_wrap, SLANG_INT_TYPE,
		  SLANG_STRING_TYPE, SLANG_STRING_TYPE);

   /* General Functions */
   MY_ADD_FUNC_2 (gen, "query_string", gen_query_string, SLANG_STRING_TYPE,
		  SLANG_STRING_TYPE, SLANG_STRING_TYPE);   
   MY_ADD_FUNC_1 (gen, "query_password", gen_query_password, SLANG_STRING_TYPE,
		  SLANG_STRING_TYPE);
   MY_ADD_FUNC_1 (gen, "query_yesno", gen_query_yesno, SLANG_INT_TYPE,
		  SLANG_STRING_TYPE);
   MY_ADD_FUNC_4 (gen, "query_integer", gen_query_integer, SLANG_INT_TYPE,
		  SLANG_STRING_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE,
		  SLANG_INT_TYPE);
   MY_ADD_FUNC_4 (gen, "query_double", gen_query_double, SLANG_DOUBLE_TYPE,
		  SLANG_STRING_TYPE, SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE, 
		  SLANG_DOUBLE_TYPE);
   MY_ADD_FUNC_2 (gen, "query_date", gen_query_date, SLANG_STRING_TYPE,
		  SLANG_STRING_TYPE, SLANG_STRING_TYPE);   
   MY_ADD_FUNC_0 (gen, "today", gen_today, SLANG_STRING_TYPE);


   /* layout functions */
   MY_ADD_FUNC_0 (lo, "begin", layout_begin, SLANG_INT_TYPE);
   MY_ADD_FUNC_0 (lo, "add", layout_add, SLANG_INT_TYPE);
   MY_ADD_FUNC_2 (lo, "compile", layout_compile, SLANG_VOID_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE);
   MY_ADD_FUNC_1 (lo, "width", layout_width, SLANG_INT_TYPE,
		  SLANG_INT_TYPE);
   MY_ADD_FUNC_1 (lo, "height", layout_height, SLANG_INT_TYPE,
		  SLANG_INT_TYPE);
   MY_ADD_FUNC_5 (lo, "display", layout_display, SLANG_VOID_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE, SLANG_INT_TYPE,
		  SLANG_INT_TYPE, SLANG_INT_TYPE);
   MY_ADD_FUNC_0 (lo, "fulldisplay", layout_fulldisplay, SLANG_VOID_TYPE);

   
   return 0;
   }

int add_args (char **args, int count)
   {
   int i;
   char name[30], **value, *pos;
   
   for (i = 0; i < count; i++)
      {
      pos = strstr (args[i], "=");
      if (pos)
	 {
	 strncpy (name, args[i], pos - args[i]);
	 name[pos-args[i]] = '\0';
	 value = malloc (sizeof (char *));
	 *value = strdup (&args[i][pos - args[i] + 1]);
	 if (SLadd_intrinsic_variable (name, value, SLANG_STRING_TYPE, 1) == -1)
	    {
	    fprintf (stderr, "Error adding intrinsic variable: %s\n", name);
	    return -1;
	    }
	 /* fprintf (stderr, "Found arg: `%s' = `%s'\n", name, *value); */
	 }
      else
	 {
	 fprintf (stderr, "Argument: \"%s\" malformed\n", args[i]);
	 return -1;
	 }
      }
   
   return 0;
   }

int main (int argc, char *argv[])
   {
   char *file;
   
   if (slang_init () != 0)
      {
      fprintf (stderr, "Unable to initialize S-Lang\n");
      return 1;
      }
   
   if (argc < 2)
      {
      fprintf (stderr, "Usage: %s report [args]\n", argv[0]);
      return 1;
      }
      
   if (print_init () != 0)
      {
      fprintf (stderr, "Error initializing print interface\n");
      return 1;
      }
   
   if (rdb_init () != 0)
      {
      fprintf (stderr, "Error initializing database interface\n");
      return 1;
      }

   if (add_functions () != 0)
      {
      fprintf (stderr, "Error adding custom functions to interface\n");
      return 1;
      }

   if (add_args (&argv[2], argc - 2) == -1)
      {
      fprintf (stderr, "Error adding arguments\n");
      return 1;
      }

   file = (strcmp (argv[1], "-") == 0) ? NULL : argv[1];
   
   if (SLang_load_file (file) == -1)
      {
      fprintf (stderr, "Error with file: %s: %s\n", argv[1],
	       SLerrno_strerror (SLang_Error));
      return 1;
      }
   
   print_cleanup ();
   rdb_cleanup ();
   
   return 0;
   }
