/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef bondmain_h
#define bondmain_h

/**=============================================================================
 * db_init(char * cfname)
 *
 * Parameters: cfname - name of the config file to use
 *
 * bond_init calls the startup functions of dbclient, debug, and roof
 * bond_clean calls the free functions for a number of modules
 *
 * Return value: -1 - An error has occured - will be printed with the errormsg 
 * function
 *			    0 - Execution was successful
 * =============================================================================
 */

#include <glib.h>
#include <gmodule.h>

#include "uicommonplugin.h"

typedef struct _Bond Bond;
typedef struct _GlobalRegistor GlobalRegistor;

#include "bond.h"

struct _GlobalRegistor
	{
	/* lists of all registrations and watch towers */
	/* registration key lookup */
	GHashTable *reghashbykey;
	GHashTable *reghashbyfield;
	GHashTable *reghashbywidget;
	GHashTable *reghashbydbobjectsrc;
	/* watch lookups */
	GHashTable *watchhashbykey;
	GHashTable *watchhashbyfield;
	GHashTable *watchhashbywidget;
	GHashTable *watchhashbydbobjectsrc;
	};

struct _Bond
	{
	/* bondxml document */
	/* This is defined in uicommonplugin.h and is the gui definition */
	UiXML *uixml;
	/* BondObjects Cache */
	char *lastlongwidgetname;
	BondObject *lastbobj;
	/* BondObjects */
	GHashTable *globalbobj;		  /* by widget name */
	GHashTable *globalbobj_byobj;	/* by db object */
	/* Cache of db objects who have cache set to true */
	GHashTable *objcache;
	/* BondRelations in waiting for spawn */
	GList *brelinwait;
	/* Bondxml parser */
	/* Operation nodes */
	GHashTable *hashoprnode;	  /* by name of widget */
	GHashTable *hashdbobjectsrc; /* by db object source */
	/* Watchtower and registry */
	GlobalRegistor *globalreg;
	/* Function symbols */
	GModule *symbols;
	GList *othermodules;
	};

gint bond_init(gint argc, gchar * argv[], Bond ** bond);
gint bond_cleanup(Bond * bond);
gint bond_showall(Bond * bond);
void bond_saveall(Bond * bond);
void bond_closeall(Bond * bond);
gint bond_main(Bond * bond);
gint bond_linkagainst(Bond * bond, gchar * library, gchar * initfunc);

#endif
