/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* treshna Enterprises Ltd. 
 * (C) 2004 Andrew Hill 
 */
#ifndef bondxml_h
#define bondxml_h

#include <libxml/parser.h>

#include "ui.h"
#include "uicommonplugin.h"
#include "dbfilter.h"
#include "bondmain.h"

/**
 * PLEASE NOTE:
 *
 * The xml file is parsed twice. Bondxml.c parses the xml file on the second
 * parse. parse.c in the uigtk directory does it in the first parse. This first
 * parse is responsible for building all the user interface. It creates
 * all the gtk widgets and works out what the widgets are called from
 * the node structure. The first phase creates the UiWidgets* and
 * the hashtables used to look them up.
 *
 * This second parse goes though and adds all the database information
 * like fields, tables, what depends on what, what callback functions exist etc.
 * This parse creates the OperationsNode*.
 */ 

/* State of parser */
typedef enum
{
    ParseStart, ParseDbObject, ParseLookup, ParseFilter, ParseSearch,
    ParseTrigger, ParseFunction, ParseArgument, ParseAPI, ParseReportfile
}
ParseState;

/* Trigger callback events, when do you run the trigger */
typedef enum
{
    EventNone, EventDefault, EventChange, EventEntry, EventExit, EventClick,
    EventBeforePaint, EventAfterPaint, EventBeforeSave, EventAfterSave,
    EventSelect, EventOpen, EventClose, EventAdd, EventDelete, EventGridEdit
}
TriggerEvent;

/* Specifies the different type of high-level API calls that can be made. */
typedef enum
{
    ApiNone, ApiOpen, ApiClose, ApiAdd, ApiNew, ApiDelete, ApiLoadBy, 
    ApiLoadAll, ApiSQL, ApiRefresh, ApiSave, ApiQuit, ApiReport, ApiClear, 
    ApiFlush, ApiNext, ApiPrev, ApiFirst, ApiLast, ApiMoveTo, ApiAbortSave
}
ApiCall;

/* If input in entrybox is restricted somehow this is where its mentoned */
typedef enum
{
    InputRestrictNone, InputRestrictInt, InputRestrictFloat, InputRestrictDate,
    InputRestrictTime, InputRestrictString, InputRestrictToUpper, 
    InputRestrictToLower, InputRestrictFirstToUpper, InputRestrictPhone, 
    InputRestrictAll, InputRestrictRound
}
InputRestrict;

typedef struct _OperationsNode OperationsNode;
typedef struct _OperationsNodeDb OperationsNodeDb;
typedef struct _OperationsNodeFilter OperationsNodeFilter;
typedef struct _OperationsNodeSearch OperationsNodeSearch;
typedef struct _OperationsNodeTrigger OperationsNodeTrigger;
typedef struct _OperationsNodeTriggerArgument OperationsNodeTriggerArgument;

struct _OperationsNodeFilter
	{
	gchar *widgetname;
	gchar *field;
	gchar *table;
	gchar *value_constant;
	gchar *value_field;
	gchar *value_table;
	gchar *value_widget;
	gchar *value_dbobjectsrc;
	gchar *value_dbobjectsrc_widget;
	sql_condition_operator opr;
	};

struct _OperationsNodeSearch
	{
	gchar *widgetname;
	gchar *field;
	gchar *table;
	gboolean casesensitive;
	sql_condition_operator opr;
	UiWidget *widget;
	};

struct _OperationsNodeDb
	{
	/* Basic stuff */
	gchar *widgetname;
	gchar *name;                /* name attached to the node we are in */
	gchar *table;               /* database table */
	gchar *sql;                 /* sql statement if any */
	GList *loadby;              /* Table we are suppose to load ourselves 
				     * by */
	gboolean unreferencesql;    /* if the sql needs to be evaualeted first 
				     * to unreference lookups */
	gboolean loadall;           /* if loadall is set to true */
	gboolean cache;             /* am i suppose to keep a copy of u always 
				     */
	gboolean reloadonchange;    /* If dbobject is suppose to reload on 
				     * change */
	gchar *sortby;              /* sortby field */

	/* Filter */
	GList *filters;
	GList *search;
	gboolean dontexec;	    /* dont execute the sql. */
	};

struct _OperationsNodeTriggerArgument
	{
	gchar *name;			/* name attached to the argument */
	gchar *widgetname;		/* widget argument */
	gchar *field;			/* table.fieldname argument */
	gchar *table;			/* Tablename for the fieldname */
	gchar *constant;		/* If it is a constant instead. */
	gchar *dbobjectsrc;		/* if i am to send a whole dbobject */
	GList *fieldpathlist;
	gboolean fieldpathtomake;
	};

struct _OperationsNodeTrigger
	{
	gchar *widgetname;		/* What called me, the widget name */
	TriggerEvent event;		/* When to run trigger */
	gchar *functionlanguage;	/* Function details if function */
	gchar *functionname;
	void *ptr2func;
	ApiCall apicall;		/* type of action */
	gboolean valuelink;		/* is it a direct link to another 
					 * field */
	gboolean reportpreview;		/* if to show a report preview if its 
					 * a report */
	gboolean confirmaction;		/* if an api call you can sometimes want
					 * a confirmation box (ie delete) */
        gboolean runpapyrus;		/* If papyrusgui is suppose to be called */
	gchar *reportfile;		/* name of xml file to run */
	gchar *reportfiletype;		/* pdf/ps etc */
	gchar *shell;			/* exec a shell command */
	gchar *apiwidgetname;		/* name of widget where api action 
					 * carried out on */
	gchar *sql;
	gchar *outputfile;		/* name of report output file */
	GList *argumentlist;		/* a list of OperationsNodeTriggers */
	GList *loadby;
	Bond *bond;			/* Used for parsing into sig_trigger */
	};

/* Structure built from XML file on operational behavour */
struct _OperationsNode
	{
	gchar *name;			/* name if any attached to node */
	gchar *widgetname;		/* widget name of widget that we are 
					 * contained in. ie window */
	gchar *nodename;                /* name on node that appears in the 
					 * xml */
	UiWidget *widget;		/* pointer to actual widget */
	UiWidgetType widgettype;	/* type of widget */
	GList *parent;			/* who do we rely on for information */
	GList *depend;			/* different to parent, who do i 
					 * depend on */
	GList *childrenwindow;		/* If there are any widgets that are 
					 * children that needed to be specially 
					 * delte with. This is normally NULL */ 
	gchar *field;			/* database field */
	gchar *table;			/* database table */
	gchar *dbobjectsrc;		/* name of the dbobject this belongs to 					 */
	gchar *dbobjectsrc_widget;	/* name of the dbobject widget this 
					 * belongs to */
	gchar *defaultvalue;		/* str it defaults to */
	gchar *nullvalue;		/* null value description (ie for 
					 * dropdown) */
	gchar *nulllistvalue;		/* if dropdown and empty list */
	gboolean allowednullvalue;	/* if null values ok */
	gboolean fixdups;		/* if a dropdown list where duplicates 
					 * have to be fixed */
	gboolean readonly;		/* if readonly field */
	gboolean saveoffset;		/* for static dropdown boxes */
	gboolean defaultvisible; 	/* if this widget defaults to been 
					 * visible or not */
	gchar *primary;			/* primary key value in the case its a 
					 * lookup */
	gchar *primary_table;		/* if there is a table attached to 
					 * primary key */
	gchar *nodetype;		/* nodetype, if entry box, dropdown, 
					 * lookup, etc */
	
	InputRestrict inputrestrict;	/* if its a field with input restricted 
					 * somehow. Input restrict  */

	GList *fieldlist;		/* fieldlist if one exists, this is for
					 * lookups */
	GList *tablelist;		/* tablelist, same length as fieldlist.
					 */
	GList *appendtextlist;		/* If you want to use some text between 
					 * fields in dropdown lookups */
	GList *triggerlist;		/* List of children triggers */
	GList *populatelist;		/* for static dropdown boxes */
	OperationsNodeDb *dbobj;	/* Datasource or a Look up database set 
					 * if specified */
	OperationsNode *parentnode;  	/* Node i am derived from */
	OperationsNode *nextnode;  
	OperationsNode *prevnode;
	GList *childrennode; 		/* all the children of this OprNode. 
					 * This is the actual children list */
	
	GList *variablestofree;		/* anything that mem_free is to be 
					 * called on. Garbage collection left 
					 * a bit late and all.  */
	};

/* More structures are defined in uicommonplugin.h */

gint bondxml_init(Bond *bond);
gint bondxml_cleanup(Bond *bond);
void bondxml_free(void *ptr);
gint bondxml_parse(Bond *bond, xmlNode * node, OperationsNode * oprparent);
OperationsNode *bondxml_getnode(Bond *bond, gchar * widgetname);
gchar *bondxml_getwidgetlongname(Bond *bond, gchar * widgetshortname);

void bondxml_debug(OperationsNode * node);
gint bondxml_test(Bond *bond, gint testnum);

#endif
