/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <glib.h>

#include "db.h"
#include "ui.h"
#include "bc.h"

#include "bond.h"
#include "dropdown.h"

static gboolean
dropdown_checkdups_internal(GList * values, gchar * str)
	{
	GList *walk;

	for (walk = g_list_first(values); walk; walk = walk->next)
		if (str != walk->data)
			if (IsEqual((walk->data), str))
				return TRUE;
	return FALSE;
	}

static void
dropdown_checkdups(GList * values)
	{
	GList *walk;
	gchar *str, *tmpstr;

	for (walk = g_list_first(values); walk; walk = walk->next)
		{
		str = walk->data;
		if (dropdown_checkdups_internal(values, str) == TRUE)
			{
			/* This is a memory leak but i dont know how to fix 
			 * it??. Luckly this code should never be called 
			 * and its a very small leak. */
			tmpstr = mem_strdup_printf("%s.", str);
			walk->data = tmpstr;
			}
		}
	}
/**
 * dropdown_getlookup:
 * @parentbobj: A dropdown BondObject.
 * 
 * 
 * 
 * Returns: A BondObject not to be freed.
 */
BondObject *
dropdown_getlookup(BondObject * parentbobj)
	{
	if (parentbobj->children)
		return parentbobj->children->data;
	return NULL;
	}

static void
dropdown_getlist_free(BondObject * bobj, GList * list)
	{
	g_list_free(list);
	}

static void
dropdown_freeelement(void *rowindex)
	{
	mem_free(rowindex);
	}

/* Creates a GList of every item in the dropdown box and adds it to a hash
 * table so it can be accessed. This builds the data from the populatelist
 * when it is used instead of lookup. */
static GList *
dropdown_getlist_poplist(BondObject * bobj)
	{
	gint offset = 0;
	gint *rowindex, *rowindexlist;
	gchar *key, *value;
	GList *walk, *retlist = NULL;
	gboolean addrecords = TRUE, saveoffset = TRUE;

	saveoffset = bobj->oprnode->saveoffset;
	if (bobj->primarylookuprowindex)
		addrecords = FALSE;
	else
		{
		bobj->primarylookuprowindex =
		    g_hash_table_new_full(g_int_hash, g_int_equal, NULL, 
					  dropdown_freeelement);
		bobj->primarylookupvalue = g_hash_table_new_full(g_str_hash, 
		     g_str_equal, NULL, dropdown_freeelement);
		bobj->primarylookupkey = g_hash_table_new_full(g_str_hash, 
		     g_str_equal, NULL, dropdown_freeelement);
		}
	rowindexlist = mem_calloc(sizeof(guint) * 
		    g_list_length(bobj->oprnode->populatelist));
	bobj->garbage = mem_addtogarbage(bobj->garbage, 15, 
		    dropdown_freeelement, rowindexlist);

	for (walk = g_list_first(bobj->oprnode->populatelist); walk != NULL;
	        walk = walk->next)
		{
		if (addrecords)
			{
			rowindex = mem_calloc(sizeof(guint));
			*rowindex = offset;
			value = mem_strdup(walk->data);

			if (saveoffset)
				key = mem_strdup_printf("%d", offset);
			else
				key = mem_strdup(walk->data);
			/* m.essage("adding %s %s = %s: %d", 
			 (gchar *)walk->data, key,value, offset); */
			/* debuglog(90,"Got item %s, key %s from
			   %s.%s",value,key,lookup->oprnode->table,
			    lookup->oprnode->field); */
			/* Store values hash table values for each item in 
			 * dropdown box */
			g_hash_table_insert(bobj->primarylookuprowindex, key, rowindex);
			g_hash_table_insert(bobj->primarylookupvalue, value, key);
			g_hash_table_insert(bobj->primarylookupkey, key, value);
			}
		/* add onto the dropdown list */
		retlist = g_list_prepend(retlist, walk->data);
		offset++;
		}
	return g_list_reverse(retlist);
	}

static GList *
dropdown_getlist_lookup(BondObject * lookup, gint offset)
	{
	static gchar *tmpstr = "";
	void **resultarray;
	gint *rowindexlist, *rowindex;
	gint rowcount, colcount, i, y;
	gchar *value, *key, useprimary = 0;
	GList *fieldlist = NULL, *retlist = NULL;

	if (lookup->oprnode->fieldlist)
		{
		errormsg("Feature not supported yet.");
		return NULL;
		}
	if (lookup->oprnode->field == NULL)
		{
		errormsg("No field or tables specified on widget %s",
			lookup->widgetname);
		return NULL;
		}
	if (value == NULL)
		value = tmpstr;

	/* Add primary field and requested fields to list that you want 
	 * to extract from. */
	if (lookup->oprnode->primary)
		{
		fieldlist = g_list_append(fieldlist, lookup->oprnode->primary);
		fieldlist = g_list_append(fieldlist, 
			lookup->oprnode->primary_table);
		useprimary = 1;
		}

	fieldlist = g_list_append(fieldlist, lookup->oprnode->field);
	fieldlist = g_list_append(fieldlist, lookup->oprnode->table);
	/* Get all the values */
	if (fieldlist == NULL)
		errormsg("No fields or primary fields specified on %s",
			lookup->widgetname);
	else
		resultarray = db_cache_extract(lookup->obj, 
			fieldlist, &rowcount, &colcount);
	/* create an int array of all the row offsets */
	rowindexlist = mem_calloc(sizeof(guint) * rowcount);
	rowindex = rowindexlist;
	/* add index list to garbage */
	lookup->garbage =
	    mem_addtogarbage(lookup->garbage, 15, 
		    dropdown_freeelement, rowindexlist);
	/* work out y. */
	if (useprimary)
		y = (rowcount * 2) - 2;
	else
		y = rowcount - 1;
	/* Start at end of the dropdown list and work forward to start
	 * of the list. rowcount is the total number of rows in the dropdown
	 * box to display. */
	for (i = rowcount - 1; i >= 0; i--)
		{
		/* rowindex is a moving pointer to your current position
		 * in the dropdown box. Offset is if there is a null
		 * value first in the list */
		*rowindex = i + offset;
		/* if there is no primary variable set use it. */
		if (useprimary == 0)
			{
			if (resultarray[y] == NULL)
				resultarray[y] = tmpstr;
			value = resultarray[y];
			key = resultarray[y];
			y = y - 1;
			}
		else 
			{
			if (resultarray[y + 1] == NULL)
				resultarray[y + 1] = tmpstr;
			value = resultarray[y + 1];
			key = resultarray[y];
			y = y - 2;
			}
		/* debuglog(90,"Got item %s, key %s from " "%s.%s
		   %d",value,key,lookup->oprnode->table,lookup->oprnode->field,
		   *rowindex); */
		/* Store lookup values hash table values for each item 
		 * in dropdown box */
		g_hash_table_insert(lookup->primarylookuprowindex, key, rowindex);
		g_hash_table_insert(lookup->primarylookupvalue, value, key);
		g_hash_table_insert(lookup->primarylookupkey, key, value);
		retlist = g_list_prepend(retlist, value);
		rowindex++;
		}
	mem_free(resultarray);
	g_list_free(fieldlist);
	return retlist;
	}

GList *
dropdown_getlist(BondObject * bobj)
	{
	BondObject *lookup;
	GList *retlist = NULL;
	gint numrows, oldrow, offset = 0;

	/* 	gint *rowindex; */
	gchar *value;

	g_assert(bobj);
	/* If its a static list with no content or lookup then just do a simple
	   parse */
	if (bobj->oprnode->populatelist)
		return dropdown_getlist_poplist(bobj);
	/* look for lookup compenent */
	lookup = dropdown_getlookup(bobj);
	if (lookup == NULL)
		{
		errormsg("No lookup is specified on dropdown box %s, can not "
		 "display it.", bobj->widgetname);
		return NULL;
		}
	if (lookup->obj == NULL)
		{
		errormsg("No database object associated with widget %s. "
			"Aborting widget drawing.", bobj->widgetname);
		return NULL;
		}
	/* free old hashs */
	if (lookup->primarylookuprowindex)
		g_hash_table_destroy(lookup->primarylookuprowindex);
	if (lookup->primarylookupvalue)
		g_hash_table_destroy(lookup->primarylookupvalue);
	if (lookup->primarylookupkey)
		g_hash_table_destroy(lookup->primarylookupkey);
	lookup->primarylookupvalue = NULL;
	lookup->primarylookupkey = NULL;
	lookup->primarylookuprowindex = NULL;

	if (bobj->oprnode->nullvalue 
	        && bobj->oprnode->allowednullvalue)
		offset = 1;
	   
	numrows = db_numrecord(lookup->obj);
	if (numrows > 0)
		{
		lookup->primarylookuprowindex =
		    g_hash_table_new(g_int_hash, g_int_equal);
		lookup->primarylookupvalue = g_hash_table_new(
			g_str_hash, g_str_equal);
		lookup->primarylookupkey = g_hash_table_new(g_str_hash, 
			g_str_equal);
		}
	debuglog(90, "Sql %s", lookup->obj->query);
	debuglog(90, "Returned %d/%d items in %s", numrows, lookup->obj->num,
	         lookup->obj->basetable);

	oldrow = lookup->obj->row;
	retlist = dropdown_getlist_lookup(lookup, offset);
	/* move back to the old position in the dbobject of the lookup table. */
	if (numrows > 0 && oldrow < numrows && oldrow >= 0)
		db_moveto(lookup->obj, oldrow);
	
	/* If you need to add a value null value into the top of the list */
	if (bobj->oprnode->nullvalue 
	        && bobj->oprnode->allowednullvalue)
		{
		value = bobj->oprnode->nullvalue;

		/* TODO LOOK AT THIS FURTHER */
		/*
		  rowindex = mem_calloc(sizeof(guint));
		  rowindex = 0;
		*/
		/* m.essage("adding null value into list %s",bobj->widgetname); */

		retlist = g_list_prepend(retlist, value);
		}
	return retlist;
	}

static void
dropdown_handleempty(BondObject * parentbobj)
	{
	BondObject *bobj;

	bobj = dropdown_getlookup(parentbobj);
	if (!bobj)
		{
		debuglog(30, "%s doesn't contain a lookup.", 
			parentbobj->widgetname);
		ui_setvalue(parentbobj->bond->uixml, parentbobj->widget, "");
		return ;
		}
	debuglog(90, "Handling empty list! %s", bobj->oprnode->nulllistvalue);
	if (bobj->oprnode->nulllistvalue)
		ui_setvalue(parentbobj->bond->uixml, parentbobj->widget,
		            bobj->oprnode->nulllistvalue);
	else
		ui_setvalue(parentbobj->bond->uixml, parentbobj->widget, "");
	}

static void
dropdown_handlenullvalue(BondObject * bobj, GList *poplist)
	{
	/* m.essage("handling null list! %s",bobj->widgetname); */
	if (bobj->oprnode->nullvalue)
		ui_setvalue(bobj->bond->uixml, bobj->widget, 
			bobj->oprnode->nullvalue);
	else
	if (bobj->oprnode->allowednullvalue == FALSE)
		{
		/* Need to be able to get the first position in a list 
		 * which currently isnt possible */
		ui_setvalue(bobj->bond->uixml, bobj->widget, 
			g_list_first(poplist)->data);
		}
	else
		ui_setvalue(bobj->bond->uixml, bobj->widget, "");
	}

gchar *
dropdown_getvalue_bykey(BondObject * bobj, gchar * key)
	{
	g_assert(bobj);
	if (!dropdown_isdynamic(bobj))
		{
		bobj = dropdown_getlookup(bobj);
		g_assert(bobj);
		if (bobj->primarylookupkey == NULL)
			return NULL;
		if (bobj->oprnode->primary == NULL)
			return key;
		}
	if (key == NULL)
		{
		warningmsg("Key is NULL in %s", bobj->widgetname);
		return NULL;
		}
	return g_hash_table_lookup(bobj->primarylookupkey, key);
	}

gchar *
dropdown_getkey_byvalue(BondObject * bobj, gchar * value)
	{
	g_assert(bobj);
	if (!dropdown_isdynamic(bobj))
		{
		bobj = dropdown_getlookup(bobj);
		g_assert(bobj);
		if (bobj->primarylookupvalue == NULL)
			return NULL;
		}
	if (value == NULL)
		{
		warningmsg("Value is NULL in %s", bobj->widgetname);
		return NULL;
		}
	return g_hash_table_lookup(bobj->primarylookupvalue, value);
	}

gint
dropdown_getrowindex_bykey(BondObject * bobj, gchar * key)
	{
	gint *row;

	bobj = dropdown_getlookup(bobj);
	g_assert(bobj);
	if (bobj->primarylookuprowindex == NULL)
		return -1;
	if (key == NULL)
		{
		warningmsg("Key is NULL in %s", bobj->widgetname);
		return -1;
		}
	row = g_hash_table_lookup(bobj->primarylookuprowindex, key);
	return *row;
	}

gint
dropdown_draw(BondObject * bobj, gchar * listkey, gboolean literefresh)
	{
	gchar *listvalue;
	GList *poplist;

	g_assert(bobj);
	/* m.essage("drawing %s", bobj->widgetname); */
	/* check if i need to repopulate */
	if (literefresh == FALSE)
		{
		poplist = dropdown_getlist(bobj);
		if (poplist)
			{
			/* tHiS IS very slow. Call at caution. hence it 
			 * only been called if defined in xml */
			if (bobj->oprnode->fixdups == TRUE)
				dropdown_checkdups(poplist);
			ui_dropdown_populate(bobj->bond->uixml, bobj->widget, poplist);
			dropdown_getlist_free(bobj, poplist);
			}
		else
			{
			ui_dropdown_clear(bobj->bond->uixml, bobj->widget);
			dropdown_handleempty(bobj);
			return -1;
			}
		}
	if (listkey)
		{
		listvalue = dropdown_getvalue_bykey(bobj, listkey);
		debuglog(90, "Setting value %s from %s", listvalue, listkey);
		if (listvalue)
			ui_setvalue(bobj->bond->uixml, bobj->widget, listvalue);
		else
			{
			/* If invalid record is been selected. 
			 * Ie selecting something that is not in the list */
			dropdown_handlenullvalue(bobj,poplist);
			}
		}
	else
		{
		debuglog(90, "No value to display for %s", bobj->widgetname);
		dropdown_handlenullvalue(bobj,poplist);
		}
	return 0;
	}
/**
 * dropdown_iscontrollingbox:
 * @bobj: BondObject to check.
 * 
 * A dropdown box consits of a lookup and a combo/dropdown box. The lookup
 * is a fictional widget which contains all the data.  This function works
 * out if this is the dropdown box, not the lookup.
 * 
 * Returns: %TRUE if @bobj is the actual dropdown box.
 */
gboolean
dropdown_iscontrollingbox(BondObject * bobj)
	{
	BondObject *lookup;
	
	if (bobj->widgettype != UiDropDown)
	     return FALSE;

	lookup = dropdown_getlookup(bobj);
	if (lookup == NULL)
		return TRUE;
	if (lookup->dbobjectsrc)
		return TRUE;
	return FALSE;
	}

gboolean
dropdown_isdynamic(BondObject * bobj)
	{
	if (bobj->oprnode->populatelist)
		return TRUE;
	return FALSE;
	}

/*void
  testdebug(void *key, void *value, void *ptr)
  {
  message("Key: %s Value :%s",key,value);
  }
  void
  testdebug1(void *key, void *value, void *ptr)
  {
  gint *ptr2;
  ptr2 = (gint*)value;
  message("b. Key: %s Value :%d",(gchar*)key,(gint)(*ptr2));
  }
  g_hash_table_foreach(lookup->primarylookupvalue,testdebug,NULL);
*/
gint
dropdown_changingrow(BondObject * bobj)
	{
	BondObject *lookup;
	gchar *key, *value;
	gint *row, retval;
	message("Changing row for %s",bobj->widgetname);
	g_assert(bobj);
	/* check if this is a special type of dropdown box that 
	 * controls movement in a dbobject. (ie has a dbobjectsrc 
	 * instead of its own dbobject) */
	bobj->ready = FALSE;
	lookup = dropdown_getlookup(bobj);
	if (!lookup || !lookup->dbobjectsrc)
		{
		debuglog(80,"No lookup present - aborting"); 
		return 0;
		}
	/* Get the key and offset row to be moved to in the dbobjectsrc */
	ui_getvalue(bobj->bond->uixml, bobj->widget, &value);
	key = dropdown_getkey_byvalue(bobj, value);
	g_assert(key);
	row = g_hash_table_lookup(lookup->primarylookuprowindex, key);
	debuglog(90,"Row postion returned of %d. Key is %s. "
		"Changing dbobject %s to this row",
		*row, key, lookup->obj->basetable);
	retval = db_moveto(lookup->obj, *row);
	bobj->ready = TRUE;
	return retval;
	}

void
dropdown_free(BondObject * bobj)
	{
	BondObject *lookup;

	lookup = dropdown_getlookup(bobj);
	if (!lookup)
		return ;

	if (lookup->primarylookupvalue)
		g_hash_table_destroy(lookup->primarylookupvalue);
	if (lookup->primarylookupkey)
		g_hash_table_destroy(lookup->primarylookupkey);
	if (lookup->primarylookuprowindex)
		g_hash_table_destroy(lookup->primarylookuprowindex);
	lookup->primarylookupvalue = NULL;
	lookup->primarylookupkey = NULL;
	lookup->primarylookuprowindex = NULL;
	}
