/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <glib.h>

#include "bc.h"
#include "db.h"
#include "ui.h"

#include "evaluate.h"
#include "bond.h"

gint
bond_evaluate_test(Bond * bond, BondObject * bobj)
	{
	GList *dependon;
	gchar str[] = "SELECT * FROM cardno WHERE cardno=<widget>Thiswidget</widget>";
	gchar *string;

	if (!bobj)
		{
		bond_spawn(bond, NULL, "mainwindow", NULL, FALSE);
		bobj = bond_getbyname(bond, "mainwindow");
		}

	string = bond_evaluate(bobj, str, &dependon);
	debugmsg("returned string is %s", string);
	return 0;
	}

void
bond_evaluate_freedependon(GList *tofree)
	{
	EvaluateDependOn *depend;
	GList *walk;
	for(walk=g_list_first(tofree);walk!=NULL;walk=walk->next)
		{
		depend = walk->data;
		if (depend->widget)
			mem_free(depend->widget);
		mem_free(depend);
		}
	g_list_free(tofree);
	}
	
static gchar *
bond_evaluate_widget(BondObject * parentbobj, gchar * content, GList **dependon)
	{
	BondObject *bobj;
	static gchar nullvalue[] = "NULL";
	gchar *retstr = NULL;
	EvaluateDependOn *depend;
	
	/* get widget and find value if drawn */
	bobj = bond_getbyname(parentbobj->bond, content);
	depend = mem_calloc(sizeof(EvaluateDependOn));
	depend->bobj = bobj;
	depend->type = DependWidget;
	if (bobj)
		depend->widget = bobj->widget;
	else
		depend->widget = ui_getwidget(parentbobj->bond->uixml,content);
	depend->widgetname = mem_strdup(content);
	
	*dependon = g_list_prepend(*dependon,depend);
	
	if (!bobj)
		{
		debuglog(40, "Failed to find widget %s", content);
		return NULL;
		}
	bond_save_getui(bobj, &retstr);
	if (retstr == NULL)
		retstr = nullvalue;
	return retstr;
	}

static gchar *
bond_evaluate_field(BondObject * bobj, gchar * content, GList **dependon)
	{
	errormsg("Not written yet");
	return NULL;
	}

static gboolean
bond_evaluate_checkifneeded(gchar * string)
	{
	gint i, slen;

	slen = strlen(string);

	for (i = 0; i < slen; i++)
		{
		if (string[i] == '<')
			return TRUE;
		}
	return FALSE;
	}

static gchar *
bond_evaluate_getsubstring(gchar * string, int startpos, int endpos)
	{
	static int i;
	static gchar retbuf[320];
	gchar *retstr = NULL;

	if (endpos - startpos > 300)
		return NULL;
	for (i = startpos; i < endpos; i++)
		retbuf[i - startpos] = string[i];
	retbuf[i - startpos] = 0;
	if (startpos < endpos)
		retstr = mem_strdup(retbuf);
	/* m.essage("returning %s",retstr); */
	return retstr;
	}

static gboolean
bond_evaluate_checktag(gchar * string, int startpos, int endpos, gchar ** retstr)
	{
	if (*retstr != NULL)
	     mem_free(*retstr);
	*retstr = bond_evaluate_getsubstring(string, startpos, endpos);

	/* m.essage("Tag: %s", *retstr); */
	if (!strcmp(*retstr, "widget") == 0)
		return TRUE;
	else if (!strcmp(*retstr, "field") == 0)
		return TRUE;

	return FALSE;
	}

static gchar *
bond_evaluate_tag(BondObject * bobj, gchar * string, gchar * tag, 
                  gint firsttagstart, gint firsttagend, gint lasttagstart, gint lasttagend,
				  GList **dependon)
	{
	gchar *retstr = NULL;
	gchar *content;

	content = bond_evaluate_getsubstring(string, firsttagend + 1, lasttagstart - 2);

	if (strcmp(tag, "widget") == 0)
		retstr = bond_evaluate_widget(bobj, content, dependon);
	else if (strcmp(tag, "field") == 0)
		retstr = bond_evaluate_field(bobj, content, dependon);
	if (retstr)
		retstr = mem_strdup(retstr);
	mem_free(content);
	return retstr;
	}

static gchar *
bond_evaluate_buildstr(gchar * string, gchar * evaluatestr, gint firsttagstart, gint lasttagend)
	{
	gchar *retstr = NULL;
	gint i, j, slen, slen_ev;

	firsttagstart--;
	lasttagend++;

	if (evaluatestr == NULL)
		{
		debuglog(90, "Evaluation of %s returned NULL", string);
		sleep(1);
		return mem_strdup(string);
		}
	slen = strlen(string);
	slen_ev = strlen(evaluatestr);
	/*m.essage("%c %c Are whats there at pos",string[firsttagstart],
	   string[lasttagend]); */
	retstr = mem_alloc(slen + slen_ev + 2);

	/* Do beginning of string. */
	for (i = 0; i < firsttagstart; i++)
		retstr[i] = string[i];
	j = i;
	/* Insert evaluation string bit. */
	for (i = 0; i < slen_ev; i++)
		retstr[i + j] = evaluatestr[i];
	j = j + i;
	/* Put rest of string on the end. */
	for (i = lasttagend; i < slen; i++)
		retstr[i + (j - lasttagend)] = string[i];
	retstr[i + (j - lasttagend)] = 0;
	return retstr;
	}

static gchar *
bond_evaluate_lookfortags(BondObject * bobj, gchar * string, GList **dependon)
	{
	gint i, slen;
	gint startcount = 0, nexttag = 0;
	gint firsttagstart = -1, firsttagend = -1, lasttagstart = -1, lasttagend = -1;
	gchar *tag1 = NULL, *tag2 = NULL;
	gchar *retstr = NULL;
	gchar *newstr, *evaluatestr;

	slen = strlen(string);

	for (i = 0; i < slen; i++)
		{
		if (string[i] == '<')
			startcount = i;
		if (startcount && string[i] == '>')
			{
			if (nexttag == 0 && tag1)
				{
				mem_free(tag1);
				tag1 = NULL;
				}

			if (nexttag == 1 && tag2)
				{
				mem_free(tag2);
				tag2 = NULL;
				}

			if (nexttag == 0 && 
			    bond_evaluate_checktag(string, startcount + 1,
				i, &tag1))
				{
				firsttagstart = startcount + 1;
				firsttagend = i;
				nexttag = 1;
				}
			else if (nexttag == 1 && 
				 bond_evaluate_checktag(string,startcount + 2,
				    i, &tag2))
				{
				lasttagstart = startcount + 2;
				lasttagend = i;
				nexttag = 0;
				if (strcmp(tag1, tag2) == 0)
					{
					evaluatestr = bond_evaluate_tag(bobj, string, tag1,
					                                firsttagstart, firsttagend, 
													lasttagstart, lasttagend, 
													dependon);
					/* m.essage("Buildstr of %s, firsttagstart %d, lasttagend %d",evaluatestr,firsttagstart, lasttagend); */
					newstr = bond_evaluate_buildstr(string, evaluatestr, 
													firsttagstart, lasttagend);
					mem_free(string);
					mem_free(evaluatestr);
					evaluatestr = NULL;
					string = newstr;
					/* Begin everything all over again */
					slen = strlen(string);
					startcount = 0;
					i = 0;
					continue;
					}
				}
			startcount = 0;
			}
		}
	retstr = string;
	if (tag1)
		mem_free(tag1);
	if (tag2)
		mem_free(tag2);
	return retstr;
	}

/**
 * bond_evaluate:
 * 
 * Use to evaluage <widget> and <field> calls inside sql statements.
 *
 */
gchar *
bond_evaluate(BondObject * bobj, gchar * string, GList **dependon)
	{
	gchar *retstr = NULL;
	*dependon = NULL;
	
	if (!bobj)
		return mem_strdup(string);
	if (!string)
		{
		warningmsg("NULL string passed");
		return NULL;
		}
	if (bond_evaluate_checkifneeded(string) == FALSE)
		{
		/* m.essage("returning earlier"); */
		return mem_strdup(string);
		}
	retstr = bond_evaluate_lookfortags(bobj, mem_strdup(string), dependon);
	/* m.essage("returnign %s",retstr); */
	return retstr;
	}
