/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <string.h>

#include "ui.h"
#include "db.h"

#include "postit.h"
#include "bond.h"
#include "bondxml.h"
#include "bondtrigger.h"
#include "dropdown.h"
#include "watchtower.h"
#include "biglists.h"

gint imbusygoaway;

static gint
postit_sig_create(BondObject * bobj, void *ptrindex, void *(*argfunc) (),
                  gchar * event)
	{
	gint retval = 0;
	GList *walk, *tofree = NULL;
	SignalId *sigid;

	if (!bobj->widget)
		return -1;
	/* Remove any existing signals */
	for (walk = bobj->signallist; walk != NULL; walk = walk->next)
		{
		sigid = walk->data;
		if (sigid->ptrindex == ptrindex)
			{
			/* m.essage("disconnect signal on %d",sigid->signalid); */
			ui_signaldisconnect(bobj->bond->uixml, bobj->widget,
			                    sigid->signalid);
			sigid->signalid = 0;
			tofree = g_list_prepend(tofree, sigid);
			}
		}
	/* Free signal list */
	if (tofree)
		for (walk = g_list_first(tofree); walk != NULL; 
			walk = walk->next)
			{
			bobj->signallist = 
				g_list_remove(bobj->signallist, tofree->data);
			mem_free(walk->data);
			retval++;
			}
	/* Create new signal */
	sigid = mem_calloc(sizeof(SignalId));
	sigid->ptrindex = ptrindex;
	sigid->signalid = ui_signalconnect(bobj->bond->uixml, bobj->widget,
	                                   event, argfunc, ptrindex);
	/* m.essage("Signal created on %d",sigid->signalid);  */
	/* Store the signal id */
	bobj->signallist = g_list_prepend(bobj->signallist, sigid);

	return retval;
	}

/* Closes a form and if all forms have been closed, quits the program */
static gboolean
postit_sig_formclose(UiWidget * widget, void *ignore, void *bobj)
	{
	/*debuglog(80, "A delete event occured in %s which I am ignoring.",
	   ((BondObject *)bobj)->widgetname); */ 
	bond_close((BondObject *) bobj, TRUE);
	
	return TRUE;
	}

static void
postit_sig_form_notvisible(void *key_widgetname, void *value_bobj, gpointer ptr)
	{
	BondObject *bobj;
	int *windowsvisible;

	windowsvisible = ptr;
	bobj = (BondObject *) value_bobj;
	if (bobj && bobj->widget)
		{
		if (ui_gettype(bobj->bond->uixml, bobj->widget) == UiForm)
			if (ui_isvisible(bobj->bond->uixml, bobj->widget) == TRUE)
				*windowsvisible = TRUE;
		}
	}

/**
 * postit_checkforclosedwindows:
 * @bobj: Bondobject of no consequence.
 *
 * Check to see if any windows are still open. If nothing is open then
 * quit from the application.
 *
 * returns: %TRUE if there is something visible else %FALSE and in that
 * case it doesnt really matter cause your app is terminating anyway.
 */    
gboolean
postit_checkforclosedwindows(BondObject *bobj)
	{
	int windowsvisible = FALSE;

	/* If no forms are visible */
	g_hash_table_foreach(((BondObject *) bobj)->bond->globalbobj,
	                     postit_sig_form_notvisible, &windowsvisible);
	/* then quit the program... */
	if (windowsvisible == FALSE)
		ui_mainquit(((BondObject *) bobj)->bond->uixml);
	return windowsvisible;
	}

static void
postit_sig_formclose_create(BondObject * bobj)
	{
	postit_sig_create(bobj, bobj, (void *)postit_sig_formclose, 
			"delete_event");
	}

static void
posit_sig_focusout_modifycase(gchar * str, InputRestrict ir)
	{
	gint i, slen;

	slen = strlen(str);
	switch (ir)
		{
	case InputRestrictToUpper:
		for (i = 0; i < slen; i++)
			str[i] = g_ascii_toupper(str[i]);
		break;
	case InputRestrictToLower:
		for (i = 0; i < slen; i++)
			str[i] = g_ascii_tolower(str[i]);
		break;
	case InputRestrictFirstToUpper:
		for (i = 0; i < slen; i++)
			if (i == 0 || str[i - 1] == ' ')
				str[i] = g_ascii_toupper(str[i]);
		break;
	default:
		errormsg("Unknown type here");
		break;
		}
	}

static void
postit_sig_changed(UiWidget * widget, BondObject * bobj)
	{
	gchar *tmpvalue = NULL;

	if (imbusygoaway == 1)
		{
		/* errormsg("WHAT DO YOU THINK YOUR DOING???"); */
		return ;
		}
	if (bobj->ready == FALSE)
		return ;

	imbusygoaway = 1;
	if (bobj->widgettype == UiDropDown)
		{
		/* Bug workaround for gtk bug */
		ui_getvalue(bobj->bond->uixml, bobj->widget, &tmpvalue);
		if (strlen(tmpvalue) == 0)
			{
			imbusygoaway = 0;
			return ;
			}
		}
	debuglog(35, "Widget changed %s", bobj->widgetname);
	bond_setchanged(bobj, TRUE, NULL);
	/* Register a dropdown object change */
	/* This code is not actually called normally, and bond_setchanged() 
	 * handles all the dropdown cross dependcy stuff, not this function as 
	 * it may seem. These lines should proberly be removed at some point. */
	/* This is if you have a dropdown box as a control widget to choose 
	 * what field you currently have selected. Like a search list of 
	 * peoples names you can scroll though */
	/*if (bobj->widgettype == UiDropDown)
		if (dropdown_iscontrollingbox(bobj))
			bond_registerobjchange(bobj, dropdown_changingrow, TRUE); */
	imbusygoaway = 0;
	}

/* Check if the key press is ok. Returns: 1 if keypress is invalid, else 0
 * if value.  */
static gint
postit_sig_changed_checkkey(void *ptr, gchar * keypress)
	{
	gint retval=0;
	BondObject *bobj = ptr;

	if (imbusygoaway == 1)
		{
		/* errormsg("WHAT DO YOU THINK YOUR DOING???\n"
		 "stop typing so fast."); */
		return -1;
		}

	mem_verify(bobj);
	g_assert(keypress);
	switch (bobj->oprnode->inputrestrict)
		{
	case InputRestrictInt:
		/* 127 is DEL, sub 32 is non-alpha numeric keys like ESC, 
		 * tab etc */
		if ((keypress[0] >= '0' && keypress[0] <= '9') ||
		        keypress[0] == '-' || keypress[0] == 127)
			return 0;
		else if (keypress[0] >= 32)
			{
			debugmsg("Invalid key press %s", keypress);
			return 1;
			}
		break;
	case InputRestrictFloat:
		/* 127 is DEL, sub 32 is non-alpha numeric keys like ESC, tab */
		if ((keypress[0] >= '0' && keypress[0] <= '9')
		        || keypress[0] == '.' || keypress[0] == '-'
		        || keypress[0] == 127)
			return 0;
		else if (keypress[0] >= 32)
			{
			debugmsg("Invalid key press %s", keypress);
			return 1;
			}
		break;
	case InputRestrictDate:
	case InputRestrictTime:
		break;
	case InputRestrictNone:
		break;
	case InputRestrictString:
	case InputRestrictToUpper:
	case InputRestrictToLower:
		posit_sig_focusout_modifycase(keypress,
					      bobj->oprnode->inputrestrict);
		break;
	case InputRestrictFirstToUpper:
		break;
	case InputRestrictPhone:
		if ((keypress[0] >= '0' && keypress[0] <= '9')
		        || keypress[0] == '-' || keypress[0] == ' '
		        || keypress[0] == '(' || keypress[0] == ')'
		        || keypress[0] == '+' || keypress[0] == 127)
			return 0;
		else if (keypress[0] >= 32)
			{
			debugmsg("Invalid key press %s", keypress);
			return 1;
			}
		break;
	case InputRestrictAll:
		return 1;
	default:
		break;
		}
	return retval;
	}

static void
postit_sig_changed_create(BondObject * bobj)
	{
	UiWidget *widget;

	switch (bobj->widgettype)
		{
	case UiNull:
	case UiGridColumn:
		break;
	case UiGrid:
		ui_signaldisconnectbyfunc(bobj->bond->uixml, bobj->widget,
		                          postit_sig_rowselect, (void *)bobj);
		ui_signalconnect(bobj->bond->uixml, bobj->widget, "changed",
		                 (void *)postit_sig_rowselect, (void *)bobj);
		break;		
	case UiEntryBox:
	case UiSpinButton:
	case UiTextBox:
		ui_signaldisconnectbyfunc(bobj->bond->uixml, bobj->widget,
		                          postit_sig_changed, (void *)bobj);
		ui_signalconnect(bobj->bond->uixml, bobj->widget, "changed",
		                 (void *)postit_sig_changed, (void *)bobj);
		if (bobj->oprnode->inputrestrict != InputRestrictNone)
			{
			/* Disconnect older functions */
			ui_signaldisconnectbyfunc(bobj->bond->uixml,
					bobj->widget,
					(void *)postit_sig_focusout,
			                (void *)bobj);
			ui_signaldisconnect_keypress(bobj->bond->uixml,
			                bobj->widget, bobj->uisignalkey);
			bobj->uisignalkey = NULL;
			/* Connect new functions */
			ui_signalconnect(bobj->bond->uixml, bobj->widget,
			                 "focus_out_event",
			                 (void *)postit_sig_focusout, 
					 (void *)bobj);
			bobj->uisignalkey =
			    ui_signalconnect_keypress(bobj->bond->uixml, 
				bobj->widget, postit_sig_changed_checkkey,
			        (void *)bobj);
			}
		break;
	case UiDropDown:
		/* m.essage("Dropdown box create postit_sig_create()"); */
		widget = bobj->widget;
		/* widget = ui_dropdown_getentrybox(bobj->bond->uixml,
		 bobj->widget); */
		ui_signaldisconnectbyfunc(bobj->bond->uixml, widget,
		                          postit_sig_changed, (void *)bobj);
		ui_signalconnect(bobj->bond->uixml, widget, "changed",
		                 (void *)postit_sig_changed, (void *)bobj);
		break;
	case UiCheckButton:
	case UiToggleButton:
	case UiRadioButton:
		ui_signaldisconnectbyfunc(bobj->bond->uixml, bobj->widget,
		                          postit_sig_changed, (void *)bobj);
		ui_signalconnect(bobj->bond->uixml, bobj->widget, "toggled",
		                 (void *)postit_sig_changed, (void *)bobj);
		break;
	default:
		errormsg("Can't create signal on widget %s, not coded yet.",
		         bobj->widgetname);
		}
	}

/**
 * postit_initsignals:
 * @bobj: Bond Object to initalise.
 * 
 * Initialise all the callback signals on @bobj.  This will create the forum
 * close callback, any set up callbacks for all triggers on the widget.  
 * It creates the field changed call backs also for postit_sig_changed() 
 * if the widget is active.
 * 
 * Returns: non-zero on error.
 */
gint
postit_initsignals(BondObject * bobj)
	{
	GList *walk;
	OperationsNodeTrigger *trigger;

	if (bobj->signalsset == TRUE)
		return -1;
	/* Create change signal */
	if (bond_isfieldactive(bobj))
		postit_sig_changed_create(bobj);
	/* If form protect it against deleting */
	if (bobj->widgettype == UiForm)
		postit_sig_formclose_create(bobj);
	/* Create trigger callback signals, complex bit */
	for (walk = g_list_first(bobj->oprnode->triggerlist); walk != NULL;
	        walk = walk->next)
		{
		trigger = walk->data;
		if (trigger->widgetname == NULL)
			continue;
		/* m.essage("Creating trigger from %s", 
		 bobj->oprnode->widgetname); */
		postit_sig_trigger_create(bobj, trigger);
		}

	bobj->signalsset = TRUE;
	return 0;
	}

/**
 * postit_sig_focusout:
 * @widget: Widget which focus leaves from.
 * @event: A void* 
 * @bobj: Bond Object for @widget.
 * 
 * Check on restricted entry widgets that the value typed in on field exit
 * is fine and ok to use.  Also seek enlightenment by god at the same time.
 * 
 * Returns: %FALSE normally.
 */
gboolean
postit_sig_focusout(UiWidget * widget, UiEvent * event, BondObject * bobj)
	{
	gchar datestr[40] = "";
	gchar *tmpstr, *buffer;
	GDate *d;

	if (imbusygoaway == 1)
		{
		return FALSE;
		}

	if (!widget || !bobj)
		{
		errormsg("focus_out_event called without relevant "
		         "information passed.");
		return FALSE;
		}
	imbusygoaway = 1;

	switch (bobj->oprnode->inputrestrict)
		{
	case InputRestrictInt:
	case InputRestrictFloat:
		if (bobj->oprnode->allowednullvalue == FALSE &&
		        bobj->widgettype != UiDropDown)
			{
			ui_getvalue(bobj->bond->uixml, widget, &tmpstr);
			buffer = mem_strdup(tmpstr);
			if (strlen(tmpstr) == 0)
				if (bobj->oprnode->allowednullvalue == FALSE)
					ui_setvalue(bobj->bond->uixml, 
							widget, "0");
			mem_free(buffer);
			}
		break;
	case InputRestrictDate:
	case InputRestrictTime:
		d = g_date_new();
		ui_getvalue(bobj->bond->uixml, widget, &tmpstr);
		g_date_set_parse(d, tmpstr);
		if (!g_date_valid(d))
			{
			ui_setvalue(bobj->bond->uixml, widget, datestr);
			g_date_free(d);
			imbusygoaway = 0;
			return FALSE;
			}
		if (bobj->oprnode->inputrestrict == InputRestrictTime)
			g_date_strftime(datestr, 39, "%R", d);
		else
			g_date_strftime(datestr, 39, "%F", d);
		ui_setvalue(bobj->bond->uixml, widget, datestr);
		g_date_free(d);
		break;
	case InputRestrictNone:
	case InputRestrictString:
		break;
	case InputRestrictPhone:
		break;
	case InputRestrictToUpper:
	case InputRestrictToLower:
	case InputRestrictFirstToUpper:
		ui_getvalue(bobj->bond->uixml, widget, &tmpstr);
		buffer = mem_strdup(tmpstr);
		if (strlen(tmpstr) > 0)
			{
			posit_sig_focusout_modifycase(buffer,
				bobj->oprnode->inputrestrict);
			ui_setvalue(bobj->bond->uixml, widget, buffer);
			}
		mem_free(buffer);
		break;
	case InputRestrictAll:
	default:
		break;
		}
	imbusygoaway = 0;
	return FALSE;
	}

/**
 * postit_sig_rowselect:
 * @widget: Widget that the event occured in.
 * @bobj: Bondobject thats passed though with the signal, this
 * is the @widget BondObject.
 * 
 * This is executed when a row in a list is selected.
 * 
 */
void
postit_sig_rowselect(UiWidget * widget, BondObject * bobj)
	{
	g_assert(widget);
	g_assert(bobj);
	g_assert(bobj->widgetname);
	if (bobj->ready == FALSE)
		return ;
	if (imbusygoaway == 1)
		return ;
	debuglog(30, "Row in %s selected", bobj->widgetname);
	bond_registerobjchange(bobj, biglist_selectrow, TRUE);
	}

/**
 * postit_sig_trigger_create:
 * @bobj: Bond object
 * @trigger: Trigger generated from bondxml document.
 *
 * Create triggers for callbacks.
 */
gint
postit_sig_trigger_create(BondObject * bobj, OperationsNodeTrigger * trigger)
	{
	gchar *event = NULL;
	void *ptr;

	g_assert(bobj);
	g_assert(trigger);
	g_assert(trigger->bond);
	/* get widget to place signal on. If dropdown place on entry box */
	/* if (bobj->widgettype != UiDropDown)
		widget = bobj->widget;
	else
		widget = ui_dropdown_getentrybox(bobj->bond->uixml, bobj->widget); */
	/* work out what type of signal to place */
	switch (trigger->event)
		{
	case EventNone:
		break;
	case EventChange:
		event = mem_strdup("changed");
		break;
	case EventEntry:
		event = mem_strdup("focus_in_event");
		break;
	case EventExit:
		event = mem_strdup("focus_out_event");
		break;
	case EventClick:
		if (bobj->widgettype == UiMenuItem)
			event = mem_strdup("activate");
		else
			event = mem_strdup("clicked");
		break;
	case EventSelect:
		/* Need to change to fix bug. 2005-01-16. was changed */
		event = mem_strdup("changed");
		break;
	default:
		break;
		}
	/* handle key press if attached also */

	/* create the signals */
	if (event)
		{
		ptr = trigger;
		debuglog(50, "I'm creating a trigger signal on %s on event %s.",
		         bobj->widgetname, event);
		postit_sig_create(bobj, ptr, (void *)postit_sig_trigger, event);
		mem_free(event);
		return 0;
		}
	return -1;
	}

/**
 * postit_sig_trigger:
 * @widget: Widget that the trigger is to be applied to.
 * @trigger: The trigger defination defined in the bond xml document.
 * 
 * This excutes a trigger on a widget callbacks.
 */
void
postit_sig_trigger(UiWidget * widget, OperationsNodeTrigger * trigger)
	{
	gchar *widgetname;
	Bond *bond;
	BondObject *bobj;

	g_assert(trigger);
	g_assert(trigger->bond);

	bond = trigger->bond;
	widgetname = trigger->widgetname;
	/* m.essage("wname: %s, 2nd: %s", widgetname, ui_getname(bond->uixml,
	   widget)); */
	if (!widgetname)
		widgetname = ui_getname(bond->uixml, widget);
	g_assert(widgetname);
	bobj = bond_getbyname(bond, widgetname);
	g_assert(bobj);
	if (!bobj->ready)
		return ;
	if (bobj->widgettype == UiDropDown)
		{
		gchar *tmpvalue = NULL;
		/* Bug workaround for gtk bug */
		ui_getvalue(bobj->bond->uixml, bobj->widget, &tmpvalue);
		if (strlen(tmpvalue) == 0)
			return ;
		}
	bondtrigger_action(bobj, trigger);
	}

/**
 * postit_sig_disconnect:
 * @bobj: BondObject which has a widget associated with it.
 * 
 * Discounnect any signals that maybe attached to @bobj. This is done when
 * a @bobj is freed.
 * 
 * Returns: Non-zero on error.
 */
gint
postit_sig_disconnect(BondObject * bobj)
	{
	GList *walk;
	SignalId *sigid;

	g_assert(bobj);
	if (!bobj->widget)
		return -1;

	/* Remove any existing signals */
	for (walk = bobj->signallist; walk != NULL; walk = walk->next)
		{
		sigid = walk->data;
		/* m.essage("freeing signalid %d on
		   %s",sigid->signalid,bobj->widgetname); */
		ui_signaldisconnect(bobj->bond->uixml, bobj->widget, sigid->signalid);
		mem_free(sigid);
		}
	if (bobj->signallist)
		g_list_free(bobj->signallist);

	switch (bobj->widgettype)
		{
	case UiGrid:
		ui_signaldisconnectbyfunc(bobj->bond->uixml, bobj->widget,
		                          (void *)postit_sig_rowselect,
		                          (void *)bobj);
		break;
	case UiEntryBox:
	case UiSpinButton:
	case UiTextBox:
		ui_signaldisconnectbyfunc(bobj->bond->uixml, bobj->widget,
		                          postit_sig_changed, (void *)bobj);
		if (bobj->oprnode->inputrestrict != InputRestrictNone)
			{
			ui_signaldisconnectbyfunc(bobj->bond->uixml, bobj->widget,
			                          (void *)postit_sig_focusout,
			                          (void *)bobj);
			ui_signaldisconnect_keypress(bobj->bond->uixml, bobj->widget,
			                             bobj->uisignalkey);
			bobj->uisignalkey = NULL;
			}

		break;
	case UiDropDown:
/*		widget = ui_dropdown_getentrybox(bobj->bond->uixml, bobj->widget); */
		ui_signaldisconnectbyfunc(bobj->bond->uixml, bobj->widget,
		                          postit_sig_changed, (void *)bobj);
		break;
	case UiCheckButton:
	case UiToggleButton:
	case UiRadioButton:
		ui_signaldisconnectbyfunc(bobj->bond->uixml, bobj->widget,
		                          postit_sig_changed, (void *)bobj);
		break;
	default:
		break;
		}
	return 0;
	}

gint
postit_sig_blockemit(UiXML *uixml, gint state)
	{
	if (state != -1)
	     imbusygoaway = state;
	return imbusygoaway;
	}
