/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <glib.h>

#include "db.h"
#include "bc.h"

#include "bond.h"
#include "bondmain.h"
#include "watchtower.h"

int reg_totalreg;

/* I dont trust g_list_copy. It does weird weird things I'm so telling u. ??>*/

static gint watchtower_checkregistraint(GlobalRegistor * globalreg,
                                        Registration * reg);
static void watchtower_free(GlobalRegistor * globalreg, WatchTower * watch);
static GList *watchtower_check_bywatch(GlobalRegistor * globalreg,
                                       WatchTower * watch);
static void registration_free(GlobalRegistor * globalreg, Registration * reg);

void
watchtower_cleanup(Bond * bond)
	{
	warningmsg("write me!");
	}

/**
 * watch_test:
 * 
 * Internal tests. not functional.
 */
gint
watch_test(gint testnum, BondObject * bobj1, BondObject * bobj2,
           BondObject * bobj3)
	{
	gint retval = 0;

	switch (testnum)
		{
	case 0:
		break;
	default:
		break;
		}
	return retval;
	}

/**
 * watchtower_create:
 * @watching: BondObject that is going to do the watching.
 * @runonchange: Function to excute when watch has been successful.
 * 
 * Creates a watch and adds it to its big list.
 * 
 * 
 * Returns: Newly created watchtower.
 */

WatchTower *
watchtower_create(BondObject * watching, void (*runonchange) (void *, void *))
	{
	WatchTower *watch;

	g_assert(watching);
	watch = mem_calloc(sizeof(WatchTower));
	watch->watching = watching;
	watch->runonchange = runonchange;

	return watch;
	}

void
watchtower_set_field(WatchTower * watch, gchar * table, gchar * field)
	{
	g_assert(watch);
	watch->fieldname = field;
	watch->tablename = table;
	}

/**
 * watchtower_run:
 * @watch: Watch that is calling
 * @bobj: bobj something that triggered the run (ie where you calling from)
 * 
 * Excute a watchter thats associated with @bobj.
 */
gint
watchtower_run(WatchTower * watch, BondObject * bobj)
	{
	/* save back any temporary data first if bobj set */
	if (bobj && bobj->changed == TRUE && bobj->obj)
		{
		/* do setnomark save on bobj */
		bond_save(bobj, FALSE, FALSE);
		}
	/* run the change filter if its present */
	if (watch->runonchange)
		watch->runonchange(watch, bobj);
	else
		return -2;
	return 0;
	}

static gchar *
watchtower_register_getfield(WatchTower * watch)
	{
	if (watch->fieldname && watch->tablename)
		return mem_strdup_printf("%s.%s", watch->tablename, watch->fieldname);
	return NULL;
	}

static void
watchtower_hash_free_key(void *ptr)
	{
	mem_free(ptr);
	}
	
static void
watchtower_hash_free(void *ptr)
	{
	g_list_free(ptr);
	}

static void
watchtower_hash_init(GlobalRegistor * reg)
	{
	reg->watchhashbykey = g_hash_table_new_full(g_str_hash, g_str_equal,
	                NULL, watchtower_hash_free);
	reg->watchhashbyfield = g_hash_table_new_full(g_str_hash, g_str_equal,
	                watchtower_hash_free_key, watchtower_hash_free);
	reg->watchhashbywidget = g_hash_table_new_full(g_direct_hash, g_direct_equal,
	                NULL, watchtower_hash_free);
	reg->watchhashbydbobjectsrc = g_hash_table_new_full(g_str_hash, 
					g_str_equal,NULL,watchtower_hash_free);
	}

/* union of 2 lists, return only items that are in common */
static GList *
watchtower_check_unionlist(GList * list1, GList * list2)
	{
	GList *walk1, *walk2, *retlist = NULL;

	if (list2 == NULL)
		return list1;
	if (list1 == NULL)
		return g_list_copy(list2);
	for (walk1 = g_list_first(list1); walk1 != NULL; walk1 = walk1->next)
		for (walk2 = g_list_first(list2); walk2 != NULL; 
				walk2 = walk2->next)
			if (walk2->data == walk1->data)
				{
				retlist = g_list_prepend(retlist, walk1->data);
				break;
				}
	g_list_free(list1);
	return retlist;
	}

/* merge of 2 lists, return all items in common */
static GList *
watchtower_check_mergelist(GList * list1, GList * list2)
	{
	GList *walk1, *retlist = NULL;

	if (list2 == NULL)
		return list1;
	if (list1 == NULL)
		return g_list_copy(list2);
	retlist = g_list_copy(list2);
	for (walk1 = g_list_first(list1); walk1 != NULL; walk1 = walk1->next)
		{
		retlist = g_list_remove(retlist, walk1->data);
		retlist = g_list_prepend(retlist, walk1->data);
		}
	g_list_free(list1);
	return retlist;
	}

/* g_list_free result afterwards */
static GList *
watchtower_check_bywatch(GlobalRegistor * globalreg, WatchTower * watch)
	{
	GList *list1 = NULL, *list2 = NULL;
	gchar *key;

	if (watch->fieldname && watch->tablename)
		{
		key = watchtower_register_getfield(watch);
		if (key)
			{
			list2 = g_hash_table_lookup(globalreg->reghashbyfield, key);
			mem_free(key);
			list1 = watchtower_check_unionlist(list1, list2);
			}
		}
	if (watch->widget)
		{
		list2 = g_hash_table_lookup(globalreg->reghashbywidget, watch->widget);
		list1 = watchtower_check_unionlist(list1, list2);
		}
	if (watch->key)
		{
		list2 = g_hash_table_lookup(globalreg->reghashbykey, watch->key);
		list1 = watchtower_check_unionlist(list1, list2);
		}
	if (watch->dbobjectsrc)
		{
		list2 = g_hash_table_lookup(globalreg->reghashbydbobjectsrc,
		                            watch->dbobjectsrc);
		list1 = watchtower_check_unionlist(list1, list2);
		}
	return list1;
	}

/** and right about now the real fear sets in. Oh no make it stop............
 line goes dead.
 
 damit.
segfault.
 */

static GList *
watchtower_check_verify(GList * list1, Registration * reg)
	{
	gint pass, count;
	gchar *key1, *key2;
	WatchTower *watch;
	GList *walk, *retlist = NULL;

	for (walk = g_list_first(list1); walk != NULL; walk = walk->next)
		{
		pass = 0;
		count = 0;
		watch = walk->data;
		if (watch->fieldname && reg->fieldname)
			{
			key1 = mem_strdup_printf("%s.%s", watch->tablename,
				watch->fieldname);
			key2 = mem_strdup_printf("%s.%s", reg->tablename, 
				reg->fieldname);
			count++;
			if (strcmp(key1, key2) == 0)
				pass++;

			mem_free(key1);
			mem_free(key2);
			}
		if (watch->dbobjectsrc && reg->dbobjectsrc)
			{
			count++;
			if (strcmp(watch->dbobjectsrc, reg->dbobjectsrc) == 0)
				pass++;
			}
		if (watch->widget && reg->registraint->widget)
			{
			count++;
			if (watch->widget == reg->registraint->widget)
				pass++;
			}
		if (watch->key && reg->key)
			{
			count++;
			if (strcmp(watch->key, reg->key) == 0)
				pass++;
			}
		if (count == pass)
			retlist = g_list_prepend(retlist, watch);
		}

	return retlist;
	}

/* g_list_free result afterwards */
static GList *
watchtower_check_byregistration(GlobalRegistor * globalreg, Registration * reg)
	{
	GList *list1 = NULL, *list2 = NULL;
	gchar *key;

	if (reg->fieldname && reg->tablename)
		{
		key = mem_strdup_printf("%s.%s", reg->tablename, reg->fieldname);
		list2 = g_hash_table_lookup(globalreg->watchhashbyfield, key);
		list1 = watchtower_check_mergelist(list1, list2);
		mem_free(key);
		}
	if (reg->registraint->widget != NULL)
		{
		list2 = g_hash_table_lookup(globalreg->watchhashbywidget,
		                            reg->registraint->widget);
		list1 = watchtower_check_mergelist(list1, list2);
		}
	if (reg->key != NULL)
		{
		list2 = g_hash_table_lookup(globalreg->watchhashbykey, reg->key);
		list1 = watchtower_check_mergelist(list1, list2);
		}
	if (reg->dbobjectsrc != NULL)
		{
		list2 = g_hash_table_lookup(globalreg->watchhashbydbobjectsrc,
		                            reg->dbobjectsrc);
		list1 = watchtower_check_mergelist(list1, list2);
		}
	if (list1)
		list1 = watchtower_check_verify(list1, reg);
	return list1;
	}

static gint
watchtower_checkregistraint(GlobalRegistor * globalreg, Registration * reg)
	{
	WatchTower *watch;
	GList *affected, *walk;
	gint retval = 0;

	affected = watchtower_check_byregistration(globalreg, reg);
	for (walk = g_list_first(affected); walk != NULL; walk = walk->next)
		{
		watch = walk->data;
		debuglog(50, "IM BEEN WATCHED!! %s watched by %s",
		         reg->registraint->widgetname, watch->watching->widgetname);
		watchtower_run(watch, reg->registraint);
		retval = 1;
		/* watchtower_debug(); registration_debug(reg); */
		}
	if (affected)
		g_list_free(affected);
	return retval;
	}
	
/* You need to free the key if 1 is returned */
static gint
watchtower_hash_insert(GHashTable * hash, void *key, void *ptr)
	{
	GList *watchlist;

	if (!ptr || !key)
		{
		errormsg("Can't add to hashlist. NULL pointer.");
		return -1;
		}
		
	/* See if they key has already been registered. */
	watchlist = g_hash_table_lookup(hash, key);
	if (watchlist == NULL)
		{
		/* if first time insert into the hash table a list */
		watchlist = g_list_append(NULL, ptr);
		g_hash_table_insert(hash, key, watchlist);
		return 0;
		}
	else 	/* Append to end of list of key */
		watchlist = g_list_append(watchlist, ptr);
		
	return 1;
	}

static gint
watchtower_hash_remove(GHashTable * hash, void *key, void *ptr,
                       gboolean copykey)
	{
	void *newkey;
	GList *watchlist, *newlist, *walk;
	
	/* get a GList* of all items registered under this hash */
	watchlist = g_hash_table_lookup(hash, key);
	if (watchlist == NULL)
		{
		/* This fails if there is a record of ptr but not under  key */
		/* debuglog(60,"nothing is registered on %s.",(gchar*)key); */
		return -1;
		}
	/* one item to free. Keep it simple */
	if (g_list_length(watchlist) == 1)
		{
		/* Does a g_list_free() as there is only one item in the list. */
		if (g_hash_table_remove(hash, key) == FALSE)
			warningmsg("Failed to destroy hash on key %s",(gchar*)key);
		return 0;
		}
	/* If first item is that norty one that needs removing
	 * then rebuild the list cause you know your screwed */
	if (watchlist->data == ptr)
		{
		newlist = NULL;
		watchlist = watchlist->next;
		newkey = key;
		/* make a copy of the key if the hash table removal automatically 
		 * removes it */
		if (copykey)
			newkey = mem_strdup_printf("%s", (gchar *) key);

		for (walk = watchlist; walk != NULL; walk = walk->next)
			newlist = g_list_append(newlist, walk->data);

		g_hash_table_remove(hash, key);
		g_hash_table_insert(hash, newkey, newlist);
		}
	else
		{
		g_list_remove(watchlist, ptr);
		}

	return 0;
	}
/**
 * watchtower_register:
 * @watch: Watchtower to added to the registery 
 * (after watchtower_create is called)
 * 
 * 
 * 
 * Returns: A list of registrations currently effected by the watchi
 * (these are called in which case)
 */
GList *
watchtower_register(WatchTower * watch)
	{
	GList *affected, *walk;
	gchar *field;
	Registration *reg;
	GlobalRegistor *globalreg;
    
	g_assert(watch);
	g_assert(watch->watching);
	globalreg = watch->watching->bond->globalreg;
	g_assert(globalreg);

	watch->watching->watchtowerlist =
	    g_list_prepend(watch->watching->watchtowerlist, watch);

	field = watchtower_register_getfield(watch);
	if (watch->key == NULL)
		{
		warningmsg("No key yet on %s so i can't create a watch"
			"yet for events associated with it.",
			watch->watching->widgetname);
		mem_free(field);
		return NULL;
		}
	watchtower_hash_insert(globalreg->watchhashbykey, watch->key, watch);
	/* free field if its needed to be done */
	if (field)
		if (watchtower_hash_insert(globalreg->watchhashbyfield, field, watch) == 1)
			mem_free(field);
	watchtower_hash_insert(globalreg->watchhashbywidget, watch->widget, watch);
	watchtower_hash_insert(globalreg->watchhashbydbobjectsrc, 
		watch->dbobjectsrc, watch);
	/* find out if anything else is registered to this */
	affected = watchtower_check_bywatch(globalreg, watch);
	for (walk = g_list_first(affected); walk != NULL; walk = walk->next)
		{
		reg = walk->data;
		debuglog(90, "Running watchtower %s due to registry of %s", 
			watch->watching->widgetname,reg->registraint->widgetname);
		watchtower_run(watch, reg->registraint);
		}
	return affected;
	}

/**
 * watchtower_forcerun:
 * 
 * 
 */
gint
watchtower_forcerun(BondObject * bobj)
	{
	GList *walk;

	if (!bobj->watchtowerlist)
		return -1;
	for (walk = g_list_first(bobj->watchtowerlist); walk != NULL;
	        walk = walk->next)
		watchtower_run(walk->data, NULL);
	return 0;
	}

/**
 * watchtower_change:
 * 
 * Check for a watch looking on @bobj. If so run the change code.
 * This function is called if a bond object is changed.
 */
gint
watchtower_change(BondObject * bobj)
	{
	gint retval = 0;
	GList *walk1, *walk2, *affected;
	WatchTower *watch;
	GlobalRegistor *globalreg;

	g_assert(bobj);
	globalreg = bobj->bond->globalreg;

	if (!bobj->beenwatched)
		return 0;
	for (walk1 = g_list_first(bobj->registrationlist); walk1 != NULL;
	        walk1 = walk1->next)
		{
		affected = watchtower_check_byregistration(globalreg, walk1->data);
		for (walk2 = g_list_first(affected); walk2 != NULL;
		        walk2 = walk2->next)
			{
			watch = walk2->data;
			watchtower_run(watch, bobj);
			retval = 1;
			}
		if (affected)
			g_list_free(affected);
		}
	return retval;
	}

static void
watchtower_free(GlobalRegistor * globalreg, WatchTower * watch)
	{
	gchar *key;

	g_assert(watch);
	g_assert(watch->watching);

	key = watchtower_register_getfield(watch);
	if (key)
		{
		watchtower_hash_remove(globalreg->watchhashbyfield, key, watch, FALSE);
		mem_free(key);
		}
	if (watch->key)
		watchtower_hash_remove(globalreg->watchhashbykey, watch->key,
		                       watch, FALSE);
	if (watch->dbobjectsrc)
		watchtower_hash_remove(globalreg->watchhashbydbobjectsrc,
		                       watch->dbobjectsrc, watch, FALSE);
	if (watch->widget)
		watchtower_hash_remove(globalreg->watchhashbywidget, watch->widget,
		                       watch, FALSE);

	mem_free(watch);
	}

/**
 * watchtower_unregister:
 * 
 * Unregister and free any watch towers that may have been created by @bobj
 * 
 */
void
watchtower_unregister(BondObject * bobj)
	{
	GList *walk;

	g_assert(bobj);
	if (bobj->watchtowerlist)
		for (walk = g_list_first(bobj->watchtowerlist); walk != NULL;
		        walk = walk->next)
			watchtower_free(bobj->bond->globalreg, walk->data);
	}

static void
watchtower_debug_internal(void *_key, void *_value, void *ptr)
	{
	WatchTower *watch;

	watch = (WatchTower *) _value;
	debug_output("%s watching.\n", watch->watching->widgetname);
	if (watch->fieldname)
		debug_output("F: %s.%s\n", watch->tablename, watch->fieldname);
	if (watch->widget)
		debug_output("W: %s\n", ui_getname(watch->watching->bond->uixml,
		                                   watch->widget));
	if (watch->dbobjectsrc)
		debug_output("O: %s\n", watch->dbobjectsrc);
	}
 
void
watchtower_debug(GlobalRegistor * globalreg)
	{
	g_hash_table_foreach(globalreg->watchhashbykey, 
		watchtower_debug_internal,NULL);
	}

static void
registration_free(GlobalRegistor * globalreg, Registration * reg)
	{
	gchar *key;

	g_assert(reg);
	g_assert(reg->registraint);

	if (reg->fieldname && reg->tablename)
		{
		key = g_strdup_printf("%s.%s", reg->tablename, reg->fieldname);
		watchtower_hash_remove(globalreg->reghashbyfield, key, reg, TRUE);
		g_free(key);
		}
	if (reg->dbobjectsrc)
		watchtower_hash_remove(globalreg->reghashbydbobjectsrc,
		                       reg->dbobjectsrc, reg, FALSE);
	if (reg->registraint->widget)
		watchtower_hash_remove(globalreg->reghashbywidget,
		                       reg->registraint->widget, reg, FALSE);
	if (reg->key)
		{
		if (watchtower_hash_remove(globalreg->reghashbykey, 
					reg->key, reg, TRUE)==-1)
			warningmsg("Nothing to free. key %s",reg->key);
		g_free(reg->key);
		}
	reg_totalreg --;
	mem_free(reg);  
	}

static void
registration_freekey(void *key)
	{
	mem_free(key);
	}

void
registration_init(Bond * bond)
	{
	bond->globalreg = mem_calloc(sizeof(GlobalRegistor));
	bond->globalreg->reghashbykey = g_hash_table_new_full(g_str_hash,
	                                g_str_equal, registration_freekey,
	                                watchtower_hash_free);

	bond->globalreg->reghashbyfield = g_hash_table_new_full(g_str_hash,
	                                  g_str_equal,registration_freekey,
	                                  watchtower_hash_free);

	bond->globalreg->reghashbywidget = g_hash_table_new_full(g_direct_hash,
	                                   g_direct_equal, NULL,
	                                   watchtower_hash_free);

	bond->globalreg->reghashbydbobjectsrc = g_hash_table_new_full(
					   g_str_hash,   
					   g_str_equal, NULL,
	                                   watchtower_hash_free);

	watchtower_hash_init(bond->globalreg);
	}

gchar *
registration_getkey(Object * obj, gchar * tablename, gchar * fieldname)
	{
	static int freeidstr;
	gint needsfreeing;
	gchar *tmpstr;
	gchar *idstr;
	gchar *retstr;
	
	g_assert(tablename);
	idstr = db_id_getuniqueid_fast(obj, tablename, obj->row, &tmpstr, 
			&needsfreeing);
	if (idstr == NULL)
		{
		debugmsg("id returned NULL for fast extraction. "
			"Doing workaround using old code.");
		idstr = db_id_getuniqueid_asstring(obj, tablename, &tmpstr); 
		freeidstr = 1;
		}
		
	if (idstr == NULL)
		{
		warningmsg("Unable to extract uniqueid for %s", tablename);
		return NULL;
		}
	retstr = mem_strdup_printf("%s;&:%s;&:%s", idstr, tablename, fieldname);
	if (freeidstr)
		{
		mem_free(idstr); 
		freeidstr = 0;
		}
	if (needsfreeing == 1)
		{
		mem_free(tmpstr);
		mem_free(idstr);
		}

	return retstr;
	}

void
registration_cleanup(Bond * bond)
	{
	g_hash_table_destroy(bond->globalreg->reghashbykey);
	g_hash_table_destroy(bond->globalreg->reghashbywidget);
	g_hash_table_destroy(bond->globalreg->reghashbydbobjectsrc);
	g_hash_table_destroy(bond->globalreg->reghashbyfield);
	mem_free(bond->globalreg);
	bond->globalreg = NULL;
	}
/**
 * registration_remove_bybobj:
 * @bobj: Bondobject to remove any registration history off
 * 
 * Removes any registrations this object may have had previously.
 */
void
registration_remove_bybobj(BondObject * bobj)
	{
	GList *walk;
	Registration *reg;

    /*	if (bobj->registrationlist) 
     		message("%s has been unregistered",
     		bobj->widgetname); 
	else message("%s has nothing to be unregistered",
	   bobj->widgetname); 
	message("%d items to be removed from regsitration list", 
	g_list_length(bobj->registrationlist));
	*/ 

	for (walk = g_list_first(bobj->registrationlist); walk != NULL;
	        walk = walk->next)
		{
		reg = walk->data;
		registration_free(bobj->bond->globalreg, reg);
		}

	if (bobj->registrationlist)
		g_list_free(bobj->registrationlist);
	bobj->beenwatched = FALSE;
	bobj->registrationlist = NULL;
	}
/**
 *
 *
 */
GList *
registration_check(BondObject * bobj)
	{
	gchar *key;
	GList *retlist;

	/* m.essage("reg check at %s %d items %d",bobj->widgetname,
			g_list_length(bobj->registrationlist),bobj->obj->row); 
	*/
	
	if (bobj->oprnode->field == NULL || bobj->obj == NULL)
		return NULL;
	
	key = registration_getkey(bobj->obj, bobj->oprnode->table, 
			bobj->oprnode->field);

	if (key == NULL)
		{
		errormsg("Key is null on %s",bobj->widgetname);
		return NULL;
		}
	retlist = g_hash_table_lookup(bobj->bond->globalreg->reghashbykey,
		                           key);
	mem_free(key);
	return retlist;
	}

/**
 * registration_add:
 * @bobj: Bond Object to register.
 * @obj: Database object assocated.
 * @tablename: 
 *
 * Registrations a BondObject and everything about it. This is done every
 * time a @bobj changes its data source or is re-drawnn
 */

Registration *
registration_add(BondObject * bobj, Object * obj, gchar * tablename,
                 gchar * fieldname, gboolean quickadd)
	{
	Registration *reg = NULL;
	GlobalRegistor *globalreg;
	gchar *key;

	g_assert(obj);
	g_assert(bobj);
	g_assert(fieldname);

	if (tablename == NULL)
		tablename = db_field_gettable(obj, fieldname);
	g_assert(tablename);
	globalreg = bobj->bond->globalreg;

	key = registration_getkey(obj, tablename, fieldname);
	if (key == NULL)
		errormsg("Registration failed on %s",bobj->widgetname);
	
	reg = (Registration *) mem_alloc(sizeof(Registration));
	reg->registraint = bobj;
	reg->row = obj->row;
	reg->fieldname = fieldname;
	reg->tablename = tablename;
	reg->dbobjectsrc = bobj->oprnode->dbobjectsrc;
	reg->key = g_strdup(key);
	/* insert the data into the registery */

	reg_totalreg ++;
	if (watchtower_hash_insert(globalreg->reghashbykey, key, reg) == 1)
		mem_free(key);
	
	/* debuglog(90,"Registeration of %s %d",reg->key,reg->row); */
	/* if doing a quick add only add hash of key. No checks for watchs 
	 * on it. */
	if (quickadd)
		return reg;
	/* nope now we do the full on registration */
	if (reg->dbobjectsrc)
		watchtower_hash_insert(globalreg->reghashbydbobjectsrc,
		                       reg->dbobjectsrc, reg);


	if (reg->fieldname && reg->tablename)
		{
		key = mem_strdup_printf("%s.%s", reg->tablename, 
				reg->fieldname);
		if (watchtower_hash_insert(globalreg->reghashbyfield, key, 
					reg) == 1)
			mem_free(key);
		}

	if (reg->registraint->widget)
		watchtower_hash_insert(globalreg->reghashbywidget,
		                       reg->registraint->widget, reg);

	/* check if something is watching for this registration */
	if (watchtower_checkregistraint(globalreg, reg))
		{
		debuglog(50, "Oh no i'm been watched! %s %s.%s", 
			bobj->widgetname, reg->tablename, reg->fieldname);
		bobj->beenwatched = TRUE;
		}
	else
		bobj->beenwatched = FALSE;
	return reg;
	}

static void
registration_debugtotal_count(void *key, void *value, void *ptr)
	{
	int *counter;
	counter = ptr;
	if (value)
		*counter = *counter + g_list_length((GList*)value);
	}

int
registration_hash_length(GHashTable * table)
	{
	int counter = 0;

	g_hash_table_foreach(table, registration_debugtotal_count, (void *)&counter);
	return counter;
	}

void
registration_debugtotal(Bond * bond)
	{
	debug_output("Total registrations: %d\n",
	             registration_hash_length(bond->globalreg->reghashbywidget));
	}

void
registration_debugall(GList * reglist)
	{
	Registration *reg;
	GList *walk;

	debug_output("Total registrations: %d\n", g_list_length(reglist));
	for (walk = g_list_first(reglist); walk != NULL; walk = walk->next)
		{
		reg = walk->data;
		registration_debug(reg);
		}
	}

void
registration_debug(Registration * reg)
	{
	if (reg == NULL)
		{
		errormsg("reg is null.");
		return ;
		}
	debug_minorheader();
	debug_output("%s registration.\n", reg->registraint->widgetname);
	if (reg->fieldname)
		debug_output("F: %s.%s\n", reg->tablename, reg->fieldname);
	if (reg->key)
		debug_output("K: %s\n", reg->key);
	if (reg->dbobjectsrc)
		debug_output("O: %s\n", reg->dbobjectsrc);
	debug_minorfooter();
	}

/**
 * registration_getbykey:
 * @globalreg: Global registor.
 * @key: Key generated by registration_getkey().
 * 
 * Return: GList* of Registration's. Dont free.
 */
GList *
registration_getbykey(GlobalRegistor *globalreg, gchar *key)
	{
	return g_hash_table_lookup(globalreg->reghashbykey, key);
	}
