/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <stdio.h>
#include <stdlib.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "xml_display.h"

static int xml_display_node_pad (xmlNode *node, int depth);

static int xml_display_attribute_pad (xmlAttr *attribute, int depth)
{
	xmlNode *child;

	if (!attribute)
		return -1;

	printf ("%*sAttribute: %s\n", depth, "", attribute->name);

	for (child = attribute->children; child != NULL; child = child->next)
		{
		xml_display_node_pad (child, depth + 1);
		}

	return 0;
}

int xml_display_attribute (xmlAttr *a)
{
	return xml_display_attribute_pad (a, 0);
}

static int xml_display_node_pad (xmlNode *node, int depth)
{
	xmlNode *child;
	xmlAttr *attr;

	if (!node)
		return -1;

	printf ("%*sNode: %s\n", depth, "", node->name);

	if (node->content)
		printf ("%*sContent: %s\n", depth, "", node->content);

	for (attr = node->properties; attr != NULL; attr = attr->next)
		xml_display_attribute_pad (attr, depth + 1);

	for (child = node->children; child != NULL; child = child->next)
		xml_display_node_pad (child, depth + 1);

	return 0;
}


int xml_display_node (xmlNode *node)
{
	return xml_display_node_pad (node, 0);
}

static int xml_display_doc_pad (xmlDoc *doc, int depth)
{
	xmlNode *node;

	if (!doc)
		return -1;

	printf ("%*sDoc: %s\n", depth, "", doc->name);

	for (node = doc->children ; node != NULL; node = node->next)
		{
		xml_display_node_pad (node, depth + 1);
		}

	return 0;
}

int xml_display_doc (xmlDoc *doc)
{
	return xml_display_doc_pad (doc, 0);
}
