#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <glob.h>
#include <dlfcn.h>

#include <glib.h>
#include <gtk/gtk.h>

#include "uigtk.h"
#include "bc.h"
#include "parse.h"
#include "uigeneral.h"
#include "uidropdown.h"
#include "uilist.h"
#include "uisignal.h"
#include "uimessage.h"

#include "uicommonplugin.h"

/**
 * plugin_exportfunctions:
 * @plugin: plugin to add function information
 * 
 * 
 * Returns: non-zero on error
 */
static gint
plugin_exportfunctions(Plugin * plugin)
{
	gint retval = 0;

	/* Top Level */
	retval -= plugin_addfunction(plugin, "init", (void *)uigtk_init);
	retval -= plugin_addfunction(plugin, "parse", (void *)uigtk_parse);
	retval -= plugin_addfunction(plugin, "main", (void *)uigtk_main);
	retval -= plugin_addfunction(plugin, "mainquit", (void *)uigtk_mainquit);
	retval -= plugin_addfunction(plugin, "cleanup", (void *)uigtk_cleanup);
	/* Common functions */
	retval -= plugin_addfunction(plugin, "getname", (void *)uigtk_getname);
	retval -= plugin_addfunction(plugin, "getname_short", (void *)uigtk_getwidgetshortname);
	retval -= plugin_addfunction(plugin, "getwidget", (void *)uigtk_getwidget);
	retval -= plugin_addfunction(plugin, "gettype", (void *)uigtk_gettype);
	retval -= plugin_addfunction(plugin, "getchildren", (void *)uigtk_getchildren);
	retval -= plugin_addfunction(plugin, "getparent", (void *)uigtk_getparent);
	retval -= plugin_addfunction(plugin, "widgetlongname", (void *)uigtk_widgetlongname);
	retval -= plugin_addfunction(plugin, "showwidget", (void *)uigtk_showwidget);
	retval -= plugin_addfunction(plugin, "hidewidget", (void *)uigtk_hidewidget);
	retval -= plugin_addfunction(plugin, "getvalue", (void *)uigtk_getvalue);
	retval -= plugin_addfunction(plugin, "setvalue", (void *)uigtk_setvalue);
	retval -= plugin_addfunction(plugin, "isvisible", (void *)uigtk_isvisible);
	retval -= plugin_addfunction(plugin, "seteditable", (void *)uigtk_seteditable);
	retval -= plugin_addfunction(plugin, "geteditable", (void *)uigtk_geteditable);

	/* dropdown stuff */
	retval -= plugin_addfunction(plugin, "dropdown_populate", (void *)uigtk_dropdown_populate);
	retval -= plugin_addfunction(plugin, "dropdown_getentrybox", (void *)uigtk_dropdown_getentrybox);
	retval -= plugin_addfunction(plugin, "dropdown_clear", (void *)uigtk_dropdown_clear);
	retval -= plugin_addfunction(plugin, "dropdown_getpopulatelist", (void *)uigtk_dropdown_populatelist);

	/* list stuff */
	retval -= plugin_addfunction(plugin, "list_getselection", (void *)uigtk_list_getselection);
	retval -= plugin_addfunction(plugin, "list_setselection", (void *)uigtk_list_setselection);
	retval -= plugin_addfunction(plugin, "list_append", (void *)uigtk_list_append);
	retval -= plugin_addfunction(plugin, "list_clear", (void *)uigtk_list_clear);
	retval -= plugin_addfunction(plugin, "list_setcellvalue", (void *)uigtk_list_setcellvalue);

	/* signal stuff */
	retval -= plugin_addfunction(plugin, "signalconnect", (void *)uigtk_signalconnect);
	retval -= plugin_addfunction(plugin, "signaldisconnect", (void *)uigtk_signaldisconnect);
	retval -= plugin_addfunction(plugin, "signaldisconnectbyfunc", (void *)uigtk_signaldisconnectbyfunc);
	retval -= plugin_addfunction(plugin, "signalconnect_keypress", (void *)uigtk_signalconnect_keypress);
	retval -= plugin_addfunction(plugin, "signaldisconnect_keypress", (void *)uigtk_signaldisconnect_keypress);
	/* retval -= plugin_addfunction(plugin, "signalemitstopbyname", (void
	 *)uigtk_signalemitstopbyname); */
	/* retval -= plugin_addfunction(plugin, "issignalattached",
	   uigtk_issignalattached); */

	retval -= plugin_addfunction(plugin, "setuserdata", (void *)uigtk_setuserdata);
	retval -= plugin_addfunction(plugin, "getcontainerchildren", (void *)uigtk_getcontainerchildren);
	retval -= plugin_addfunction(plugin, "message_display", (void *)uigtk_message_display);
	retval -= plugin_addfunction(plugin, "raisewindow", (void *)uigtk_raisewindow);
   
	return retval;
}

/**
 * plugin_init:
 * @plugin: plugin structure passed in
 *
 * Will build a list of functions in the library and return them to 
 * parent library.
 *
 * Returns: 0 on success
 */
gint
plugin_init(Plugin * plugin)
{
	if (!IsEqual(plugin->name, "gtk"))
	{
		errormsg("Not the correct library, can not continue.");
		return -1;
	}

	debug_output("Loading gtk+-2.0 user interface library.\n");

	plugin_exportfunctions(plugin);
	return 0;
}

/**
 * plugin_addfunction:
 * @plugin:
 * 
 * Function used by plugin libraries to export functions. 
 */
gint
plugin_addfunction(Plugin * plugin, gchar * funcname, void *(*funcptr) ())
{
	g_assert(plugin);
	plugin->func = mem_realloc(plugin->func, (sizeof(PluginFunction *) * (plugin->numfunctions + 1)));
	plugin->func[plugin->numfunctions] = mem_alloc(sizeof(PluginFunction));
	plugin->func[plugin->numfunctions]->name = mem_strdup(funcname);
	plugin->func[plugin->numfunctions]->ptr = funcptr;
	plugin->numfunctions++;
	return 0;
}

void
plugin_cleanup(Plugin * plugin)
{
	gint i;

	mem_verify(plugin);
	for (i = 0; i < plugin->numfunctions; i++)
	{
		mem_free(plugin->func[i]->name);
		mem_free(plugin->func[i]);
	}

	mem_free(plugin->func);
}

/**
 * ui_init:
 * @xmlfile: Bond XML which contains the UI descripition
 * 
 * Run all the gtk and XML initilising including gtk_init();
 * Create the global instance of xmlwindow, which will contain a list of all the windows.
 * This will make it nice and easy for window creating.
 *
 * Returns: non-zero on error
 */
UiXML *
uigtk_init(gchar * xmlfile, gint argc, gchar * argv[])
{
	UiXML *bondxml;

	gtk_init(&argc, &argv);
	bondxml = uixml_init(xmlfile);

	if (!bondxml || !bondxml->doc)
		return NULL;
	return bondxml;
}

void
uigtk_parse(UiXML * bondxml)
{
	xmlNodePtr root = NULL;

	root = xmlDocGetRootElement(bondxml->doc);
	uixml_parse(root, 0, NULL, bondxml);
}

void
uigtk_cleanup(UiXML * bondxml)
{
	plugin_cleanup(bondxml->plugin);
}

void
uigtk_main(UiXML * bondxml)
{
	gtk_main();
}

void
uigtk_mainquit(UiXML * bondxml)
{
	gtk_main_quit();
}
