/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_funcs.h"
#include "bc.h"
#include "uimenu.h"
#include "uigtk.h"
#include "uigeneral.h"
#include "parse.h"

static GtkWidget *
uigtk_menuitem_create(xmlNodePtr parentnode, GtkWidget * menu, UiXML *bondxml)
{
	gchar *stock, *label;
	GtkWidget *menuitem;
	GtkWidget *menuimage;

/*
  add_employee1 = gtk_image_menu_item_new_with_mnemonic (_("_Add Employee"));
  gtk_widget_set_name (add_employee1, "add_employee1");
  gtk_widget_show (add_employee1);
  gtk_container_add (GTK_CONTAINER (ed_employee_menu_item_menu), add_employee1);

  image1 = gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_MENU);
  gtk_widget_set_name (image1, "image1");
  gtk_widget_show (image1);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (add_employee1), image1);
*/


	/* a menuitem from stock can only be an image menu item. So in any
	   instance where the label attr has a 'gtk' prefix we will assume
	   this is a stock menu item */
	stock = xmlGetProp(parentnode, "stock");
	label = xmlGetProp(parentnode, "label");
	if (stock && label) {
		if (!IsEqual(stock, label)) {
			/* Get the stock image, with a menu item size */
			menuimage = gtk_image_new_from_stock(stock, GTK_ICON_SIZE_MENU);
			/* Create a new image menu item with a label */
			menuitem = gtk_image_menu_item_new_with_mnemonic(label);
			/* Set the image on to the menu item */
			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), menuimage);
		}
	}
	else if (stock)
		menuitem = gtk_image_menu_item_new_from_stock(stock, NULL);
	else if (label) {
		if (!strncmp("gtk-", label, 4))
			menuitem = gtk_image_menu_item_new_from_stock(label, NULL);
		else
			menuitem = gtk_menu_item_new_with_mnemonic(label);
	}
	else
		menuitem = gtk_menu_item_new();
	
	uixml_parsename(parentnode, menuitem, bondxml);

	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	gtk_widget_show(menuitem);
	return menuitem;
}

GtkWidget *
uigtk_menu_create(xmlNodePtr parentnode, GtkWidget * menubar, UiXML *bondxml)
{
	gchar *content;
	GtkWidget *menu;
	GtkWidget *parentmenu;
	GtkWidget *separator;
	GtkWidget *submenu;
	xmlNodePtr node;
	
	content = xmlGetProp(parentnode, "label");
	if (content)
		parentmenu = gtk_menu_item_new_with_mnemonic(content);
	else
		parentmenu = gtk_menu_item_new();
	gtk_widget_show(parentmenu);
	menu = gtk_menu_new();
	
	/* Parse the menu items */
	for (node = parentnode->children; node != NULL; node = node->next) {
		if (IsEqual(node->name, "menuitem"))
			uigtk_menuitem_create(node, menu, bondxml);
		/* check to see if there are any submenus */
		else if (IsEqual(node->name, "menu")) {
			submenu = uigtk_menu_create(node, menu, bondxml);
			uixml_parsename(parentnode, submenu, bondxml);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu), submenu);
			gtk_widget_show(submenu);
		}
		else if (IsEqual(node->name, "separator")) {
			separator = gtk_separator_menu_item_new();
			gtk_menu_shell_append(GTK_MENU_SHELL(menu), separator);
			gtk_widget_show(separator);
		}
	}
			 
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(parentmenu), menu);
	
	return parentmenu;
}

GtkWidget *
uigtk_menubar_create(xmlNodePtr parentnode, GtkWidget * parentwidget)
{
	gchar *content = NULL;
	GtkWidget *menubar;
	
	menubar = gtk_menu_bar_new();
	debuglog(50,"Creating menubar which shall have many options");
     
	content = xmlGetProp(parentnode, "visible");
	if (content && IsFalse(content))
		gtk_widget_hide(menubar);
	else
		gtk_widget_show(menubar);

	return menubar;
}

gint
uigtk_menubar_add(GtkWidget *menuitem, GtkWidget *menubar)
{
	gtk_menu_shell_append(GTK_MENU_SHELL(menubar), menuitem);
	return 0;
}
