#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_display.h"
#include "bc.h"
#include "uispinbutton.h"
#include "parse.h"

/**
 * ui_button_create:
 * @parentnode: an xml node in the bond xml file
 * @parentwidget: parent widget of this node
 *
 * Creates a button based on the properties specified in the Xml node.
 *
 * Attributes:
 * digits
 * numeric
 * snap-to-ticks
 * step
 * page
 * defaultvalue
 * wrap
 * min
 * max
 * 
 * Returns: pointer to widget created
 */
GtkWidget *
uigtk_spinbutton_create(xmlNode * widgetnode, GtkWidget * parentwidget)
{
	gdouble value;
	guint digits = 0;
	gboolean flag = FALSE;
	gboolean range = FALSE;
	gchar *content = NULL;
	GtkWidget *widget;
	guint step = 1;
	guint page = step * 10;
	gdouble min = 0;
	gdouble max = 100;

	widget = gtk_spin_button_new_with_range(min, max, step);

	content = xmlGetProp(widgetnode, "name");
	if (content)
		debuglog(50, "Creating spin box %s", content);
	else
		debuglog(50, "Creating spin box %s", parentwidget->name);
	
	

	content = xmlGetProp(widgetnode, "digits");
	if (content) {
		digits = atoi(content);
		gtk_spin_button_set_digits(GTK_SPIN_BUTTON(widget), digits);
	}
	content = xmlGetProp(widgetnode, "numeric");
	if (content && IsTrue(content))
		gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	else
		gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), FALSE);

	content = xmlGetProp(widgetnode, "snap-to-ticks");
	if (content && IsTrue(content))
		gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(widget), TRUE);
	else
		gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(widget), FALSE);
     
	content = xmlGetProp(widgetnode, "step");
	if (content) {
		step = atoi(content);
		flag = TRUE;
	}
  
	content = xmlGetProp(widgetnode, "page");
	if (content) {
		page = atoi(content);
		flag = TRUE;
	}
  
	content = xmlGetProp(widgetnode, "defaultvalue");
	if (content) {
		value = atoi(content);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), value);
	}

	content = xmlGetProp(widgetnode, "wrap");
	if (content && IsTrue(content))
		gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(widget), TRUE);
	else
		gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(widget), FALSE);

	content = xmlGetProp(widgetnode, "min");
	if (content) {
		min = atoi(content);
		range = TRUE;
	}

	content = xmlGetProp(widgetnode, "max");
	if (content) {
		max = atoi(content);
		range = TRUE;
	}
     
	if (range) {
		gtk_spin_button_set_range(GTK_SPIN_BUTTON(widget), min, max);
		range = FALSE;
	}
	if (flag) {
		gtk_spin_button_set_increments(GTK_SPIN_BUTTON(widget), step, page);
		flag = FALSE;
	}
     
	gtk_widget_show(widget);
	return widget;
}
