/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_display.h"
#include "bc.h"
#include "uitextview.h"
#include "parse.h"

/**
 * uigtk_textview_create:
 * @widgetnode: An xmlNodePtr to the current node in Bond.
 * @parentwidget: The containing widget.
 * @scrolledwindow: Scrolled window widget.
 *
 * Creates a scrolled window, text view and buffer widget.
 *
 * Properties:
 * editable - "true | false"
 * wrap - "word | char | none"
 * cursorvisible - "true | false"
 *
 * Returns: 
 */
GtkWidget *
uigtk_textview_create(xmlNodePtr widgetnode, GtkWidget * parentwidget,
                      GtkWidget ** scrolledwindow)
	{
	GtkWidget *view;
	GtkTextBuffer *buffer;
	xmlChar *content = NULL;
	xmlNodePtr node;
	GtkTextTagTable *tagTable;

	*scrolledwindow = NULL;
	tagTable = gtk_text_tag_table_new();
	buffer = gtk_text_buffer_new(GTK_TEXT_TAG_TABLE(tagTable));
	view = gtk_text_view_new_with_buffer(GTK_TEXT_BUFFER(buffer));

	gtk_widget_show(view);

	content = xmlGetProp(widgetnode, "editable");
	if (content && IsFalse(content))
		gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
	else
		gtk_text_view_set_editable(GTK_TEXT_VIEW(view), TRUE);

	content = xmlGetProp(widgetnode, "cursorvisible");
	if (content && IsFalse(content))
		gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE);
	else
		gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), TRUE);

	content = xmlGetProp(widgetnode, "wrap");
	if (content)
		{
		if (IsEqual(content, "char"))
			gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_CHAR);
		else if (IsEqual(content, "word"))
			gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_WORD);
		}
	else
		gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_NONE);

	content = xmlGetProp(widgetnode, "justify");
	if (content)
		{
		if (IsEqual(content, "right"))
			gtk_text_view_set_justification(GTK_TEXT_VIEW(view),
			                                GTK_JUSTIFY_RIGHT);
		else if (IsEqual(content, "center"))
			gtk_text_view_set_justification(GTK_TEXT_VIEW(view),
			                                GTK_JUSTIFY_CENTER);
		else if (IsEqual(content, "fill"))
			gtk_text_view_set_justification(GTK_TEXT_VIEW(view),
			                                GTK_JUSTIFY_FILL);
		}
	else
		gtk_text_view_set_justification(GTK_TEXT_VIEW(view), GTK_JUSTIFY_LEFT);

	content = NULL;
	for (node = widgetnode->children; node != NULL; node = node->next)
		{
		if (IsEqual(node->name, "content"))
			{
			content = xmlNodeGetContent(node);
			}
		}

	if (content)
		{
		gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer), content, -1);
		debuglog(50, "Retrieved textual content: %s", content);
		}

	gtk_text_view_set_buffer(GTK_TEXT_VIEW(view), GTK_TEXT_BUFFER(buffer));
	/* Create a scrolled window around the text box */
	*scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(*scrolledwindow),
	                               GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(*scrolledwindow),
	                                      GTK_WIDGET(view));
	gtk_widget_show(*scrolledwindow);

	debuglog(50, "Creating a textview");

	return GTK_WIDGET(view);
	}
