#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

/* #include <glib.h> */
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_display.h"
#include "bc.h"
#include "uitoggle.h"
#include "parse.h"

/**
 * uigtk_toggle_button_create:
 * @parentnode: An xmlNodePtr in the Bond XML.
 * @parentwidget: The toggle buttons containing widget.
 *
 * Creates a toggle button.
 *
 * Properties:
 * active - "true | false"
 *     Set the toggle button to active state.
 *
 * inconsistent | mode - "true | false"
 *     Set the toggle button to inconsistent.
 *
 * label
 *     Set the label of the toggle button.
 *
 * Returns: The created toggle button, if successful.
 */
GtkWidget *
uigtk_toggle_button_create(xmlNodePtr parentnode, GtkWidget * parentwidget)
	{
	GtkWidget *widget;
	GtkWidget *alignment;
	GtkWidget *hbox;
	GtkWidget *image;
	GtkWidget *label;
	gchar *content = NULL;

	widget = gtk_toggle_button_new();
	debuglog(53, "Creating toggle %s", parentnode->name);

	content = xmlGetProp(parentnode, "active");
	if (content && IsTrue(content))
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
	else
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), FALSE);

	content = xmlGetProp(parentnode, "inconsistent");
	if (content && IsTrue(content))
		gtk_toggle_button_set_inconsistent(GTK_TOGGLE_BUTTON(widget), TRUE);
	else
		gtk_toggle_button_set_inconsistent(GTK_TOGGLE_BUTTON(widget), FALSE);

	content = xmlGetProp(parentnode, "mode");
	if (content && IsTrue(content))
		gtk_toggle_button_set_inconsistent(GTK_TOGGLE_BUTTON(widget), TRUE);
	else
		gtk_toggle_button_set_inconsistent(GTK_TOGGLE_BUTTON(widget), FALSE);

	content = xmlGetProp(parentnode, "image");
	if (content)
		{
		widget = gtk_toggle_button_new();
		alignment = gtk_alignment_new(0.5, 0.5, 0, 0);
		gtk_widget_show(alignment);
		gtk_container_add(GTK_CONTAINER(widget), alignment);

		hbox = gtk_hbox_new(FALSE, 2);
		gtk_widget_show(hbox);
		gtk_container_add(GTK_CONTAINER(alignment), hbox);

		image = gtk_image_new_from_stock(content, GTK_ICON_SIZE_BUTTON);
		gtk_widget_show(image);
		gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);

		if ((content = xmlGetProp(parentnode, "label")) != NULL)
			label = gtk_label_new_with_mnemonic(content);
		else
			label = gtk_label_new_with_mnemonic("");
		gtk_widget_show(label);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
		}
	else
		{
		content = xmlGetProp(parentnode, "label");
		if (content)
			gtk_button_set_label(GTK_BUTTON(widget), content);
		}

	gtk_widget_show(widget);
	return widget;
	}
