#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_display.h"
#include "bc.h"
#include "uihtml.h"
#include "uibutton.h"

#include "parse.h"

/**
 * ui_button_create:
 * @parentnode: an xml node in the bond xml file
 * @parentwidget: parent widget of this node
 *
 * Creates a button based on values of label and stock. 
 * If only label is set: and the label is a stock id, then the button
 *   created is a stock button, otherwise gtk will create a button with
 *   a label text the value of the label.
 * If stock and label are set: then a stock property will take precedence over the label property.
 * If only stock is set: same as if only label was set.
 * If neither are set: A button with no label is created.
 *
 * Attributes:
 * label: sets label of button
 * stock: stock id to use. eg "gtk-add"
 * 
 * Returns: pointer to the widget created.
 */
UiHtml *
uihtml_button_create(UiXML * uixml, xmlNode * node, UiHtml * parentwidget, 
					gboolean * visible)
{
	UiHtml *button;
	gchar *content = NULL, *accesskeystr;
	gchar accesskey = 0;

	button = uihtml_create(node);
	button->widgettype = UiButton;

	button->openinghtml = mem_strdup("<input type=\"submit\"");

	if(node && 
	   node->children && 
	   node->children->next && 
	   node->children->next->children &&
	   node->children->next->children->next)
	  {
		/* if api call is "quit", don't make the button */
		if(IsEqual(xmlGetProp(node->children->next->children->next, "call"), "quit"))
		  {
			button->openinghtml = button->closinghtml = NULL;
			return button;
		  }
		  /* if api call is "clear", make a reset button */
		if(IsEqual(xmlGetProp(node->children->next->children->next, "call"), "clear"))
		  button->openinghtml = mem_strdup("<input type=\"reset\"");
	  }
	  
	if (uixml->defaultvisible)
		*visible = TRUE;
	else
		*visible = FALSE;
	
	content = xmlGetProp(node, "visible");
	if (content && IsTrue(content))
		*visible = TRUE;
	else if (content && IsFalse(content))
		*visible = FALSE;
	button->visible = *visible;

	
	content = xmlGetProp(node, "label");
	if(content)
	  {
		/* remove underscores for button label, create accesskey from underscore */
		int j=0;
		int i=0;
		while(content[j] != 0)
		  {
			if (content[j] == '_')
			  {
				j++;
				if(!accesskey)
				  accesskey = tolower(content[j]);
				
			  }
			content[i] = content[j];
			i++;
			j++;
		  }
		content[i] = 0;
		
		button->openinghtml = 
		  mem_strdup_printf("%s value=\"%s\"", button->openinghtml, content);
		
		/* replace spaces with underscores for button name */
		i=0;
		while(content[i] != 0)
		  {
			if (g_ascii_isspace(content[i]))
			  content[i] = '_';
			i++;
		  }
		content[i] = 0;
		
		button->openinghtml = mem_strdup_printf("%s name=\"%s\"", 
												button->openinghtml, 
												g_ascii_strdown(content, -1));
	  }
	
	if(accesskey)
	  accesskeystr = mem_strdup_printf(" accesskey=\"%c\"", accesskey);
	else
	  accesskeystr = mem_strdup("");
	
	button->openinghtml = mem_strdup_printf("%s%s/>", button->openinghtml, accesskeystr);
	button->closinghtml = NULL;

	return button;
}
