
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_display.h"
#include "bc.h"
#include "uidropdown.h"
#include "parse.h"

GList *
uihtml_dropdown_populatelist(void *bond, xmlNodePtr poplist)
{
  GList *list = NULL;
  xmlNodePtr node = NULL;

  /* Assume the DTD is correct and just parse the children of <populatelist>.
	 xmlNodeGetContent calls xmlStrdup() to allocate space for the content */
  for (node = poplist->children; node; node = node->next)
	{
	  if (IsEqual(node->name, "listitem"))
		list = g_list_append(list, xmlNodeGetContent(node));
	}

  if (!list)
	errormsg("Nothing in the populated list! %d",xmlGetLineNo(poplist));
	
  return list;
}

static void
uihtml_dropdown_buildlistfree(GList * list)
{
  GList *walk;

  for (walk = g_list_first(list); walk; walk = walk->next)
	g_free(walk->data);
  g_list_free(list);
}

UiHtml *
uihtml_dropdown_create(UiXML * uixml, xmlNode * parentnode, UiHtml * parentwidget,
					   gboolean * visible)
{
  xmlChar *content = NULL;
  UiHtml *select;
  gint maxwidth = -1, minwidth = -1;
  select = uihtml_create(parentnode);
  select->widgettype = UiGeneral;
  
  debuglog(50, "Creating dropdown box %s", parentnode->name);
	
  content = xmlGetProp(parentnode, "visible");
  if (content && IsFalse(content))
	*visible = FALSE;
  else 
	*visible = TRUE;

  /*	for (node = parentnode->children; node; node = node->next)
	{
	if (IsEqual(node->name, "populatelist"))
	{
	popdownlist = uihtml_dropdown_populatelist(NULL, node);
	gtk_combo_set_popdown_strings(GTK_COMBO(widget), popdownlist);
  */
#if 0
  /* append each string in the list to the dropdown box */
  for (walk = g_list_first(popdownlist); walk; walk = walk->next)
	gtk_combo_box_append_text(GTK_COMBO(widget), walk->data);
#endif
  /*
	uihtml_dropdown_buildlistfree(popdownlist);
	}
	}
  */
  select->openinghtml = mem_strdup("<select");
	
  content = xmlGetProp(parentnode, "name");
  if(content)
	select->openinghtml = 
	  mem_strdup_printf("%s name=\"%s\"", select->openinghtml, content);

  content = xmlGetProp(parentnode, "maxwidth");
  if(content)
	maxwidth = atoi(content);
  
  content = xmlGetProp(parentnode, "minwidth");
  if(content)
	minwidth = atoi(content);
  
  if(minwidth >= 0 || maxwidth >= 0)
	{
	  select->openinghtml = mem_strdup_printf("%s style=\"", select->openinghtml);
	  if(minwidth >= 0)
		select->openinghtml = mem_strdup_printf("%smin-width:%dpx;", select->openinghtml, minwidth);
	  if(maxwidth >= 0)
		select->openinghtml = mem_strdup_printf("%smax-width:%dpx;", select->openinghtml, maxwidth);
	  select->openinghtml = mem_strdup_printf("%s\"", select->openinghtml);
	}	
  
  select->openinghtml = mem_strdup_printf("%s><option>Select...", select->openinghtml);
  select->closinghtml = mem_strdup("</select>");
  
  return select;
}

void
uihtml_dropdown_populate(UiXML * uixml, UiHtml * widget, GList * list)
{
  gtk_combo_set_popdown_strings(GTK_COMBO(widget), list);

#if 0

  GList *walk;

  for (walk = g_list_first(list); walk; walk = walk->next)
	gtk_combo_box_append_text(GTK_COMBO(widget), walk->data);
#endif
}

void
uihtml_dropdown_clear(UiXML * uixml, GtkCombo * widget)
{
  GtkList *child;

  child = GTK_LIST(GTK_COMBO(widget)->list);
  gtk_list_clear_items(child, 0, -1);
}
