/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "uihtml.h"
#include "xml_display.h"
#include "xml_funcs.h"
#include "bc.h"
#include "uibox.h"
#include "parse.h"
#include "uigeneral.h"
#include "uidropdown.h"

void
uihtml_addcontainer(UiHtml * child, UiHtml * parent)
{
  /** do not remove this assert, when everything is uihtml-ised it won't appear
   *
   * *** treat the cause not the symptom ***
   */
  if (GTK_IS_WIDGET(child) || GTK_IS_WIDGET(parent))
	{
	  errormsg("Invalid widgeT");
	  return;
	}
  child->parent = parent;
  parent->children = g_list_prepend(parent->children,child);

#if 0
  gtk_container_add(GTK_CONTAINER(parent), child);
#endif
}

/**
 * uihtml_project_create:
 * 
 *  
 *
 */
UiHtml *
uihtml_project_create(UiXML * uixml, xmlNode * node)
{
  UiHtml *project;
  gchar *content = NULL;

  project = uihtml_create(node);
  project->widgettype = UiGeneral;

  content = xmlGetProp(node, "name");
  if(!content)
	content = mem_strdup("");
  


  return project;
}

/**
 * uihtml_window_create:
 * @widgetnode: an xml node in the bond XML file.
 * @parentwidget: widget that will be the parent of this.
 *
 * Creates a window based on properties specified in the XML
 * @node.
 *
 * Attributes:
 * name:    Specifying the name to be given to this widget.
 * title:   Title of the window.
 * visible: Can be either "true" or "false".
 *
 * Example:
 * <window name="givenname" title="This is the title" visible="true">
 *
 * Returns: Pointer to the widget created.
 */
UiHtml *
uihtml_window_create(UiXML * uixml, xmlNode * node, UiHtml * parentwidget,
					 gboolean * visible)
{
  UiHtml *uihtml;
  gchar *content = NULL, *name, *title;

  uihtml = uihtml_create(node);
  uihtml->widgettype = UiForm;

  if (uixml->defaultvisible)
	*visible = TRUE;
  else
	*visible = FALSE;

  
  if (!(title = xmlGetProp(node, "title")))
	title = mem_strdup("FORM");
  
  if (!(name = xmlGetProp(node, "name")))
	errormsg("Window does not contain a name.");
	     
  content = xmlGetProp(node, "visible");
  if (content && IsTrue(content))
	*visible = TRUE;
  else if (content && IsFalse(content))
	*visible = FALSE;

  uihtml->visible = *visible;
	
  /*todo, decide on action for form*/
  uihtml->openinghtml = 
	mem_strdup_printf("<h2>%s</h2>\n<FORM id=\"%s\" method=\"get\"\naction=\"http://treshna.com/~sam/request.php\"><input type=\"hidden\" name=\"sid\" value=\"%s\"",
					  title,name, uixml->sessionid);
  uihtml->closinghtml = mem_strdup("</FORM>");
	
  return uihtml;
#if 0	
  /* TO BE REMOVED */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  if (title)
	{
	  gtk_window_set_title(GTK_WINDOW(window), title);
	  debuglog(37, "Creating window with title %s", title);
	}
	   
  return window;
#endif
}

/**
 * uihtml_label_create:
 * @widgetnode: a xml node in the bond XML file.
 * @parentwidget: widget that will be the parent of this.
 *
 * Creates a label based on properties specified in the XML
 * @node. It makes the widget visible by default.
 *
 * Options:
 *
 *
 * Example:
 * <label>label txt</label>
 *
 * Returns: Pointer to the widget created.
 */
UiHtml *
uihtml_label_create(UiXML * uixml, xmlNode * node, UiHtml * parentwidget,
					gboolean * visible)
{
  UiHtml *label;
  xmlChar *content = NULL;
  gchar *justify;
	  
  g_assert(node);
  label = uihtml_create(node);
  label->widgettype = UiLabel;
  label->openinghtml = mem_strdup("<div class=\"label");

  if(uixml->defaultvisible)
	*visible = TRUE;
  else
	*visible = FALSE;
	  
  content = xmlGetProp(node, "visible");
  if (content && IsTrue(content))
	*visible = TRUE;
  else if (content && IsFalse(content))
	*visible = FALSE;
	  
  label->visible = *visible;
	  
  content = xmlGetProp(node, "justify");
  if (content)
	{
	  if (IsEqual(content, "right"))
		justify = mem_strdup(content);
	  else if (IsEqual(content, "center"))
		justify = mem_strdup(content);
	  else if (IsEqual(content, "fill"))
		justify = mem_strdup("justify");
	  else
		justify = mem_strdup("left");
	}
  else
	justify = mem_strdup("left");
  
  if(node->children->content)
	content = mem_strdup(node->children->content);
  else 
	content = mem_strdup("");
	  
  label->openinghtml = 
	mem_strdup_printf("%s\" style=\"{text-align:%s; width:100%;}\">%s",
					  label->openinghtml, justify, content);
  label->closinghtml = mem_strdup("</div>");
	  
  debuglog(50, "Creating label to say '%s'", content);
	  
  return label;
}

UiHtml *
uihtml_scroll_bar_create(UiXML *uixml, xmlNode * widgetnode, UiHtml * parentwidget, gboolean * visible)
{
  UiHtml *scrollbar = NULL;
  GtkAdjustment *adjust;
  gchar *content = NULL;

  adjust = GTK_ADJUSTMENT(gtk_adjustment_new(1.1, 10.6, 5.4, 0.4, 0.5, 3.5));

  content = xmlGetProp(widgetnode, "alignment");
  if (content)
	{
	  if (IsEqual(content, "horizontal"))
		scrollbar = gtk_hscrollbar_new(adjust);
	  else if (IsEqual(content, "vertical"))
		scrollbar = gtk_vscrollbar_new(NULL);
	}

  if (scrollbar)
	gtk_widget_show(scrollbar);
  else
	errormsg("Error processing scrollbar");
  return scrollbar;

}

/**
 * uihtml_scale_create:
 * @widgetnode: a xml node in the bond XML file.
 * @parentwidget: widget that will be the parent of this.
 *
 * Creates a scale widget based on properties specified in the XML
 * @node. It makes the widget visible by default.
 *
 * Options:
 * <minvalue> minimum value of the scale. set to 0 by default. 
 * <maxvalue> should be greater then 0.
 * <alignment> indicates if the scale is horizontal or vertical
 * <step> sould be between the min value and the maxvalue
 * Returns: Pointer to the widget created.
 */
UiHtml *
uihtml_scale_create(UiXML * uixml, xmlNode * widgetnode, UiHtml * parentwidget, 
					gboolean * visible)
{
  UiHtml *scale = NULL;
  gdouble min = 0;
  gdouble max = 0;
  gdouble step = 0;
  gchar *content = NULL;

  content = xmlGetProp(widgetnode, "minvalue");
  if (content)
	min = atof(content);

  content = xmlGetProp(widgetnode, "maxvalue");
  if (content)
	max = atof(content);

  content = xmlGetProp(widgetnode, "step");
  if (content)
	step = atof(content);

  content = xmlGetProp(widgetnode, "alignment");
  if (content)
	{
	  if (IsEqual(content, "vertical"))
		scale = gtk_vscale_new_with_range(min, max, step);
	  else
		scale = gtk_hscale_new_with_range(min, max, step);
	}

  if (scale)
	gtk_widget_show(scale);
  else
	debugmsg("failed to create scale");
  return scale;

}

UiHtml *
uihtml_status_bar_create(UiXML * uixml, xmlNode * widgetnode, UiHtml * parentwidget, 
						 gboolean * visible)
{
  UiHtml *statusbar;
  gchar *content = NULL;
  xmlNodePtr node;
  guint message_context = 0;

  statusbar = gtk_statusbar_new();
  gtk_widget_show(statusbar);

  for (node = widgetnode->children; node != NULL; node = node->next)
	{
	  content = xmlNodeGetContent(node);
	  if (!content)
		continue;
	  if (uixml_node(node, "message"))
		{
		  message_context =
			gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar),
										 content);
		  gtk_statusbar_push(GTK_STATUSBAR(statusbar), message_context,
							 content);
		}
	}
  return statusbar;

}

UiHtml *
uihtml_entry_create(UiXML * bondxml, xmlNode * widgetnode,
					UiHtml * parentwidget, gboolean * visible)
{
  guint len;
  gchar *content = NULL;
  UiHtml *input = NULL;
	  
  input = uihtml_create(widgetnode);
  input->widgettype = UiEntryBox;
  
  content = xmlGetProp(widgetnode, "name");
  if (content)
  input->openinghtml=mem_strdup_printf
	("<input name=\"%s\"", content);
	
  /* Get attributes, this is new bond */
  content = xmlGetProp(widgetnode, "defaultvalue");
  if (content)
	input->openinghtml=mem_strdup_printf("%s value=\"%s\"",
										 input->openinghtml, content);
  
  if(bondxml->defaultvisible)
	*visible = TRUE;
  else
	*visible = FALSE;
  content = xmlGetProp(widgetnode, "visible");
  if (content && IsFalse(content))
	*visible = FALSE;
  else if (content && IsTrue(content))
	*visible = TRUE;

  input->visible=*visible;
  
  content = xmlGetProp(widgetnode, "hide-input");
  if (content && IsTrue(content))
	input->openinghtml = mem_strdup_printf("%s type=\"password\"", 
										   input->openinghtml);
  else 
	input->openinghtml = mem_strdup_printf("%s type=\"text\"", 
										   input->openinghtml);
  
  content = xmlGetProp(widgetnode, "maxlength");
  if (content && (len = atoi(content)))
	input->openinghtml=mem_strdup_printf("%s maxlength=\"%d\"", 
										   input->openinghtml, len);
  
  input->openinghtml = mem_strdup_printf("%s />", input->openinghtml);
  input->closinghtml = NULL;
  
  content = xmlGetProp(widgetnode, "editable");
  if (content && IsFalse(content))
	{
	}
  
  return input;
}

gchar *
uihtml_getname(UiXML * bondxml, UiHtml * widget)
{
  return g_hash_table_lookup(bondxml->widgets, widget);
}

UiHtml *
uihtml_getwidget(UiXML * bondxml, gchar * name)
{
  UiHtml *widget;

  widget = g_hash_table_lookup(bondxml->longname, name);
  if (!widget)
	return g_hash_table_lookup(bondxml->shortname, name);
  return widget;
}

gchar *
uihtml_getwidgetshortname(UiXML * bondxml, UiHtml * widget)
{
  return
	uixml_getwidgetshortname_fromname(g_hash_table_lookup
									  (bondxml->widgets, widget));
}

gchar *
uihtml_widgetlongname(UiXML * bondxml, gchar * widgetname)
{
  UiHtml *widget;

  widget = uihtml_getwidget(bondxml, widgetname);
  if (!widget)
	return NULL;
  return g_hash_table_lookup(bondxml->widgets, widget);
}

void
uihtml_showwidget(UiXML * bondxml, UiHtml * widget)
{
  g_assert(widget);
  
  uihtml_output(0,bondxml,widget);
  
  /* gtk_widget_show(widget); */
}

void
uihtml_hidewidget(UiXML * bondxml, UiHtml * widget)
{
  g_assert(widget);
  gtk_widget_hide(widget);
}

UiWidgetType
uihtml_gettype(UiXML * bondxml, UiHtml * widget)
{
  GtkType widgettype;

  g_assert(widget);

  widgettype = GTK_WIDGET_TYPE(widget);
  if (widgettype == GTK_TYPE_ENTRY)
	return UiEntryBox;
  if (widgettype == GTK_TYPE_COMBO)
	return UiDropDown;
  if (widgettype == GTK_TYPE_TREE_VIEW)
	return UiGrid;
  if (widgettype == GTK_TYPE_TREE_VIEW_COLUMN)
	return UiGridColumn;
  if (widgettype == GTK_TYPE_SCROLLED_WINDOW)
	return UiScrolledWindow;
  if (widgettype == GTK_TYPE_CHECK_BUTTON)
	return UiCheckButton;
  if (widgettype == GTK_TYPE_SPIN_BUTTON)
	return UiSpinButton;
  /* if (widgettype == GTK_TYPE_TEXT) return UiTextBox; */
  if (widgettype == GTK_TYPE_LABEL)
	return UiLabel;
  if (widgettype == GTK_TYPE_TOGGLE_BUTTON)
	return UiToggleButton;
  if (widgettype == GTK_TYPE_RADIO_BUTTON)
	return UiRadioButton;
  if (widgettype == GTK_TYPE_WINDOW)
	return UiForm;
  if (widgettype == GTK_TYPE_FRAME)
	return UiFrame;
  if (widgettype == GTK_TYPE_TABLE)
	return UiFrame;
  if (widgettype == GTK_TYPE_BOX)
	return UiFrame;
  if (widgettype == GTK_TYPE_VBOX)
	return UiFrame;
  if (widgettype == GTK_TYPE_HBOX)
	return UiFrame;
  if (widgettype == GTK_TYPE_HANDLE_BOX)
	return UiGeneral;
  if (widgettype == GTK_TYPE_TOOLBAR)
	return UiGeneral;
  if (widgettype == GTK_TYPE_HANDLE_BOX)
	return UiFrame;
  if (widgettype == GTK_TYPE_NOTEBOOK)
	return UiNotebook;
  if (widgettype == GTK_TYPE_BUTTON)
	return UiButton;
  if (widgettype == GTK_TYPE_TEXT_BUFFER)
	return UiTextBox;
  if (widgettype == GTK_TYPE_MENU_ITEM)
	return UiMenuItem;
  if (widgettype == GTK_TYPE_IMAGE_MENU_ITEM)
	return UiMenuItem;
  if (widgettype == GTK_TYPE_MENU_BAR)
	return UiGeneral;

  debugmsg("Write me!, i dont understand this type %d on widget %s",
		   widgettype, uihtml_getname(bondxml, widget));

  return UiGeneral;
}

/* Must be a long name passed in */
GList *
uihtml_getchildren(UiXML * bondxml, gchar * widgetname)
{
  gchar *tmpstr;
  GList *retlist;

  if (!widgetname || strlen(widgetname) == 0)
	tmpstr = (gchar *) g_strdup("");
  else
	tmpstr = (gchar *) g_strdup_printf("%s/", widgetname);

  retlist = uixml_widgetchildren(bondxml, tmpstr);
  g_free(tmpstr);

  return retlist;
}

/**
 * uihtml_dropdown_getparent:
 *
 *
 */
UiHtml *
uihtml_getparent(UiXML * bondxml, UiHtml * widget)
{
  gint i;
  gchar *widgetname;
  UiHtml *retwidget = NULL;

  g_assert(widget);
  widgetname = g_strdup(uihtml_getname(bondxml, widget));

  for (i = strlen(widgetname) - 1; i >= 0; i--)
	{
	  if (widgetname[i] == '/')
		{
		  widgetname[i] = 0;
		  retwidget = uihtml_getwidget(bondxml, widgetname);
		  break;
		}
	  widgetname[i] = 0;
	}
  g_free(widgetname);
  return retwidget;
}

/**
 * uihtml_getvalue:
 * 
 * todo, someone needs to say something.
 *
 */
gint
uihtml_getvalue(UiXML * bondxml, UiHtml * widget, gchar ** value)
{
  static gchar tval[] = "t";
  static gchar fval[] = "f";
  gboolean state;
  GtkType widgettype;
  GtkTextIter start, end;

  g_assert(widget);

  widgettype = GTK_WIDGET_TYPE(widget);
  if (widgettype == GTK_TYPE_ENTRY)
	{
	  *value = (gchar *) gtk_entry_get_text(GTK_ENTRY(widget));
	}
  else if (widgettype == GTK_TYPE_COMBO)
	{
	  *value =
		(gchar *) gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(widget)->entry));
	}
  else if (widgettype == GTK_TYPE_TEXT_BUFFER)
	{
	  /* buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget)); */
	  gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(widget), &start, 0);
	  gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(widget), &end, -1);
	  *value =
		(gchar *) gtk_text_buffer_get_text(GTK_TEXT_BUFFER(widget), &start,
										   &end, FALSE);
	  /* m.essage("Value %s got", *value); */
	}
  else if (widgettype == GTK_TYPE_SPIN_BUTTON)
	{
	  GtkSpinButton *spin = GTK_SPIN_BUTTON(widget);

	  *value = (gchar *) gtk_entry_get_text(&spin->entry);
	}
  else if (widgettype == GTK_TYPE_CHECK_BUTTON
		   || widgettype == GTK_TYPE_TOGGLE_BUTTON
		   || widgettype == GTK_TYPE_RADIO_BUTTON)
	{
	  state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	  if (state)
		*value = tval;
	  else
		*value = fval;
	  /* m.essage("returned %s", *value); */
	}
  else
	{
	  *value = NULL;
	  errormsg
		("This widget: '%s' hasn't been added yet. You are welcome to code it.",
		 uihtml_getname(bondxml, widget));
	  return -1;
	}

  /* else if (widgettype == GTK_TYPE_TOGGLE_BUTTON || widgettype ==
	 GTK_TYPE_CHECK_BUTTON) { state = (gint)
	 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)); *value =
	 g_strdup_printf("%d", state); #warning "Memory leak here" return 0; }
	 else if (widgettype == GTK_TYPE_RADIO_BUTTON) { return
	 uihtml_radio_getvalue(widget, value); } */
  /*
	else if (widgettype == GTK_TYPE_TEXT) { *value =
	gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1); return 0; } */ 
  return 0;
}

/**
 * uihtml_setvalue:
 * @widget:
 * @value: String value to set the widget to.
 *
 * Set the value in a widget to @value.  This works on most widget types, expect
 * for a few for obvious reasons, like clists.
 *
 */
gint
uihtml_setvalue(UiXML * bondxml, UiHtml * widget, gchar * value)
{
  gboolean freevalue = FALSE, state;
  GtkType widgettype;

  g_assert(widget);
  if (!value)
	{
	  value = mem_strdup("");
	  freevalue = TRUE;
	}

  widgettype = GTK_WIDGET_TYPE(widget);
  if (widgettype == GTK_TYPE_ENTRY)
	gtk_entry_set_text(GTK_ENTRY(widget), value);

  else if (widgettype == GTK_TYPE_TEXT_BUFFER)
	gtk_text_buffer_set_text(GTK_TEXT_BUFFER(widget), value, -1);

  else if (widgettype == GTK_TYPE_LABEL)
	gtk_label_set_text(GTK_LABEL(widget), value);

  else if (widgettype == GTK_TYPE_SPIN_BUTTON)
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), atof(value));

  else if (widgettype == GTK_TYPE_CHECK_BUTTON ||
		   widgettype == GTK_TYPE_TOGGLE_BUTTON ||
		   widgettype == GTK_TYPE_RADIO_BUTTON)
	{

	  if (value && strcmp(value, "t") == 0)
		state = TRUE;
	  else
		state = FALSE;
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
								   (gboolean) state);
	}
  else if (widgettype == GTK_TYPE_COMBO)
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(widget)->entry), value);

  else
	{
	  errormsg
		("This widget hasn't been added yet. You are welcome to code it. This"
		 "is for widget %s", uihtml_getname(bondxml, widget));
	  if (freevalue == TRUE)
		mem_free(value);
	  return -1;
	}
  if (freevalue == TRUE)
	mem_free(value);

#if 0

  else if (widgettype == GTK_TYPE_TOGGLE_BUTTON
		   || widgettype == GTK_TYPE_CHECK_BUTTON)
	{
	  state = FALSE;
	  if (IsEqual(value, "t") || IsEqual(value, "1") || IsEqual(value, "T")
		  || IsEqual(value, "Y"))
		state = TRUE;

	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
								   (gboolean) state);
	  return 0;
	}
  else if (widgettype == GTK_TYPE_RADIO_BUTTON)
	return uihtml_radio_setvalue(widget, value);

  else if (widgettype == GTK_TYPE_TEXT)
	{
	  gtk_editable_delete_text(GTK_EDITABLE(widget), 0, -1);
	  if (value)
		gtk_editable_insert_text(GTK_EDITABLE(widget), value, strlen(value),
								 &pos);
	  gtk_editable_set_position(GTK_EDITABLE(widget), 0);
	  return 0;
	}
#endif

  return 0;
}

/**
 * uihtml_isvisible:
 *
 * Work out if a widget is visible.
 *
 */
gboolean
uihtml_isvisible(UiXML * bondxml, UiHtml * widget)
{
  /* g_assert(widget); */
  return GTK_WIDGET_VISIBLE(widget);
}

/**
 * uihtml_seteditable:
 * @widget: 
 * @enable: %TRUE if you want widget editable, else %FALSE
 *
 */
#if 0
void
uihtml_seteditable(UiXML * bondxml, UiHtml * widget, gboolean state)
{
  gint widgettype;
  GtkEditable *editable;

  if (GTK_IS_WIDGET(widget))
	gtk_widget_set_sensitive(widget, state);

  widgettype = uihtml_gettype(bondxml, widget);
  switch (widgettype)
	{
/*	case UiDropDown:
	  widget = uihtml_dropdown_getentrybox(bondxml, widget);*/
	case UiEntryBox:
	case UiSpinButton:
	  editable = GTK_EDITABLE(widget);
	  gtk_editable_set_editable(editable, state);
	  break;
	case UiRadioButton:
	case UiCheckButton:

	case UiTextBox:
	  break;
	default:
	  break;
	}
}

/**
 * uihtml_geteditable:
 * @widget: 
 * returns wether the widegt is editable or not.
 */
gint
uihtml_geteditable(UiXML * bondxml, UiHtml * widget)
{
  GtkEditable *editable;
  gint widgettype;

  widgettype = uihtml_gettype(bondxml, widget);

  if (widgettype == UiEntryBox)
	{
	  editable = GTK_EDITABLE(widget);
	  return gtk_editable_get_editable(editable);
	}

  else if (widgettype == GTK_TYPE_SPIN_BUTTON)
	{
	  editable = GTK_EDITABLE(widget);
	  return gtk_editable_get_editable(editable);
	}

  return 1;
}
#endif
/**
 * uihtml_setuserdata(UiHtml *item, gpointer data):
 * @widget: 
 *
 */
void
uihtml_setuserdata(UiXML * bondxml, UiHtml * item, gpointer data)
{
  g_assert(item);

  return gtk_object_set_user_data(GTK_OBJECT(item), data);
}

/**
 * uihtml_getcontainerchildren:
 */

GList *
uihtml_getcontainerchildren(UiXML * bondxml, UiHtml * list)
{
  g_assert(list);

  return gtk_container_children(GTK_CONTAINER(list));
}

UiHtml *
uihtml_handlebox_create(UiXML * uixml, xmlNode * node, UiHtml * parentwidget,
						gboolean * visible)
{
  UiHtml *handlebox;
  handlebox = uihtml_create(node);
  handlebox->widgettype = UiGeneral;
  if (uixml->defaultvisible)
	*visible = TRUE;
  else
	*visible = FALSE;
  return handlebox;
}

gint
uihtml_setwidgetsize(UiXML * uixml, UiHtml * widget, gint width, gint height)
{
  if (width < -1)
	return -1;
  if (height < -1)
	return -2;
  /* gtk_widget_set_size_request(widget, width, height); */
  return 0;
}
