#include <stdio.h>
#include <glib.h>

#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "uihtml.h"
#include "bc.h"

#include "uigeneral.h"
#include "uisignal.h"

/**
 * uihtml_signalconnect:
 * bondxml: XML document
 * object: Widget to add signal
 * name: Name of the signal (ie clicked)
 * argfunc: Function to be called on @name event.
 * func_data: Custom data to passed to @argfunc.
 * 
 * Returns: Signal id.
 */
guint
uihtml_signalconnect(UiXML * bondxml, UiHtml * object, const gchar * name,
                    void (*argfunc) (), void *func_data)
	{
	GtkTreeSelection *select;

	g_assert(object);
	if (uihtml_gettype(bondxml, object) == UiGrid)
		{
		select = gtk_tree_view_get_selection(GTK_TREE_VIEW(object));
		return g_signal_connect(G_OBJECT(select), name, G_CALLBACK(argfunc),
		                        func_data);
		}
	return g_signal_connect(G_OBJECT(object), name, G_CALLBACK(argfunc),
	                        func_data);
	}

/**
 * uihtml_signaldisconnectbyfunc:
 * 
 * Suspect bug in gtk stops this function from working correctly. 
 */
void
uihtml_signaldisconnectbyfunc(UiXML * bondxml, UiHtml * object,
							  void (*argfunc) (), void *data)
{
  GtkTreeSelection *select;
  
  g_assert(object);
  if (uihtml_gettype(bondxml, object) == UiGrid)
	{
	  select = gtk_tree_view_get_selection(GTK_TREE_VIEW(object));
	  g_signal_handlers_disconnect_by_func(G_OBJECT(select),
										   G_CALLBACK(argfunc), data);
	}
  else
	{
	  
	  /*		   int nu; */
	  g_signal_handlers_disconnect_by_func(G_OBJECT(object),
										   G_CALLBACK(argfunc), data);
	  /*		if (nu)
		debugmsg("%d signals disconnected on %s",nu,uihtml_getname(bondxml,object)); */
	}
  
}

/**
 * uihtml_signaldisconnect:
 * bondxml: Bond uixml documentent.
 * object: widget to disconnect signals on.
 * signalid: Signal id returned by uihtml_signalconncet().
 * 
 * 
 */
void
uihtml_signaldisconnect(UiXML * bondxml, UiHtml * object, gint signalid)
	{
	GtkTreeSelection *select;

	g_assert(object);
	if (uihtml_gettype(bondxml, object) == UiGrid)
		{
		select = gtk_tree_view_get_selection(GTK_TREE_VIEW(object));
		g_signal_handler_disconnect(G_OBJECT(select), signalid);
		}
	else
	     g_signal_handler_disconnect(G_OBJECT(object), signalid);
	}

static void
uihtml_signalconnect_keypress_func(UiHtml * widget, GdkEventKey * event,
                                  void *ptr)
	{
	gint tostop;
	UiSignalKey *key;
	typedef gint(*function) (void *, gchar * str);
	function func;

	key = ptr;
	func = key->func;
	tostop = func(key->userdata, event->string);
	/* stop event key press */
	if (tostop == 1)
		g_signal_stop_emission_by_name(G_OBJECT(widget), "key_press_event");
	}

gint
uihtml_signalconnect_keypress(UiXML * bondxml, UiHtml * widget,
                             UiSignalKey * uisignalkey)
	{
	g_assert(bondxml);
	g_assert(widget);

	uisignalkey->uixml = bondxml;
	uisignalkey->parentcallback = uihtml_signalconnect_keypress_func;
	uisignalkey->signal_id = g_signal_connect(G_OBJECT(widget),
	                         "key_press_event",
	                         G_CALLBACK
	                         (uihtml_signalconnect_keypress_func),
	                         uisignalkey);

	return 0;
	}

gint
uihtml_signaldisconnect_keypress(UiXML * bondxml, UiHtml * widget,
                                UiSignalKey * signalkey)
	{
	g_assert(bondxml);
	g_assert(widget);
	g_signal_handler_disconnect(G_OBJECT(widget), signalkey->signal_id);
	return 0;
	}
