/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_funcs.h"
#include "bc.h"
#include "uitoolbar.h"
#include "uihtml.h"
#include "uigeneral.h"
#include "parse.h"
#include "uigeneral.h"
#include "uibutton.h"
#include "uiradiobutton.h"

#include "uimenu.h"

#include "uicheckbutton.h"

/**
 * uihtml_toolbar_create:
 * @widgetnode: a xml node in the bond XML file.
 * @parentwidget: widget that will be the parent of this.
 * 
 * creates a toolbar
 * after creating a toolbar, you can append, prepend nad insert items 
 * or elements into the toolbar
 *
 * Options:
 * orientation="" sets the orientation of the toolbar either horizontal or vertical
 *
 *	Item Options:
 * 	<radio><check><button><entry><label><menu>
 */


UiHtml *
uihtml_toolbar_create(UiXML * uixml, xmlNode * node, UiHtml * parentwidget,
					  gboolean * visible)
{
  UiHtml *toolbar;
  gchar *content = NULL;
	
  toolbar = uihtml_create(node);
  toolbar->widgettype = UiGeneral;
  toolbar->vhordercounter=0;

  if (uixml->defaultvisible)
	*visible = TRUE;
  else
	*visible = FALSE;
	
  toolbar->visible = *visible;
	
  debuglog(50, "Creating toolbar ");
	
  content = xmlGetProp(node, "orientation");
  if (content && IsEqual(content, "vertical"))
	{
	  toolbar->vboxtrue = 1;
	  toolbar->openinghtml = mem_strdup("<table class=\"vtoolbar\">");
	}
  else
	{
	  toolbar->vboxtrue = 0;
	  toolbar->openinghtml = mem_strdup("<table class=\"htoolbar\">");
	}
	
  toolbar->closinghtml = mem_strdup("</table>");

  return toolbar;
}

gint
uihtml_toolbar_add(xmlNode * node, UiHtml *grandchild, UiHtml * parentwidget)
{
  UiHtml * cell;
  xmlChar * content;
  gint padding=-1;

  if (GTK_IS_WIDGET(grandchild) || GTK_IS_WIDGET(parentwidget))
	{
	  errormsg("Invalid widgeT Line: %d\n",xmlGetLineNo(node));
	  return -1;
	}
	  
  cell = uihtml_create(node);
  cell->widgettype = UiGeneral;
  mem_verify(parentwidget);
  mem_verify(grandchild);
  cell->children = g_list_append(cell->children,grandchild);
  debugmsg("Grandchild: %d cell %d\n",grandchild->widgettype,cell->widgettype);
  parentwidget->children = g_list_append(parentwidget->children,cell);
  cell->parent = parentwidget;
  grandchild->parent = cell;

  cell->order = parentwidget->vhordercounter;
  parentwidget->vhordercounter ++;

  content = xmlGetProp(node, "padding");
  if (content)
	padding = atoi(content);

	cell->openinghtml = mem_strdup("");
	
	if(parentwidget->vboxtrue)
	  /*vbox*/
	  {
		cell->openinghtml = mem_strdup("<tr><td class=\"vbox-cell\"");
		cell->closinghtml = mem_strdup("</td></tr>");
	  }
	else
	  /*hbox*/
	  {
/*		printf("i am an hbox cell, my order is %d\n", cell->order);*/
		/* if first cell*/	
		if(cell->order == 0)
		  cell->openinghtml = mem_strdup("<tr>");
		
		cell->openinghtml = 
		  mem_strdup_printf("%s<td class=\"hbox-cell\"", cell->openinghtml);
		
		cell->closinghtml = mem_strdup("</td>");
		
		/* if last cell -- TO DO (also in uihtml_toolbar_add)*/
		if(0)
		  cell->closinghtml = mem_strdup("</tr>");
	  }

	if(padding >= 0)
	  cell->openinghtml = mem_strdup_printf("%s style=\"padding:%d;\"", 
											cell->openinghtml, padding);
	
	cell->openinghtml = mem_strdup_printf("%s>", cell->openinghtml);
	
	debuglog(50, "Adding widget to toolbar. Yes it did happen");
	return 0;
}

gint
uihtml_toolbar_add_space(UiHtml * parentwidget)
{
  g_assert(parentwidget);
  gtk_toolbar_append_space(GTK_TOOLBAR(parentwidget));
  return 0;
}
