#include <libxml/parser.h>
#include <libxml/tree.h>
#include <glib.h>
#include <wx/wx.h>
#include "debug.h"

#include "gladexml.h"

BondFrame* theFrame;

static xmlNodePtr FindFirstChild(xmlNodePtr fromThisNode, gchar *childname);
static xmlChar* GetText(xmlNodePtr fromThisNode);
static xmlChar* GetTextFromPath(xmlNodePtr node, gchar* lvl1);
static xmlChar* GetTextFromPath(xmlNodePtr node, gchar* lvl1, gchar* lvl2);
static GladeXmlNode* GetWrappedNode(xmlNodePtr node);

gint listcompare(gconstpointer a, gconstpointer b);		

#include <iostream>
using namespace std;

// The purpose of this compartor is very specific.
// For given two xmlNodePtr, they point to the table cell items
// PRE-CONDITION: Arguments point the table cell items.
gint listcompare(gconstpointer a, gconstpointer b)
{
	g_assert(a);
	g_assert(b);
	
	xmlNodePtr nodeA = static_cast<xmlNodePtr>(a);
	xmlNodePtr nodeB = static_cast<xmlNodePtr>(b);
	
	xmlChar* a_left;
	xmlChar* a_top;	
	
	a_left = GetTextFromPath(nodeA, "child", "left_attach");
	g_assert(a_left);	
	a_top = GetTextFromPath(nodeA, "child", "top_attach");
	g_assert(a_top);
	
	xmlChar* b_left;
	xmlChar* b_top;
	
	b_left = GetTextFromPath(nodeB, "child", "left_attach");	
	g_assert(b_left);
	b_top = GetTextFromPath(nodeB, "child", "top_attach");
	g_assert(b_top);
	
	int left_diff, top_diff;
	
	// Convert from string to numeric value
	top_diff = atoi(reinterpret_cast<char*>(a_top)) - atoi(reinterpret_cast<char*>(b_top));	
	left_diff = atoi(reinterpret_cast<char*>(a_left)) - atoi(reinterpret_cast<char*>(b_left));
	
	// Well, if row values(=top values) are different, return the difference...
	int answer;
	if (top_diff) {
		answer = top_diff;
	} else {
		// If the two cell items are in same row,
		// return the difference of column values(=left values)
		answer = left_diff;
	}
	
	printf("(%s, %s) - (%s, %s) = %d\n",
		reinterpret_cast<char*>(a_left),
		reinterpret_cast<char*>(a_top),
		reinterpret_cast<char*>(b_left),
		reinterpret_cast<char*>(b_top),
		answer
	);
	return answer;
}




//////////////////////////////////////////////////////////////////////

wxObject* BondIgnore::getWidget()
{
	for (xmlNodePtr child = node->children; child; child = child->next)	{
		cout << "GtkHandle?: " << (char*)child->name << endl;
	}
	return new wxBoxSizer(wxHORIZONTAL);
	//return new wxButton(NULL, -1, "dummy");
}


//////////////////////////////////////////////////////////////////////

wxObject* BondBoxSizer::getWidget()
{
	return new wxBoxSizer(orient);
}


//////////////////////////////////////////////////////////////////////


wxObject* BondButton::getWidget()
{
	//return new wxButton(parentWidget /*parent*/,
	return new wxButton(theFrame /*parent*/,
		-1 /*id*/,
		//getText(findFirstChildNode("label")) /**/
		"Hello"
	);
}
//////////////////////////////////////////////////////////////////////

wxObject* BondStaticText::getWidget()
{
	return new wxStaticText(
		theFrame /*parent*/,
		-1 /*id*/,
		getText(findFirstChildNode("label"))
	);
}
//////////////////////////////////////////////////////////////////////

wxObject* BondComboBox::getWidget()
{
	return new wxComboBox(
		theFrame /*parent*/,
		-1 /*id*/,
"Hello"//		getText(findFirstChildNode("label"))
	);
}

//////////////////////////////////////////////////////////////////////

wxObject* BondTextCtrl::getWidget()
{
	return new wxTextCtrl(
		theFrame /*parent*/,
		-1 /*id*/,
		getText(findFirstChildNode("text"))
	);
}
//////////////////////////////////////////////////////////////////////

wxObject* BondCheckBox::getWidget()
{
	return new wxCheckBox(
		parentWidget /*parent*/,
		-1 /*id*/,
		getText(findFirstChildNode("label"))
	);
}
//////////////////////////////////////////////////////////////////////

wxObject* BondListCtrl::getWidget()
{
	return new wxListCtrl(
		parentWidget /*parent*/,
		-1 /*id*/
	);
}
//////////////////////////////////////////////////////////////////////

wxObject* BondScrolledWindow::getWidget()
{
	return new wxScrolledWindow(
		parentWidget /*parent*/,
		-1 /*id*/
	);
}
//////////////////////////////////////////////////////////////////////

//// wx.h doesn't include grid.h
//#include <wx/grid.h>
wxObject* BondGrid::getWidget()
{
//	wxGrid* retObj = new wxGrid(
//		theFrame /*parent*/,
//		-1 /*id*/
//	);
//	retObj->CreateGrid(3, 4);
//	return retObj;
	
	return new wxGridSizer(3, 4, 4, 4);
}
//////////////////////////////////////////////////////////////////////

wxObject* BondToolBar::getWidget()
{
	return new wxToolBar(
		theFrame /*parent*/,
		-1 /*id*/,
		wxPoint(-1, -1),
		wxSize(-1, -1),
		wxTB_HORIZONTAL
	);
}
//////////////////////////////////////////////////////////////////////

wxObject* BondStatusBar::getWidget()
{
	return new wxStatusBar(
		theFrame /*parent*/,
		-1 /*id*/
	);
}
//////////////////////////////////////////////////////////////////////

wxObject* BondMenuBar::getWidget()
{
	return new wxMenuBar();
}
//////////////////////////////////////////////////////////////////////

wxObject* BondWindow::getWidget()
{
//	return new wxFrame(
//		NULL,	
//		-1,
//		"Hello there!", wxPoint(50, 50), wxSize(400,300));

	theFrame = new BondFrame("Hello", wxPoint(50, 50), wxSize(400,300));

	return theFrame;
}
//////////////////////////////////////////////////////////////////////

static xmlNodePtr
FindFirstChild(xmlNodePtr fromThisNode, gchar *childname)
{
	xmlNode* child = NULL;

	g_assert(childname);
	g_assert(fromThisNode);

	printf("%s\n", __PRETTY_FUNCTION__);
	printf("Looking for a child:%s\n", childname);

	xmlChar* x_childname = BAD_CAST(childname);

	for (child = fromThisNode->children; child;	child = child->next) {
		printf("Inspecting %s\n", (char*)child->name);
		if ( !xmlStrcmp(child->name, x_childname) ) {
			break;
		}		
	}

	return child;
}

// Return all child matching the given name
// Return NULL if child is not found
GList* FindAllChildNodesByName(xmlNodePtr node, char* childname)
{
	GList* retList = NULL;
	xmlNode* childXmlNode;

	g_assert(node);
	g_assert(childname);
	
	printf("%s\n", __PRETTY_FUNCTION__);
	printf("Looking for a child:%s\n", childname);
	//xmlChar* x_childname = xmlCharStrdup(childname);
	xmlChar* x_childname = BAD_CAST(childname);

	for (childXmlNode = node->children; childXmlNode;
		childXmlNode = childXmlNode->next ) {
		
		printf("Inspecting [%s] ", (char*)childXmlNode->name);
		// If the given child name matches, add that node
		if (!xmlStrcmp(childXmlNode->name, x_childname)) {

			printf("Adding child %s to return list...\n", x_childname);
			retList = g_list_append(retList, childXmlNode);
		}
		
	}

	//free(x_childname);

	GList* curr;
	for (curr = g_list_first(retList);
		curr;
		curr = g_list_next(curr)) {
		printf("Returning... %s\n", (char*)static_cast<xmlNodePtr>(curr->data)->name);
	}

	retList = g_list_first(retList);

	return retList;
}


static xmlChar*
GetText(xmlNodePtr fromThisNode)
{
	g_assert(fromThisNode);	
	xmlNodePtr textNode = FindFirstChild(fromThisNode, "text");

	// We couldn't find the text node;
	if (!textNode || textNode->type == XML_ELEMENT_NODE) {
		return NULL;
	}
	
	// CHECK-LATER: I assume that once textNode exist, it is not a NULL.
	return textNode->content;
}

GladeXmlNode*
GetWrappedNode(xmlNodePtr node)
{

	g_assert(node);
	xmlChar* className = GetTextFromPath(node, "class");

	printf("Creating children GladeXmlNode %s...\n", (char *)className);
	printf("Getting a drawing manager object for %s...\n", (char *)className);


	GladeXmlNode *retNode = NULL;

	if (!xmlStrcmp(className,  BAD_CAST("GtkWindow")) ) {
		puts("Returning a window\n");
		retNode = new BondWindow(node);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkCList")) ) {
        retNode = new BondListCtrl(node);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkEntry")) ) {
        retNode = new BondTextCtrl(node);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkCombo")) ) {
        retNode = new BondComboBox(node);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkButton")) ) {
        retNode = new BondButton(node);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkText")) ) {
		retNode = new BondTextCtrl(node);
	}

	else if (!xmlStrcmp(className, BAD_CAST("GtkLabel")) ) {
		retNode = new BondStaticText(node);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkHBox")) ) {
       retNode = new BondBoxSizer(node, wxHORIZONTAL);
	}

	else if (!xmlStrcmp(className, BAD_CAST("GtkVBox")) ) {
        retNode = new BondBoxSizer(node, wxVERTICAL);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkScrolledWindow")) ) {
        retNode = new BondScrolledWindow(node);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkTable")) ) {
        retNode = new BondGrid(node);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkToolbar")) ) {
        retNode = new BondToolBar(node);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkStatusbar")) ) {
        retNode = new BondStatusBar(node);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkMenuBar")) ) {
        retNode = new BondMenuBar(node);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkHandleBox")) ) {
        retNode = new BondIgnore(node);
	}

	if (!retNode) {
		puts("********************\n");
		puts("Retuning NULL!!!\n");
		puts("********************\n");
		
		exit(EXIT_FAILURE);
	}

	return retNode;
}

// PRE-CONDITION: lvl1 != NULL
// RETURN: NULL if given name of the text node was not found.
xmlChar* GetTextFromPath(xmlNodePtr node, gchar* lvl1)
{
	g_assert(lvl1);
	
	xmlNodePtr lvl1_child = FindFirstChild(node, lvl1);
	
	if (!lvl1_child) {
		return NULL;
	}
	
	return GetText(lvl1_child);
}


// PRE-CONDITION: lvl1 != NULL && lvl2 != NULL
// RETURN: NULL if given name of the text node was not found.
xmlChar* GetTextFromPath(xmlNodePtr node, gchar* lvl1, gchar* lvl2)
{
	g_assert(lvl1);
	g_assert(lvl2);
	
	xmlNodePtr lvl1_child = FindFirstChild(node, lvl1);
	
	if (!lvl1_child) {
		printf("Level 1 child is NULL!\n");
		exit(1);
		return NULL;
	}
	
	xmlNodePtr lvl2_child = FindFirstChild(lvl1_child, lvl2);
	
	if (!lvl2_child) {
		printf("Level 2 child is NULL!\n");
		exit(1);
		return NULL;
	}
	
	for (xmlNodePtr curr = lvl2_child->children; curr; curr = curr->next) {
		printf("[%s] ", (char*)curr->name);
	}
	puts("\n");
	
	return GetText(lvl2_child);
}



//////////////////////////////////////////////////////////////////////

GladeXmlNode::GladeXmlNode(xmlNodePtr nod) : node(nod), currentWidget(NULL)
{
}


// RETURN: NULL if child is not found
xmlNodePtr GladeXmlNode::findFirstChildNode(char *childname)
{
	xmlNode* child = NULL;

	// ASPECT: error
	if (!childname) {
		errormsg("Null string error!");
		exit(EXIT_FAILURE);
	}
	g_assert(node);

	printf("%s\n", __PRETTY_FUNCTION__);
	printf("Looking for a child:%s\n", childname);

	//xmlChar* x_childname = xmlCharStrdup(childname);
	xmlChar* x_childname = BAD_CAST(childname);

	for (child = node->children; child;	child = child->next) {

		printf("Inspecting %s\n", (char*)child->name);
			
		if ( !xmlStrcmp(child->name, x_childname) ) {
			break;
		}
		
	}

	//free(x_childname);
	return child;
}


// Return all child matching the given name
// Return NULL if child is not found
GList* GladeXmlNode::findAllChildNodesByName(char* childname)
{
	GList* retList = NULL;
	xmlNode* childXmlNode;

	// ASPECT: error
	if (!childname) {
		errormsg("Null string error!");
		exit(EXIT_FAILURE);
	}

	if (!node) {
		errormsg("Node is null!");
		exit(EXIT_FAILURE);
	}

	printf("%s\n", __PRETTY_FUNCTION__);
	printf("Looking for a child:%s\n", childname);
	//xmlChar* x_childname = xmlCharStrdup(childname);
	xmlChar* x_childname = BAD_CAST(childname);

	for (childXmlNode = node->children;
		childXmlNode != NULL;
		childXmlNode = childXmlNode->next ) {
		
		printf("Inspecting %s\n", (char*)childXmlNode->name);
		// If the given child name matches, add that node
		if (!xmlStrcmp(childXmlNode->name, x_childname)) {

			printf("Adding child %s to return list...\n", x_childname);
			retList = g_list_append(retList, childXmlNode);
		}
		
	}

	//free(x_childname);

	GList* curr;
	for (curr = g_list_first(retList);
		curr;
		curr = g_list_next(curr)) {
		printf("Returning... %s\n", (char*)static_cast<xmlNodePtr>(curr->data)->name);
	}

	retList = g_list_first(retList);

	return retList;
}


xmlChar*
GladeXmlNode::getText(xmlNodePtr nodeWithTextChild)
{
//	return 
//		(new GladeXmlNode(nodeWithTextChild))->
//			findFirstChildNode("text")->
//				content;
	xmlNodePtr aTextNode = (new GladeXmlNode(nodeWithTextChild))->
			findFirstChildNode("text");

	if (!aTextNode) {
		puts("**********************************************************************\n");
		printf("Oh no! getText() failed because of returned text node is null\n");
		puts("**********************************************************************\n");

		return BAD_CAST("");
	}

	return aTextNode->content;
}

xmlChar*
GladeXmlNode::getContent()
{
	return node->content;
}



//////////////////////////////////////////////////////////////////////


// Recursive widget building function
//
void
GladeXmlNode::buildWidget(wxObject* parentWidgetParam)
{
	
	// Let's keep this. It may turn out to be useful later, hmm...
	parentWidget = static_cast<wxWindow*>(parentWidgetParam);

	// If node name is NOT "Widget", print warning message
	//xmlChar* x_elementname = xmlCharStrdup("widget");
	xmlChar* x_elementname = BAD_CAST("widget");
	if (xmlStrcmp(node->name, x_elementname)) {
		warningmsg("You can't build non-widget node!");
		exit(1);
	}
	
	xmlNodePtr classNode = findFirstChildNode("class");

	printf("You are building %s...\n", classNode->children->content);

	// END Dangerous Recursive!!!
	if (!classNode) {
		printf("Class node was null!\n");
		return;
	}

	// Let's get a this widget's type!!!
	printf("Getting a widget: %s\n", classNode->children->content);

	// Here we set current widget
	currentWidget = getWidget();


	if (parentWidget) {

		if (!strcmp(currentWidget->GetClassInfo()->GetClassName(),
			"wxMenuBar")) {
			puts("+++++ Menubar is being added now... +++++\n");
			static_cast<wxFrame*>(theFrame)->
				SetMenuBar(static_cast<wxMenuBar*>(currentWidget));
		} else if (!strcmp(currentWidget->GetClassInfo()->GetClassName(),
			"wxToolBar")) {
			theFrame->
				SetToolBar(static_cast<wxToolBar*>(currentWidget));
		} else if (!strcmp(currentWidget->GetClassInfo()->GetClassName(),
			"wxStatusBar")) {
			puts("+++++ wxStatusBar is being added now... +++++\n");	
			theFrame->
				SetStatusBar(static_cast<wxStatusBar*>(currentWidget));
			puts("+++++ wxStatusBar was added now... +++++\n");	
		} else {

			char *wName = parentWidget->GetClassInfo()->GetClassName();
			puts("--------------------\n");
			printf("%s %s\n", wName, currentWidget->GetClassInfo()->GetClassName());
			puts("--------------------\n");

			if (!strcmp(wName, "wxBoxSizer")) {
				puts("Parent is wxBoxSizer...\n");
				if (  currentWidget->IsKindOf(CLASSINFO(wxSizer))  ) {
					dynamic_cast<wxSizer*>(parentWidget)->Add(static_cast<wxSizer*>(currentWidget), 1);
				} else if (  currentWidget->IsKindOf(CLASSINFO(wxWindow))  ) {
					dynamic_cast<wxSizer*>(parentWidget)->Add(static_cast<wxWindow*>(currentWidget), 1);
				}
			} else if (!strcmp(wName, "wxGridSizer")) {
				puts("Parent is wxGridSizer...\n");
				if (  currentWidget->IsKindOf(CLASSINFO(wxSizer))  ) {
					dynamic_cast<wxSizer*>(parentWidget)->Add(static_cast<wxSizer*>(currentWidget), 1);
				} else if (  currentWidget->IsKindOf(CLASSINFO(wxWindow))  ) {
					dynamic_cast<wxSizer*>(parentWidget)->Add(static_cast<wxWindow*>(currentWidget), 1);
				}
			} else if (!strcmp(wName, "wxToolBar")) {
				puts("Parent is wxToolBar...\n");
#include "no.xpm"
				wxBitmap* toolBarBitmaps = new wxBitmap( no_xpm );

				static long currentX = 0;
/*
				GList* widgetNameList = FindAllChildNodesByName("widget");

				for (widgetNameList = g_list_first(widgetNameList);
					widgetNameList;
					widgetNameList = g_list_next(widgetNameList) ) {


					GladeXmlNode* childWidget = getGladeNodeByName(
						static_cast<xmlNodePtr>(widgetNameList->data)
						);
*/				
					printf("Putting tool button %s at %d\n", "bread", currentX);
					static_cast<wxToolBar*>(parentWidget)->AddTool(
						0,
						wxNullBitmap, //*(toolBarBitmaps),
						wxNullBitmap,
						FALSE,
						currentX += 100,
						-1,
						(wxObject *) NULL,
						"button1"//const_cast<char*>(getText(childWidget->findFirstChildNode("label")))
					);
//				} // End of for-loop
				
				return; // Forget about the children...
				
			} else {
				printf("!!! Parent is NOT wxSizer or wx*Bar series... %s\n", wName);
				
				if (!strcmp(wName, "wxFrame") &&
					currentWidget->IsKindOf(CLASSINFO(wxSizer))) {
						parentWidget->SetSizer(dynamic_cast<wxSizer*>(currentWidget));
						printf("+++++ Set Sizer = %s +++++\n",
							currentWidget->GetClassInfo()->GetClassName()
						);
						theFrame->topSizer = currentWidget;
				} else {
					printf("PARENT: %s ===================== CURRENT: %s\n\n\n",
					wName,
					currentWidget->GetClassInfo()->GetClassName()
					) ;
					printf("CURRENT NAME: %s\n", getText(findFirstChildNode("name")));
					exit(1);
				}
			}

		} // end of else from wxMenuBar/wxToolBar/wxStatusBar

		
	}


	//free(x_elementname);

	/////////////////////////////////////////////////////
	// Now children handling section

	GladeXmlNode* childWidget;
	GList* widgetNameList;

	// Get all child nodes whose name is "Widget"
	widgetNameList = findAllChildNodesByName("widget");

	// By setting widgetNameList to NULL value, traversing XML tree will be
	// ended.
#define DELETE_ME_LATER
	if (!strcmp(currentWidget->GetClassInfo()->GetClassName(),
		"wxMenuBar")) {
		widgetNameList = NULL;
	}
	if (!strcmp(currentWidget->GetClassInfo()->GetClassName(),
		"wxComboBox")) {
		widgetNameList = NULL;
	}
#define DELETE_ME_LATER

	if (currentWidget->IsKindOf(CLASSINFO(wxGridSizer))) {
		g_list_sort(widgetNameList, listcompare);
	}
	
	// This list contains children
	for (widgetNameList = g_list_first(widgetNameList);
		widgetNameList;
		widgetNameList = g_list_next(widgetNameList) ) {

		GladeXmlNode* childWidget = getGladeNodeByName(
			static_cast<xmlNodePtr>(widgetNameList->data)
		);

		if (!childWidget) {
			printf("CHILD IS NULL at %s %s\n",
				static_cast<xmlNodePtr>(widgetNameList->data)->name, 
				(new GladeXmlNode(static_cast<xmlNodePtr>(widgetNameList->data))
				)->findFirstChildNode("class")->children->content
			);
		}

		// Old way of disabling GtkHandleBox problem
		// DELETE THIS LATER
/*		if (childWidget && childWidget->gtkclassname &&
			!strcmp(childWidget->gtkclassname, "GtkHandleBox")) {
				puts("**********\n");
				puts("Skipping GtkHandleBox\n");
				puts("**********\n");
				continue;
		} */
		
			// currentWidget will become a parent in the following call
			childWidget->buildWidget(currentWidget);

	}


	// POST part of buildWidget().
	// Verify which window layout class is used.
#define ISCLASS(a)	!strcmp(currentWidget->GetClassInfo()->GetClassName(), a)

	if (ISCLASS("wxGridSizer") && theFrame->topSizer == currentWidget) {
		
		puts("+++++ SETTING wxGridSizer +++++\n\n\n");
		theFrame->SetAutoLayout( TRUE );
		
		static_cast<wxBoxSizer*>(currentWidget)->Fit( theFrame );
		static_cast<wxBoxSizer*>(currentWidget)->SetSizeHints( theFrame );
	} else if (ISCLASS("wxBoxSizer") && theFrame->topSizer == currentWidget) {
			
		puts("+++++ SETTING wxGridSizer +++++\n\n\n");
		theFrame->SetAutoLayout( TRUE );

		static_cast<wxBoxSizer*>(currentWidget)->Fit( theFrame );
		static_cast<wxBoxSizer*>(currentWidget)->SetSizeHints( theFrame );
	} else if (ISCLASS("wxToolBar")) {
		static_cast<wxToolBar*>(currentWidget)->Realize();
		cout << "realizing" << endl;
		getchar();
	}
}


GladeXmlNode*
//GladeXmlNode::getGladeNodeByName(xmlChar* classname)
GladeXmlNode::getGladeNodeByName(xmlNodePtr tempNode)
{

	xmlChar* className =
		this->getText((new GladeXmlNode(tempNode))->
			findFirstChildNode("class"));

	printf("Creating children GladeXmlNode %s...\n", (char *)className);
	printf("Getting a drawing manager object for %s...\n", (char *)className);


	GladeXmlNode *retNode = NULL;

	if (!xmlStrcmp(className,  BAD_CAST("GtkWindow")) ) {
		puts("Returning a window\n");
		retNode = new BondWindow(tempNode);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkCList")) ) {
        retNode = new BondListCtrl(tempNode);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkEntry")) ) {
        retNode = new BondTextCtrl(tempNode);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkCombo")) ) {
        retNode = new BondComboBox(tempNode);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkButton")) ) {
        retNode = new BondButton(tempNode);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkText")) ) {
		retNode = new BondTextCtrl(tempNode);
	}

	else if (!xmlStrcmp(className, BAD_CAST("GtkLabel")) ) {
		retNode = new BondStaticText(tempNode);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkHBox")) ) {
       retNode = new BondBoxSizer(tempNode, wxHORIZONTAL);
	}

	else if (!xmlStrcmp(className, BAD_CAST("GtkVBox")) ) {
        retNode = new BondBoxSizer(tempNode, wxVERTICAL);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkScrolledWindow")) ) {
        retNode = new BondScrolledWindow(tempNode);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkTable")) ) {
        retNode = new BondGrid(tempNode);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkToolbar")) ) {
        retNode = new BondToolBar(tempNode);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkStatusbar")) ) {
        retNode = new BondStatusBar(tempNode);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkMenuBar")) ) {
        retNode = new BondMenuBar(tempNode);
	}

    else if (!xmlStrcmp(className, BAD_CAST("GtkHandleBox")) ) {
        retNode = new BondIgnore(tempNode);
	}


	if (retNode) {
		retNode->gtkclassname = (char*) className;
	} else {
		puts("********************\n");
		puts("Retuning NULL!!!\n");
		puts("********************\n");
	}

	return retNode;
}


