/* Francis: Standard ANSI/ISO C libraries */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Francis: No idea about what is glob.h ? */
#include <glob.h>

/* Dynamic loading... */
#include <dlfcn.h>

#include <glib.h>

/* wxWindow module */
#include <wx/wx.h>
#include "uiwxwink.h"
#include "uiwxwinglobal.h"

#include "debug.h"

#include "../src/uiplugin.h"

#include <glade/glade.h>


typedef struct
{
	const gchar *name;
	GList *list;
} widget_prefix_data;

GladeXML *globalxml;
char appxml[1024];

/**
 * plugin_exportfunctions:
 * @plugin: plugin to add function information
 *
 *
 * Returns: non-zero on error
 */
static gint
plugin_exportfunctions(Plugin * plugin)
{
	gint retval = 0;

	retval -= plugin_addfunction(plugin, "init", uiwxwin_init);
	retval -= plugin_addfunction(plugin, "main", uiwxwin_main);
	retval -= plugin_addfunction(plugin, "cleanup", uiwxwin_cleanup);

	return retval;
}

/**
 * plugin_init:
 * @plugin: plugin structure passed in
 *
 * Will build a list of functions in the library and return them to parent library.
 *
 * Returns: 0 on success
 */
gint
plugin_init(Plugin * plugin)
{
	if (!strcmp(plugin->name, "uiwxwin")) {
		warningmsg("Not the correct library, can not continue.");
		return -1;
	}

	printf("Loading wxWindow User Interface library...\n");

	plugin_exportfunctions(plugin);

	return 0;
}


extern int wxEntry(int, char *[]);

/**
 * uiwxwin_init:
 * @xmlfile: Glade XML which contains the UI descripition
 *
 * Create the global instance of xmlwindow, which will contain a list of all the windows.
 * This will make it nice and easy for window creating.
 *
 * Returns: non-zero on error
 */
gint
uiwxwin_init(gchar * xmlfile, gint argc, gchar * argv[])
{
//   gtk_init(&argc, &argv);
//   glade_init();
//
//   globalxml = glade_xml_new(xmlfile, NULL);
//
//   if (!globalxml)
//      {
//      printf("Creating XML for menu failed: %s.\n", xmlfile);
//      return -1;
//      }
//
//   glade_xml_signal_autoconnect(globalxml);
//   strcpy(appxml, xmlfile);
//   return 0;

	return  wxEntry(argc, argv);
}

void
uiwxwin_cleanup()
{
//   gtk_exit(0);
// I think there are nothing to add here since destroyer will be called
// automatically.
}



void
uiwxwin_main()
{
	int exit_value;


	// return exit_value;
}

static void
widget_prefix_add_to_list(gchar * name, gpointer value, widget_prefix_data * data)
{
	if (!strncmp(data->name, name, strlen(data->name)))
		data->list = g_list_prepend(data->list, value);
}

/**
 * glade_xml_get_widget_prefix_by_long_name:
 * @self: the GladeXML Object*
 * @longname: the long name of the widget prefix
 *
 * This function is used to get a list of pointers to the GtkWidget(s)
 * with long names that start with the string @longname in the interface descri * You would use this if you have to do something  to all of these widgets
 * after loading.
 *
 * Returns: A list of the widget that match @name as the start of their
 * name, or %NULL if none exists.
 */

GList *
glade_xml_get_widget_prefix_by_long_name(GladeXML * self, const gchar * prefix)
{
	widget_prefix_data data;
	GHashTable *longname_hash;

	struct _GladeXMLPrivate *priv = self->priv;

	data.name = prefix;
	data.list = NULL;

	longname_hash = priv->longname_hash;

	g_hash_table_foreach(longname_hash, (GHFunc) widget_prefix_add_to_list, &data);

	return data.list;
}









//////////////////////////////////////////////////////////
// uiwxwinglobal.c


#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <glib.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <gtk/gtksignal.h>

#include <glade/glade.h>

#include "uigtk.h"
#include "uigtkglobal.h"
#include "debug.h"
#include "libglade_patch.h"

/**
 * uigtk_getparentformname:
 *
 * Finds the name of the window the widget is in.
 *
 * Returns: %NULL on error
 */
gchar *
uiwxwin_getparentformname(UiWidget * inputwidget)
{
	wxWindow *widget;

	g_assert(inputwidget);

	widget = inputwidget->GetParent();

	if (widget == NULL)
		warningmsg("I was unable to find the widget parent");

	// Note that wxWindow::GetName returns wxString
	return (gchar *) widget->GetName();
}

/**
 * uigtk_getparentformwidget:
 *
 * Finds the widget of the window the widget is in.
 * The gtk stuff wouldn't work for some callbacks, so this seems
 * to get around whatever the problem was.
 *
 * Returns: %NULL on error
 */
UiWidget *
uiwxwin_getparentformwidget(UiWidget * inputwidget)
{
	gchar *widgetname =
		(gchar *) static_cast<wxWindow>(inputwidget)->;

	gchar *tmp;
	wxWindow *retwidget;
	gint i, j;

	g_assert(inputwidget);

	tmp = (gchar *) g_malloc(strlen(widgetname));
	tmp[0] = 0;
	for (i = 0; i < strlen(widgetname); i++) {
		if (widgetname[i] == '.') {
			for (j = 0; j < i; j++) {
				tmp[j] = widgetname[j];
			}
			tmp[j] = 0;
			break;
		}
	}

	debugmsg("trying to get %s from %s.\n", tmp, widgetname);
	retwidget = glade_xml_get_widget(globalxml, tmp);
	g_free(tmp);
	debugmsg("and ended up with %s.\n", (gchar *) glade_get_widget_long_name(retwidget));
	return retwidget;
}

/* returns local copy of the xml file */
UiXML *
uigtk_getxmlsubset(gchar * widgetname)
{
	GtkWidget *widget;

	widget = glade_xml_get_widget(globalxml, widgetname);
	if (widget == NULL)
		return NULL;

	return glade_get_widget_tree(widget);
}

/**
 * uiwxwin_getwidgetshortname
 *
 * Cut parent's name part
 */
gchar *
uiwxwin_getwidgetshortname(UiWidget * widget)
{
	g_assert(widget);
//	return (gchar *) glade_get_widget_name(widget);

	char *widgetlongname = static_cast<wxWindow>(widget)->GetName();

	name = strrchr(widgetlongname, (int)'.');

	// Francis - Hmm hope this is robust enough
	// WARNING - The following code treats zero-length string (i.e. "")
	// as an invalid case.
	if (!name && strlen(widgetlongname) > 0) {
		name = widgetlongname;
	} else if (!name) {
		debugmsg("Error occurred! no '.' found in the long name\n");
	}

	return name;
}

gchar *
uiwxwin_getname(UiWidget * widget)
{
	g_assert(widget);
	//return (gchar *) glade_get_widget_long_name(widget);
	return (gchar *) static_cast<wxWindow>(widget)->GetName();
}

UiWidget *
uiwxwin_getwidget(gchar * widgetname)
{
	wxWindow *widget;

	g_assert(widgetname);

	widget = glade_xml_get_widget_by_long_name(globalxml, widgetname);

	if (widget == NULL)
		widget = glade_xml_get_widget(globalxml, widgetname);

	return widget;
}

GList *
uiwxwin_widgetchildren(gchar * widgetname)
{
	gchar *tmpstr;
	GList *retlist;

	if (widgetname == NULL)
		tmpstr = (gchar *) g_strdup("");
	else if (strlen(widgetname) == 0)
		tmpstr = g_strdup("");
	else
		tmpstr = (gchar *) g_strdup_printf("%s.", widgetname);

	retlist = glade_xml_get_widget_prefix_by_long_name(globalxml, tmpstr);
	g_free(tmpstr);

	return retlist;
}

/**
 * uiwxwin_widgetlongname
 * @widgetname
 *
 * Return long widget name from a given shortname widgetname
 */
gchar *
uiwxwin_widgetlongname(gchar * widgetname)
{

	// First, we get a widget from a short widget name
	wxWindow *widget = uiwxwin_getwidget(widgetname);

	if (widget == NULL)
		return NULL;

	// Now we can get a long name from widget
	return (gchar *) widget->GetName();
}


/**
 * Map to general type name
 * @widget
 *
 */
gint
uiwxwin_gettype(UiWidget * widget)
{
	char* widgettype;
	wxClassInfo *classInfo;

	g_assert(widget);

	// Get the class name of a given widget
	classInfo = static_cast<wxObject>(widget)->GetClassInfo();
	widgettype = classInfo.GetClassName();

	if (widgettype == "wxTextCtrl")
		return EntryBox;
	if (widgettype == "wxComboBox")
		return DropDown;
	if (widgettype == "wxListBox")
		return List;
	if (widgettype == "wxListCtrl")
		return Grid;
	if (widgettype == "wxCheckBox")
		return CheckButton;
	if (widgettype == "wxStaticText")
		return TextBox;
	if (widgettype == GTK_TYPE_LABEL)
		return Label;
	if (widgettype == "wxToggleButton")
		return ToggleButton;
	if (widgettype == "wxRadioButton")
		return RadioButton;
	if (widgettype == "wxWindow")
		return Form;
	if (widgettype == "wxFrame")
		return Frame;
	if (widgettype == "wxButton")
		return Button;
	if (widgettype == "wxSpinButton") /* According to andru, he wants this as a temporary solution */
	return EntryBox;

	/* debugmsg("Write me!, i dont understand this type on widget %s", uigtk_getname(widget)); */

	// If we can't find any from above...
	return General;
}

/*//////////////////////////////////////////// */

/* Francis  */

/* BEGIN */

/**
 * uigtk_getcontainerchildren:
 */

GList *
uiwxwin_getcontainerchildren(UiWidget *container)
{
	g_assert(container);

	wxList children = static_cast<wxWindow>(container)->GetChildren();

	// Convert the wxList to the GList
	wxNode *node;
	GList *list = NULL;
	for (node = children.GetFirst(); node; node = node->GetNext()) {
		list = g_list_append(list, node->GetData());
	}

	return list;
}

/**
 * uigtk_setuserdata(UiWidget *item, gpointer data):
 * @widget:
 *
 */
void
uiwxwin_setuserdata(UiWidget * item, gpointer data)
{
	g_assert(item);

//   return gtk_object_set_user_data(GTK_OBJECT(item), data);

	// Francis - Hmm what should I do?
	static_cast<wxWindow>(item)->SetRefData(data);
	// or
	debugmsg("Not supported in wxWindow");
}

/* END */

/*//////////////////////////////////////////// */

/**
 * uigtk_dropdown_getparent:
 *
 *
 */
UiWidget *
uiwxwin_dropdown_getparent(UiWidget * widget)
{
	gint i;
	gchar *widgetname;
	//GtkWidget *retwidget = NULL;
	wxWindow *retwidget = NULL;

	g_assert(widget);

	widgetname = g_strdup(uiwxwin_getname(widget));

	for (i = strlen(widgetname) - 1; i >= 0; i--) {
		if (widgetname[i] == '.') {
         widgetname[i] = 0;
         retwidget = uiwxwin_getwidget(widgetname);
         break;
		}
		widgetname[i] = 0;
	}

	g_free(widgetname);
	return retwidget;
}

gint
uiwxwin_dropdown_clear(UiWidget * widget)
{
	static_cast<ListBox>(uiwxwin_dropdown_getlistbox(widget))->clear();

	return 0;
}

/**
 * uiwxwin_dropdown_getentrybox:
 *
 *
 */
UiWidget *
uiwxwin_dropdown_getentrybox(UiWidget * widget)
   {
   gchar *widgetname;
   GList *list;

   g_assert(widget);

   widgetname = (gchar *) glade_get_widget_long_name(widget);
   list = (GList *) glade_xml_get_widget_prefix_by_long_name(globalxml, widgetname);
   while (list != NULL)
      {
      if (GTK_WIDGET_TYPE(list->data) == GTK_TYPE_ENTRY)
         {
         return list->data;
         }
      list = list->next;
      }
   return NULL;
   }

/**
 * uiwxwin_dropdown_getlistbox:
 *
 * Get the widget which contains the dropdown list.
 *
 */
UiWidget *
uiwxwin_dropdown_getlistbox(UiWidget * widget)
{
	g_assert(widget);
	if (uiwxwin_gettype(widget) != DropDown) {
		warningmsg("Your not passing in a valid widget, i can not find the list in it. %s", uiwxwin_getname(widget));
		return NULL;
	}
	return GTK_COMBO(widget)->list;
}

/**
 * uiwxwin_dropdown_getselection:
 * @widget: A list widget (child of dropdown widget) to find out what is the selection
 *
 * Gets a GList of what has been selected in the dropdown box.  Use
 * uiwxwin_dropdown_getlistbox() to get a list of widgets.
 *
 * Returns: GList, do not free.
 */
GList *
uiwxwin_dropdown_getselection(UiWidget * widget)
   {
   g_assert(widget);
   if (uiwxwin_gettype(widget) != List)
      {
      warningmsg("You are passing in an invalid widget (%s), i want a list widget", uiwxwin_getname(widget));
      return NULL;
      }
   return (GTK_LIST(widget))->selection;
   }

/*/////////////////////////////////////////////////// */

/* Francis added */

/* BEGIN */

/**
 * uiwxwin_dropdown_setselection:
 * @widget:
 * @pos:
 */
void
uiwxwin_dropdown_setselection(UiWidget * widget, gint pos)
{
	g_assert(widget);

	static_cast<wxListBox>(widget)->SetSelection(pos);
}

/**
 * uiwxwin_dropdown_draw:
 * @widget:
 * @values:
 */
void
uiwxwin_dropdown_draw(UiWidget * widget, GList * values)
{
   g_assert(widget);

   //gtk_combo_set_popdown_strings(GTK_COMBO(widget), values);
}

/**
 * uiwxwin_getuserdata(UiWidget *item):
 * @widget:
 *
 */
void *
uiwxwin_dropdown_getuserdata(UiWidget * item)
{
	g_assert(item);


	//return gtk_object_get_user_data(GTK_OBJECT(item));
}

/* END */

/*/////////////////////////////////////////////////// */

/**
 * uiwxwin_issignalattached:
 *
 *
 */
gint
uiwxwin_issignalattached(UiWidget * widget, gchar * name)
   {
   GtkObject *obj;
   int i, nsignals;

   g_assert(widget);
   obj = GTK_OBJECT(widget);

   g_assert(widget != NULL);
   g_assert(name != NULL);

   nsignals = obj->klass->nsignals;

   for (i = 0; i < nsignals; i++)
      {
      if (strcmp(gtk_signal_name(obj->klass->signals[i]), name) == 0)
         return 1;
      /* message("signal: %s", gtk_signal_name(obj->klass->signals[i])); */
      }
   return 0;
   }

/* take note. this whole file happened by accident. */

/**
 * uiwxwin_getwidgetinform:
 *
 *
 */
UiWidget *
uiwxwin_getwidgetinform(gchar * formname, gchar * widgetname)
   {
   GtkWidget *widget;
   GList *widgetlist;
   gchar *str;
   int length;

   widget = uiwxwin_getwidget(formname);

   if (widget == NULL)
      {
      warningmsg("can't find window widget");
      return NULL;
      }

   str = (gchar *) g_malloc(sizeof(glade_get_widget_long_name(widget) + 1));
   str = strcpy(str, glade_get_widget_long_name(widget));
   length = strlen(glade_get_widget_long_name(widget));
   str[length] = '.';
   debugmsg("window name %s", str);

   widgetlist = (GList *) glade_xml_get_widget_prefix_by_long_name(globalxml, str);
   widgetlist = g_list_first(widgetlist);

   g_free(str);
   /* one step closer to the edge */
   /* and i'd be damned if i'm going to break. to the revolution */
   while (widgetlist != NULL)
      {
      widget = widgetlist->data;
      if (widget != NULL)
         {
         if (strcasecmp(glade_get_widget_name(widget), widgetname) == 0)
            {
            debugmsg("Your lucky widget is *** %s", glade_get_widget_long_name(widget));
            return widget;
            }
         }
      if (widgetlist->next != NULL)
         widgetlist = widgetlist->next;
      else
         break;
      }
   if (widgetlist == NULL)
      debugmsg("widgetlist is null, this is cause you are idiot andru, or maybe other reasons");
   return NULL;
   }

/**
 * uiwxwin_getvalue:
 *
 * todo, someone needs to say something.
 *
 */
gint
uiwxwin_getvalue(UiWidget * widget, gchar ** value)
   {
   gint state;
   GtkType widgettype;

   g_assert(widget);

   widgettype = GTK_WIDGET_TYPE(widget);
   if (widgettype == GTK_TYPE_ENTRY)
      {
      *value = gtk_entry_get_text(GTK_ENTRY(widget));
      return 0;
      }
   else if (widgettype == GTK_TYPE_TOGGLE_BUTTON || widgettype == GTK_TYPE_CHECK_BUTTON
            || widgettype == GTK_TYPE_RADIO_BUTTON)
      {
      state = (gint) gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
      *value = g_strdup_printf("%d", state);
#warning "Memory leak here"

      return 0;
      }
   else if (widgettype == GTK_TYPE_SPIN_BUTTON)
      {
      GtkSpinButton *spin = GTK_SPIN_BUTTON(widget);

      *value = gtk_entry_get_text(&spin->entry);
      return 0;
      }

   else if (widgettype == GTK_TYPE_TEXT)
      {
      *value = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
      return 0;
      }

   else if (widgettype == gtk_combo_get_type())
      {
      errormsg("This should be handled outside of this library as this is complex!");
      return -3;
      }
   errormsg("This widget: '%s' hasn't been added yet. You are welcome to code it.", uiwxwin_getname(widget));
   return -1;
   }

/**
 * uiwxwin_setvalue:
 * @widget:
 * @value: String value to set the widget to.
 *
 * Set the value in a widget to @value.  This works on most widget types, expect
 * for a few for obvious reasons, like clists.
 *
 */
gint
uiwxwin_setvalue(UiWidget * widget, gchar * value)
   {
   gint pos;						  /* pos is temp */
   gboolean state;
   GtkType widgettype;

   g_assert(widget);
   g_assert(value);

   widgettype = GTK_WIDGET_TYPE(widget);
   if (widgettype == GTK_TYPE_ENTRY)
      {
      gtk_entry_set_text(GTK_ENTRY(widget), value);
      return 0;
      }

   else if (widgettype == GTK_TYPE_TOGGLE_BUTTON || widgettype == GTK_TYPE_CHECK_BUTTON
            || widgettype == GTK_TYPE_RADIO_BUTTON)
      {
      state = FALSE;
      if (strcasecmp(value, "'t'") == 0 || strcmp(value, "1") == 0 || strcasecmp(value, "T") == 0 || strcasecmp(value, "Y") == 0)
         state = TRUE;

      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), (gboolean) state);
      return 0;
      }
   else if (widgettype == GTK_TYPE_TEXT)
      {
      gtk_editable_delete_text(GTK_EDITABLE(widget), 0, -1);
      if (value != NULL)
         gtk_editable_insert_text(GTK_EDITABLE(widget), value, strlen(value), &pos);
      gtk_editable_set_position(GTK_EDITABLE(widget), 0);
      return 0;
      }
   else if (widgettype == GTK_TYPE_LABEL)
      {
      gtk_label_set_text(GTK_LABEL(widget), value);
      return 0;
      }

   else if (widgettype == GTK_TYPE_SPIN_BUTTON)
      {
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), atof(value));
      return 0;
      }
   else if (widgettype == gtk_combo_get_type())
      {
      errormsg("This should behandled by bond not the ui plugin");
      return -2;
      }
   errormsg("This widget hasn't been added yet. You are welcome to code it. This"
            "is for widget %s", uiwxwin_getname(widget));
   return -1;
   }

/**
 * uiwxwin_dropdown_gettype:
 *
 * find out if a dropdown widget allows the user to type text into it or that the drop
 *	down box can be empty. This is configured in glade about the dropdown properties
 */
gint
uiwxwin_downdown_gettype(UiWidget * widget)
   {
   g_assert(widget);
   warningmsg("WRITE ME");
   return 1;
   }

/**
 * uiwxwin_visible:
 *
 * Works out if @widget is currently visible.
 *
 *
 */
gint
uiwxwin_visible(UiWidget * widget, gboolean toset)
   {
   g_assert(widget);

   gtk_widget_show(widget);
   return 0;
   }

/**
 * uiwxwin_isvisible:
 *
 * Work out if a widget is visible.
 *
 *
 */
gboolean
uiwxwin_isvisible(UiWidget * widget)
   {
   g_assert(widget);
   errormsg("WRITE ME");
   return 1;
   }

/*////////////////////////////////////////////////////////// */

/* Francis added */

/* BEGIN */

/**
 * uiwxwin_showwidget:
 *
 * Not to destroy existing uiwxwin_visible
 *
 *
 */
void
uiwxwin_showwidget(UiWidget * widget)
{
   g_assert(widget);

	static_cast<wxWindow>(widget)->Show();
}

/**
 * uiwxwin_hidewidget:
 *
 * Not to destroy existing uiwxwin_visible
 *
 *
 */
void
uiwxwin_hidewidget(UiWidget * widget)
{
	g_assert(widget);

	static_cast<wxWindow>(widget)->Hide();
}

GList *
uiwxwin_tab_settaborder(GList * list_orig)
   {
   GList *list = NULL;
   GList *list_runner;
   GList *ordered = NULL;
   GList *tab_ordered = NULL;

   GtkWidget *temp_widget;

   gint longest;
   gint xval;
   gint num_added = 0;

   list = g_list_copy(list_orig);

   if (list == NULL)
      return NULL;
   list_runner = g_list_first(list);
   temp_widget = list_runner->data;
   longest = temp_widget->allocation.width;
   xval = temp_widget->allocation.x;

   while (list_runner)
      {
      list_runner = g_list_next(list_runner);
      if (list_runner == NULL)
         break;
      temp_widget = list_runner->data;
      if ((temp_widget->allocation.width) > longest)
         {
         longest = temp_widget->allocation.width;
         }
      if ((temp_widget->allocation.x) < xval)
         {
         xval = temp_widget->allocation.x;
         }
      }

   ordered = g_list_sort(list, (GCompareFunc) uiwxwin_tab_sortfuncx);
   ordered = g_list_sort(ordered, (GCompareFunc) uiwxwin_tab_sortfuncy);

   while (num_added < g_list_length(ordered))
      {
      list_runner = g_list_first(ordered);

      while (list_runner)
         {
         temp_widget = list_runner->data;
         if (temp_widget->allocation.x >= xval && temp_widget->allocation.x < (xval + longest))
            {
            tab_ordered = g_list_append(tab_ordered, temp_widget);
            num_added++;
            }
         list_runner = g_list_next(list_runner);
         }

      xval = uiwxwin_tab_getnextxval(xval, longest, ordered);

      }
   return tab_ordered;

   }

/**
  * From tab.c
  *
  */
gint
uiwxwin_tab_sortfuncx(void *wid1, void *wid2)
   {
   gint x_diff;
   GtkWidget *twid1 = (GtkWidget *) wid1;
   GtkWidget *twid2 = (GtkWidget *) wid2;

   x_diff = twid2->allocation.x - twid1->allocation.x;
   return x_diff;
   }

gint
uiwxwin_tab_sortfuncy(void *wid1, void *wid2)
   {
   gint y_diff;
   GtkWidget *twid1 = (GtkWidget *) wid1;
   GtkWidget *twid2 = (GtkWidget *) wid2;

   y_diff = twid2->allocation.x - twid1->allocation.x;
   return y_diff;
   }

gint
uiwxwin_tab_getnextxval(gint xval, gint longest, GList * list1)
   {

   GList *list_runner;
   GtkWidget *temp_widget;
   gint next_xval = 0;

   if (list1 == NULL)
      return -1;

   list_runner = g_list_first(list1);
   if (list_runner == NULL)
      return -1;
   temp_widget = list_runner->data;
   while (list_runner)
      {
      list_runner = g_list_next(list_runner);
      temp_widget = list_runner->data;
      if ((temp_widget->allocation.x) < next_xval && (temp_widget->allocation.x) > (xval + longest))
         {
         next_xval = temp_widget->allocation.x;
         }

      return 0;
      }

   return next_xval;
   }

gchar *
uiwxwin_gettooltiptext(UiWidget * widget)
   {
   GtkTooltipsData *tooltip;

   g_assert(widget);

   tooltip = gtk_tooltips_data_get(GTK_WIDGET(widget));

   if (tooltip != NULL)
      {
      return g_strdup(tooltip->tip_text);
      }

   return NULL;
   }

void
uiwxwin_setclisttext(UiWidget * clist, gint row, gint column, const gchar * text)
   {
   g_assert(clist);

   gtk_clist_set_text(GTK_CLIST(clist), row, column, text);
   }

gchar *
uiwxwin_hitmenow_getlabel(UiWidget * obj)
   {

   GtkObject *object = GTK_OBJECT(obj);

   GtkButton *button;
   GtkBox *box;
   GtkLabel *lab;
   GtkBoxChild *child;

   // GtkArg *arg;
   gchar *label = NULL;
   GList *list = NULL, *listrunner = NULL;

   button = GTK_BUTTON(object);

   if (GTK_BIN(button)->child && GTK_IS_LABEL(GTK_BIN(button)->child))
      {

      // GTK_VALUE_STRING (*arg) = g_strdup (GTK_LABEL (GTK_BIN (button)->child)->label);
      label = g_strdup(GTK_LABEL(GTK_BIN(button)->child)->label);

      debugmsg("found label %s", label);
      }
   else
      {
      // GTK_VALUE_STRING (*arg) = NULL;
      if (GTK_IS_BOX(GTK_BIN(button)->child))
         {
         box = (GTK_BOX(GTK_BIN(button)->child));
         list = box->children;
         listrunner = g_list_first(list);
         while (listrunner)
            {
            child = listrunner->data;
            if (GTK_IS_LABEL(child->widget))
               {

               lab = GTK_LABEL(child->widget);
               label = g_strdup(lab->label);

               debugmsg("found me a label %s", label);
               }

            listrunner = g_list_next(listrunner);
            }
         }
      else
         {

         label = NULL;
         debugmsg("found no label");
         }
      }

   // return (gchar *)arg;
   g_list_free(list);
   g_list_free(listrunner);

   return label;
   }

guint
uiwxwin_signalconnect(UiWidget * object, const gchar * name, void (*argfunc) (), void *func_data)
   {
   g_assert(object);

   return gtk_signal_connect(GTK_OBJECT(object), name, GTK_SIGNAL_FUNC(argfunc), func_data);
   }

void
uiwxwin_signaldisconnectbyfunc(UiWidget * object, void (*argfunc) (), void *data)
   {
   g_assert(object);

   gtk_signal_disconnect_by_func(GTK_OBJECT(object), GTK_SIGNAL_FUNC(argfunc), data);
   }

void
uiwxwin_signalemitbyname_arg_pUiEvent_pgboolean(UiWidget * object, const gchar * name, UiEvent * event, gboolean * retval)
   {
   g_assert(object);

   gtk_signal_emit_by_name(GTK_OBJECT(object), name, event, retval);

   }

void
uiwxwin_signalemitstopbyname(UiWidget * object, const gchar * name)
   {
   g_assert(object);

   gtk_signal_emit_stop_by_name(GTK_OBJECT(object), name);
   }

void
uiwxwin_event_handlekeyevent(UiEvent * event, void *data)
   {
   GdkEventKey *gdkeventkey = (GdkEventKey *) event;
   int *stopit;

   stopit = (int *)data;

   switch (gdkeventkey->keyval)
      {
   case GDK_Tab:
   case GDK_Return:
      *stopit = 1;
      break;
      }
   }

void
uiwxwin_event_setvalue(UiEvent * event, UiEventType type, UiEventValue eventvalue)
   {

   if (type == UI_KEY)
      {
      switch (eventvalue)
         {
      case UI_KP_RIGHT:
         event->key.keyval = GDK_KP_Right;
         event->key.length = 0;
         event->key.send_event = 0;

      case UI_KP_DOWN:
         event->key.keyval = GDK_KP_Down;
         event->key.length = 0;
         event->key.send_event = 0;
         }

      }
   }

void
uiwxwin_editable_inserttext(UiWidget * editable, const gchar * new_text, gint new_text_length, gint * position)
   {
   g_assert(editable);

   gtk_editable_insert_text(GTK_EDITABLE(editable), new_text, new_text_length, position);
   }

void
uiwxwin_editable_deletetext(UiWidget * editable, gint start_pos, gint end_pos)
   {
   g_assert(editable);

   gtk_editable_delete_text(GTK_EDITABLE(end_pos), start_pos, end_pos);
   }

void
uiwxwin_editable_setposition(UiWidget * editable, gint position)
   {
   g_assert(editable);

   gtk_editable_set_position(GTK_EDITABLE(editable), position);
   }

void
uiwxwin_togglebutton_setactive(UiWidget * toggle_button, gboolean is_active)
   {
   g_assert(toggle_button);

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_button), is_active);
   }

/* END */

/*/////////////////////////////////////////////////// */

void
uiwxwin_assignpos(UiWidget * widget, gint x, gint y)
{
   if (widget != NULL)
      gtk_widget_set_uposition(GTK_WIDGET(widget), x, y);

}

gint
uiwxwin_list_getselectedrownum(UiWidget * widget)
{
   GList *selection;

   selection = GTK_CLIST(widget)->selection;
   if (!selection)
      return -1;
   return GPOINTER_TO_INT(selection->data);
}

gint
uiwxwin_dropdown_numitems(UiWidget *widget)
{
   GtkList *list;
   GList *children;
   widget = uiwxwin_dropdown_getlistbox(widget);
   list = GTK_LIST(widget);
   children = list->children;
   return g_list_length(children);
}
