// Author: Francis Lee <francis@treshna.com>
// Version: 1.0 since May, 2002

// Standard ANSI/ISO C libraries */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Dynamic loading stuff...
// Francis thinks for portability reason, this should be replaced
// with GModule in GLib. As far as Francis knows, 'dlfcn.h' only
// works in UNIX system, is Francis wrong?
#include <dlfcn.h>
#include <glob.h>

// GLib stuff
#include <glib.h>

// XML stuff
#include <libxml/parser.h>

// wxWindow module
#include <wx/wx.h>
#include "uiwxwin.h"
#include "gladexml.h"

#include "debug.h"
#include "../src/uiplugin.h"
gint
plugin_addfunction(Plugin * plugin, gchar * funcname, void *(*funcptr) ());

//////////////////////////////////////////////////////////////////////
// Global variables and stuctures...
typedef struct
   {
   const gchar *name;
   GList *list;
   }
widget_prefix_data;

//GladeXML *globalxml;

char appxml[1024];

xmlDoc *xmldoc;
static int save_argc;
static char **save_argv;

// This should be relayed from uiwxwin_init to uiwxwin_main
GladeXmlNode *root;

void printChildren(wxWindow * obj);



/**
 * plugin_exportfunctions:
 * @plugin: plugin to add function information
 *
 *
 * Returns: non-zero on error
 */
static gint
plugin_exportfunctions(Plugin * plugin)
   {
   gint retval = 0;

#define ADDFUNC(a,b) \
	retval -= plugin_addfunction (plugin, a, reinterpret_cast<void* (*)()>(b))

	ADDFUNC("init", uiwxwin_init);
	ADDFUNC("main", uiwxwin_main);
	ADDFUNC("cleanup", uiwxwin_cleanup);

   return retval;
   }

/**
 * plugin_init:
 * @plugin: plugin structure passed in
 *
 * Will build a list of functions in the library and return them to parent library.
 *
 * Returns: 0 on success
 */
gint
plugin_init(Plugin * plugin)
   {
   if (!plugin || strcmp(plugin->name, "uiwxwin"))
      {
      warningmsg("Not the correct library, can not continue.");
      return -1;
      }

   printf("Loading wxWindow User Interface library...\n");

   plugin_exportfunctions(plugin);

   return 0;
   }

/**
 * uiwxwin_init:
 * @xmlfile: Glade XML which contains the UI descripition
 *
 * Create the global instance of xmlwindow, which will contain a list of all the windows.
 * This will make it nice and easy for window creating.
 *
 * Returns: non-zero on error
 */
gint
uiwxwin_init(gchar * xmlfile, gint argc, gchar * argv[])
   {
   //   gtk_init(&argc, &argv);
   //   glade_init();
   //
   //   globalxml = glade_xml_new(xmlfile, NULL);
   //
   //   if (!globalxml)
   //      {
   //      printf("Creating XML for menu failed: %s.\n", xmlfile);
   //      return -1;
   //      }
   //
   //   glade_xml_signal_autoconnect(globalxml);
   //   strcpy(appxml, xmlfile);
   //   return 0;

   xmldoc = xmlParseFile(xmlfile);

   if (!xmldoc)
      {
      warningmsg("Error parsing %s", xmlfile);
      return -1;
      }

   // Save argument counts and its value vector
   save_argc = argc;
   save_argv = argv;

   xmlNodePtr curr;

   /*
   	printf("xmldoc->children\n");
   	for (curr = xmldoc->children; curr; curr = curr->next) {
   		printf("children: %s\n", (char *)curr->name);
   	}
   	puts("----------");
    
   	printf("xmldoc->children->children\n");
   	for (curr = xmldoc->children->children; curr; curr = curr->next) {
   		printf("children: %s\n", (char *)curr->name);
   	}
   	puts("----------");
   */

   // soil is under root
   GladeXmlNode *soil = new GladeXmlNode(xmldoc->children);

   // Let's draw!!! 'root' is supposed to be a window...
   root = new BondWindow(soil->findFirstChildNode("widget"));

   return 0;
   }

void
uiwxwin_cleanup()
   {
   //   gtk_exit(0);
   // I think there are nothing to add here since destroyer will be called
   // automatically.

   }

// Francis: The following line was from app.h/wxWindow header file.
//          I don't understand why keyword 'extern' was used.
extern int wxEntry(int, char *[]);

/**
 * Francis:
 * Please note that calling wxEntry() will create the application object
 * start message-looping
 */
void
uiwxwin_main()
   {

   int exit_value;

   // Francis: I know that 'exit_value' is not used at all...
   // but this will give you other people that wxEntry returns something...
   exit_value = wxEntry(save_argc, save_argv);
   }

//////////////////////////////////////////////////////////////////////
// The fundamental framework or architecture of Bond/wxWindow application
// implementation...

/**
 * This illustrate how wxWindow will work
 */

#include <wx/wx.h>

//////////////////////////////////////////////////////////////////////
// Class header files... This better be moved to some header files
class BondApp: public wxApp
   {
public:
   virtual bool OnInit();
   };

// class BondFrame : public wxFrame
// {
// public:
// BondFrame(const wxString& title, const wxPoint& pos, const wxSize& size);
//
// void OnQuit(wxCommandEvent& event);
// void OnAbout(wxCommandEvent& event);
//
// DECLARE_EVENT_TABLE()
//
// private:
// // wxTextCtrl *m_pTextCtrl;
// // wxGridSizer *m_pGridSizer;
// };

//enum {
//ID_Quit = 1,
//ID_About,
//};
//BEGIN_EVENT_TABLE(BondFrame, wxFrame)
//EVT_MENU(ID_Quit, BondFrame::OnQuit)
//EVT_MENU(ID_About, BondFrame::OnAbout)
//END_EVENT_TABLE()

//////////////////////////////////////////////////////////////////////

IMPLEMENT_APP_NO_MAIN(BondApp)
bool BondApp::OnInit()
{

   // The following call
   root->buildWidget(NULL);

   static_cast < wxFrame * >(root->currentWidget)->Show(TRUE);
   SetTopWindow(static_cast < wxFrame * >(root->currentWidget));

   printChildren(static_cast < wxWindow * >(root->currentWidget));

   return TRUE;

}

//////////////////////////////////////////////////////////////////////
// Implementation of bond frame

enum
{
   ID_Quit = 1,
   ID_About,
};

BEGIN_EVENT_TABLE(BondFrame, wxFrame)
EVT_MENU(ID_Quit, BondFrame::OnQuit)
EVT_MENU(ID_About, BondFrame::OnAbout)
END_EVENT_TABLE()BondFrame::BondFrame(const wxString & title, const wxPoint & pos, const wxSize & size):
      wxFrame((wxFrame *) NULL, -1, title, pos, size)
   {
   }

//////////////////////////////////////////////////////////////////////
// Implementation of event handler

void
BondFrame::OnQuit(wxCommandEvent & WXUNUSED(event))
   {
   Close(TRUE);
   }

void
printChildren(wxWindow * obj)
   {
   obj->GetChildren().GetFirst();

   puts("BEGIN\n");
   for (wxWindowListNode * node = obj->GetChildren().GetFirst(); node; node = node->GetNext())
      {
      printf("%s\n", node->GetData()->GetClassInfo()->GetClassName());
      printChildren(static_cast < wxWindow * >(node->GetData()));
      }
   puts("END\n");
   }

void
BondFrame::OnAbout(wxCommandEvent & WXUNUSED(event))
   {
   wxMessageBox("This is a wxWindows Hello", "About Hello", wxOK | wxICON_INFORMATION, this);
   }

/**
 * plugin_addfunction:
* @plugin:
 * 
   * Function used by plugin libraries to export functions. 
   */
gint
plugin_addfunction(Plugin * plugin, gchar * funcname, void *(*funcptr)())
   {
   g_assert(plugin);
   plugin->func = (PluginFunction**)g_realloc(plugin->func, (sizeof(PluginFunction *) * (plugin->numfunctions + 1)));
   plugin->func[plugin->numfunctions] = (PluginFunction*)g_malloc(sizeof(PluginFunction));
   plugin->func[plugin->numfunctions]->name = (gchar*)g_strdup(funcname);
   plugin->func[plugin->numfunctions]->ptr = (void*)funcptr;
   plugin->numfunctions++;
   return 0;
   }
